/*
 * Cuttermaran - The Mpeg Cutter
 * 
 * Copyright (C) 2003-2006  Tobias Arnold (cuttermaran@gmx.net)
 * 
 * This file is part of cuttermaran, a free MPEG cut program.
 * See http://www.cuttermaran.de for updates.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

using System;
using System.IO;
using System.Data;
using System.Text;
using System.Collections;
using System.Collections.Specialized;
using System.Windows.Forms;
using System.Reflection;
using System.Threading;
using System.Configuration;
using System.Diagnostics;

using StringResources;
using Coremoran.AVStreams;
using GUI.data;
using GUI.MainForm;
using Configuration;
using Configuration.data;
using Coremoran.Common;
using Coremoran.Algorithms;

namespace GUI.Actions
{
	/// <summary>
	/// Summary description for ActionObject.
	/// </summary>
	public abstract class ActionObject : IProcessInformation
	{
		protected StateController state;
		protected string currentFile; // the file which is currently processed

		public ActionObject(StateController sc)
		{
			state=sc;
		}

		public abstract void Run();

		private Control hostingControl;
		public Control HostingControl
		{
			set
			{
				hostingControl=value;
			}
		}

		/// <summary>
		/// This is used to display the filename in ThreadControl
		/// </summary>
		public string CurrentFile
		{
			set { currentFile = value; }
		}

		// Delegate, damit der Thread den DialogResult ordentlich zurckgeben knnen,
		// sonst gibt es Syncprobleme
		public delegate void SetDialogResultDelegate(DialogResult result);
		private SetDialogResultDelegate resultDelegate;
		public SetDialogResultDelegate ResultDelegateMethod
		{
			set
			{
				resultDelegate=value;
			}
		}

		// Delegate um die Beschreibung zu setzen
		public delegate void SetActionTextDelegate(string text, string currentFile);
		private SetActionTextDelegate actionText;
		public SetActionTextDelegate ActionTextMethod
		{
			set
			{
				actionText=value;
			}
		}

		// Used for the top progressbar
		public delegate void CurrentProgressDelegate(int s32_Value, bool bShowTotalProgress);
		public CurrentProgressDelegate   CurrentProgressValue;

		// used for the bottom progressbar
		public delegate void TotalProgressMaxDelegate(int s32_MaxValue);
		public delegate void TotalProgressIncrDelegate();
		// no invoke required, only 2 variables will be set
		public TotalProgressMaxDelegate  TotalProgressMaximum;
		public TotalProgressIncrDelegate TotalProgressIncrement;


		// The dialog's handle is required to call Invoke on the form
		private void EnsureParentCreated()
		{
			// Do not use hostingControl.Created here !!
			// The dialog may be created invisible where Created is false although the handle exists!!
			while (hostingControl.Handle == IntPtr.Zero)
			{
				Thread.Sleep(50); // Sleep(0) would generate 99% CPU load
			}
		}

		protected void Return(bool result,Exception e)
		{
			if (e!=null && KeepRunning) // Canceling is not an error!
			{
				if (	e is IOException
					||	e is UnauthorizedAccessException
					||	e is ApplicationException)
				{
					string text;
					if (e is CoremoranException)
						text=((CoremoranException)e).FormatMessage(Resource.CurrentResource.Manager);
					else
						text=e.Message;
					MessageBox.Show(text,Resource.CurrentResource[Resource.ERROR]);
				}
				else
				{
					ErrorDialog ed=new ErrorDialog(e);
					ed.ShowDialog();
				}
			}

			DialogResult dr=result?DialogResult.OK:DialogResult.Cancel;
			if (!KeepRunning)
				dr=DialogResult.Abort;

			EnsureParentCreated();
			hostingControl.Invoke(resultDelegate,new object [] {dr});
		}

		#region IProcessInformation Members

		// Allows to temporary disable the progress display in total progressbar
		private bool mb_AddTotalProgress = true;
		public  bool AddTotalProgress
		{
			get { return mb_AddTotalProgress; }
			set { mb_AddTotalProgress=value; }
		}

		public decimal PercentageComplete
		{
			set { CurrentProgressValue((int)value, mb_AddTotalProgress); }
		}

		public bool KeepRunning
		{
			get { return BasicPacketStream.KeepRunning; }
		}

		public string ProgressDescription
		{
			set
			{
				EnsureParentCreated();
				hostingControl.Invoke(actionText,new object [] {value, currentFile});
			}
		}

		#endregion
	}

	/// <summary>
	/// This class allows to load a video and multiple audio files 
	/// but opens only ONE ThreadControl dialog to display the progress of all files!
	/// This class can also load audio only and video only
	/// </summary>
	public class LoadAVstreamAction : ActionObject
	{
		private string           videoFile;
		private ArrayList        audioFiles;
		private BasicVideoStream videoStream;
		private ArrayList        audioStreams;

		public LoadAVstreamAction(string    VideoFile,  // =null to load audio only
		                          ArrayList AudioFiles) // =null to load video only
			: base(null)
		{
			videoFile    = VideoFile;
			audioFiles   = AudioFiles;
			audioStreams = new ArrayList();
		}

		public override void Run()
		{
			int audioCount = (audioFiles == null) ? 0 : audioFiles.Count;
			int videoCount = (videoFile  == null) ? 0 : 1;

			TotalProgressMaximum(videoCount + audioCount);

			try // load video
			{
				if (videoCount > 0)
				{
					currentFile = videoFile;
					BasicVideoStream stream = new Mpeg12Video();
					stream.CreateIndex(currentFile, this);
					if (!KeepRunning) // check after CreateIndex()
					{
						Return(false,null);
						return; // user clicked Cancel -> abort
					}
					videoStream = stream;
					TotalProgressIncrement();
				}
			}
			catch (Exception ape)
			{
				Return(false, ape);
				return;
			}

			try // load audio files
			{
				for (int i=0; i<audioCount; i++)
				{
					currentFile = (string) audioFiles[i];

					int dummypos;
					AudioMode am = AudioType.GetAudioType(currentFile, out dummypos);
				
					BasicAudioStream audioStream = AudioType.Create(am);
					if (audioStream!=null)
					{
						audioStream.CreateIndex(currentFile,this);
						if (!KeepRunning) // check after CreateIndex()
							break; // user clicked Cancel -> do not add this file!
						
						audioStreams.Add(audioStream);
					}
					else
					{
						MessageBox.Show(Resource.CurrentResource.FormatMessage(Resource.UNKNOWN_AUDIO_FORMAT,Path.GetFileName(currentFile)),
							            Resource.CurrentResource[Resource.ERROR]);
						Return(false,null);
						return;
					}

					TotalProgressIncrement();
				}
				Return(true,null);
			}
			catch (Exception ape)
			{
				Return(false,new ApplicationException("Stream Error: "+ape.Message+ape.StackTrace,ape));
			}
		}

		/// <summary>
		/// null on error or if no video has been loaded
		/// </summary>
		public BasicVideoStream VideoStream
		{
			get { return videoStream; }
		}

		/// <summary>
		/// empty arraylist on error or if no audio has been loaded
		/// </summary>
		public ArrayList AudioStreams
		{
			get { return audioStreams; }
		}
	}

	public class LoadProjectAction : ActionObject
	{
		private StateData data;
		private Hashtable video_streams;
		private Hashtable audio_streams;
		private Stream actionStream;
		private Hashtable videoMarker;


		public LoadProjectAction(Stream contentStream) : base(null)
		{
			actionStream=contentStream;
			videoMarker=new Hashtable();
		}

		private void SetMarker(string streamName,int position)
		{
			SortedList marker=CurrentMakerList(streamName);
			if (!marker.Contains(position))
				marker.Add(position,null);
		}

		private SortedList CurrentMakerList(string streamName)
		{
			streamName=streamName.ToLower();
			if (!videoMarker.Contains(streamName))
				videoMarker[streamName]=new SortedList();
			return (SortedList)videoMarker[streamName];
		}

		public override void Run()
		{
			try
			{
				data=new StateData();
				data.ReadXml(actionStream);

				TotalProgressMaximum(data.usedVideoFiles.Count + data.usedAudioFiles.Count);
				// Videos laden
				video_streams=new Hashtable();
				foreach (StateData.usedVideoFilesRow row in data.usedVideoFiles)
				{
					currentFile = row.FileName;

					Mpeg12Video m2v=new Mpeg12Video();
					m2v.CreateIndex(row.FileName,this);
					if (!KeepRunning) // check after CreateIndex()
					{
						actionStream.Close();
						Return(false,null);
						return;
					}
					// Schnittvorschlge bernehmen
					foreach (StateData.VideoSuggestionPointRow vspr in row.GetVideoSuggestionPointRows())
						m2v.AddInterestingPoint(vspr.FrameNr,vspr.Text);

					// Marker bernehmen
					foreach (StateData.VideoMarkerRow vmr in row.GetVideoMarkerRows())
						SetMarker(row.FileName,vmr.FrameNr);

					video_streams[row.FileID]=m2v;
					TotalProgressIncrement();
				}

				// Audios laden
				audio_streams=new Hashtable();
				foreach (StateData.usedAudioFilesRow row in data.usedAudioFiles)
				{
					currentFile = row.FileName;

					int dummypos;
					AudioMode am=AudioType.GetAudioType(row.FileName,out dummypos);
					BasicAudioStream audio=AudioType.Create(am);
					if (audio==null)
						throw new ApplicationException(Resource.CurrentResource.FormatMessage(Resource.UNKNOWN_AUDIO_FORMAT,row.FileName));
					audio.CreateIndex(row.FileName,this);
					if (!KeepRunning) // check after CreateIndex()
						break;

					if (!row.IsStartDelayNull())
						audio.StartDelay=row.StartDelay;
					// Schnittvorschlge bernehmen
					foreach (StateData.AudioSuggestionPointRow aspr in row.GetAudioSuggestionPointRows())
						audio.AddInterestingPoint(aspr.Timems,aspr.Text);

					audio_streams[row.FileID]=audio;
					TotalProgressIncrement();
				}
				Return(true,null);
			}
			catch (System.Xml.XmlException)
			{
				Return(false,new ApplicationException(Resource.CurrentResource[Resource.INVALID_PROJECT_FILE]));
			}
			catch (FileNotFoundException fnfEx)
			{
				Return(false,new ApplicationException(fnfEx.Message));
			}
			catch (Exception ape)
			{
				Return(false,ape);
			}
			finally
			{
				actionStream.Close();
			}
		}

		public StateData State
		{
			get
			{
				return data;
			}
		}

		public Hashtable VideoStreams
		{
			get
			{
				return video_streams;
			}
		}

		public Hashtable AudioStreams
		{
			get
			{
				return audio_streams;
			}
		}

		public Hashtable VideoMarker
		{
			get
			{
				return videoMarker;
			}
		}

	}

	public class ProcessCutAction : ActionObject
	{
		private CutList cuts;
		private bool cutAudioOnly;
		private StringCollection videoFiles;
		private StringCollection audioFiles;
		private StringCollection chapterFiles;

		public ProcessCutAction(CutList cl,string filename,bool audioOnly,StateController sc) : base(sc)
		{
			cuts=cl;
			currentFile=filename;
			cutAudioOnly=audioOnly;
		}

		public override void Run()
		{
			try
			{
				// All cuts must have the same count of audio files
				int totalCuts = cuts.Count * (1 + ((CutPoint)cuts[0]).AudioStreams.Count);

				// If checkbox "Create DVD compliant Stream" is checked the resulting video file
				// will be completely encoded afterwards which is a time consuming process
				// It would be strange if during this process the progressbar displays 100%
				if (Settings.CurrentSettings.CreateDVDCompliantStream && Settings.CurrentSettings.EncodeMode)
					totalCuts ++;

				TotalProgressMaximum(totalCuts);

				// Optimieren
				cuts.RecalculateAudioCuts();
				cuts.Cut(currentFile,cutAudioOnly,out videoFiles,out audioFiles,out chapterFiles,this);
				Return(true,null);
			}
			catch (Exception ape)
			{
				Return(false,ape);
			}
		}

		public StringCollection VideoFiles
		{
			get
			{
				return videoFiles;
			}
		}

		public StringCollection AudioFiles
		{
			get
			{
				return audioFiles;
			}
		}

		public StringCollection ChapterFiles
		{
			get
			{
				return chapterFiles;
			}
		}

	}

	public class SearchPictureAction : ActionObject
	{
		private int resultPosition;
		private CutPoint currentCutOutFrame;
		private BasicVideoStream videoStream;

		public SearchPictureAction(StateController sc,CutPoint cutOutFrame) : base(sc)
		{
			currentCutOutFrame=cutOutFrame;
			videoStream=sc.VideoStream;
		}

		public override void Run()
		{
			try
			{
				// berschrift setzen
				ProgressDescription=Resource.CurrentResource[Resource.SCANNING_PICS];

				Mpeg12Video searchFile=state.VideoStream;
				int startPos=state.CurrentDecoder.CurrentPosition;
				// Ein Bild spter anfangen, wenn CutOut==Current
				if (0 == string.Compare(searchFile.StreamName, currentCutOutFrame.VideoName, true)
					&& currentCutOutFrame.EndFrame==startPos)
					++startPos;

				// sind wir schon am Ende??
				if (startPos>=state.VideoStream.PicturesCount)
					return;

				// Zuerst das Referenzbild als YV12 holen
				FrameInfoStruct fis;
				byte [] bild_source=null;
				VideoDecoder.GetPicture(currentCutOutFrame.VideoStream,ConversionType.YV12,currentCutOutFrame.EndFrame,out fis,out bild_source);
				// hier kommt das Vergleichsbild rein
				byte [] bild=new byte[fis.size+2*fis.chroma_size];

				resultPosition=startPos;
				decimal min_delta=decimal.MaxValue;
				using (VideoDecoder vd=VideoDecoder.Create(state.VideoStream,ConversionType.YV12))
				{
					vd.GoToFrame(startPos);

					for (int i=startPos;i<startPos+Settings.CurrentSettings.SecCutOutSearch*videoStream.FramesPerSecond && i<videoStream.PicturesCount-1;++i)
					{
						if (!KeepRunning)
							break;

						this.PercentageComplete=100m*(i-startPos)/(Settings.CurrentSettings.SecCutOutSearch*videoStream.FramesPerSecond);
						try
						{
							fis=vd.CurrentFrameInfo;
							// Beim Wechsel der Auflsung knnen wir nicht
							// mehr prfen
							if (fis.size+2*fis.chroma_size!=bild_source.Length)
								continue;
							//vd.GetCurrentFrameData(bild);
							decimal delta=PictureHelper.Compare(bild_source,fis.Y,fis.size);

							if (delta<min_delta)
							{
								min_delta=delta;
								resultPosition=i;
							}
							if (delta==0) // Bild identisch, dann abbrechen
								break;
							if (i<videoStream.PicturesCount-2)
								vd.MoveNextFrame();
						}
						catch //(Exception e)
						{
						}
					}
				}

				// in resultPosition steht jetzt das ermittelte Bild
				Return(true,null);
			}
			catch (Exception ape)
			{
				Return(false,ape);
			}
		}

		public int ResultPosition
		{
			get
			{
				return resultPosition;
			}
		}
	}

	public class MuxOutputAction : ActionObject
	{
		private StringCollection videoFiles;
		private StringCollection audioFiles;
		private StringCollection chapterFiles;
		private bool isMpeg2;

		public MuxOutputAction(StringCollection videos,StringCollection audios,StringCollection chapter,bool mpeg2,StateController sc) : base(sc)
		{
			videoFiles=videos;
			audioFiles=audios;
			chapterFiles=chapter;
			isMpeg2=mpeg2;
		}

		public override void Run()
		{
			try
			{
				// berschrift setzen
				ProgressDescription=Resource.CurrentResource[Resource.MULTIPLEX_RESULT];

				// TempPath besorgen
				string destFile=Path.Combine(Settings.CurrentSettings.TempDir,string.Format("X{0}",Guid.NewGuid()));

				// Dataset fr die Parameter
				MuxingData muxPar=new MuxingData();

				MuxingData.MuxInfoRow mir=muxPar.MuxInfo.NewMuxInfoRow();
				if (isMpeg2)
					mir.MuxType=Settings.CurrentSettings.MPEG2MuxTarget+1; // 0=MPEG-1, 1=SVCD, 2=DVD
				else
					mir.MuxType=0;
				mir.VideoFile=videoFiles[0];
				mir.OutputMPG=videoFiles[1];
				// Darf Muxing Provider Meldungen ausgeben?
				if (state.CmdUnattended==true)
					mir.Unattended=true;
				else
					mir.Unattended=!Settings.CurrentSettings.MuxPause;
				// Die aktuelle Sprache mitteilen
				mir.UICulture=System.Threading.Thread.CurrentThread.CurrentUICulture.Name;

				muxPar.MuxInfo.AddMuxInfoRow(mir);

				// Audio
				foreach (string file in audioFiles)
					if (file!=string.Empty) // Dateinamen knnen leer sein, falls ein Bereich geschnitten wurde in dem kein Audio exisitiert!
						muxPar.Audio.AddAudioRow(file,mir);

				// Kapitel
				foreach (string file in chapterFiles)
					muxPar.Chapter.AddChapterRow(file,mir);

				// Aufruf des Muxers
				int encoderResult=-1;
				if (Settings.CurrentSettings.UsedMuxerInfo!=null)
					encoderResult=GUIHelper.CallExternalTool(Settings.CurrentSettings.UsedMuxerInfo,muxPar,Path.ChangeExtension(destFile,".xml"));
				else
					MessageBox.Show(Resource.CurrentResource[Resource.NO_MUXER],Resource.CurrentResource[Resource.ERROR]);

				// temporre Dateien lschen
				string [] del=Directory.GetFiles(Path.GetDirectoryName(destFile),Path.GetFileNameWithoutExtension(destFile)+".*");
				foreach (string filename in del)
					File.Delete(filename);

				Return(encoderResult==0,null);
			}
			catch (TargetInvocationException)
			{
				Return(false,null);
			}
			catch (Exception ape)
			{
				Return(false,ape);
			}
		}
	}

	public class GeneratePreviewAction : ActionObject
	{
		private CutList cuts;
		private StringCollection videoFiles=null;
		private ArrayList audioFiles=null; // ArrayList<StringCollection>
		private bool useLeftMargin;
		private bool useRightMargin;

		public GeneratePreviewAction(StringCollection videoList,ArrayList audioList,StateController sc,bool leftMargin,bool rightMargin,CutList usedCuts) : base(sc)
		{
			cuts=usedCuts;
			videoFiles=videoList;
			audioFiles=audioList;
			useLeftMargin=leftMargin;
			useRightMargin=rightMargin;
		}

		public override void Run()
		{
			string tempFileName=string.Empty;
			try
			{
				StringCollection outVideo,outAudio,outChapter;
				tempFileName=string.Format("Y{0}.mpv",Guid.NewGuid());

				// Als erstes das Gesamtprojekt optimieren
				cuts.RecalculateAudioCuts();

				// Calculate the count of cuts 
				int totalCuts = 2 * (cuts.Count-1);
				if (useLeftMargin)  totalCuts ++;
				if (useRightMargin) totalCuts ++;
				// All cuts must have the same count of audio files
				totalCuts *= (1 + ((CutPoint)cuts[0]).AudioStreams.Count);

				TotalProgressMaximum(totalCuts);

				// Start cutting
				CutList cpl=new CutList();
				for (int index=0;index<cuts.Count;++index)
				{
					CutPoint cp=cuts[index];
					if (index!=0 || useLeftMargin)
					{
						cpl.Add(CutPoint.LimitRight(cp,Settings.CurrentSettings.SecCutPreview));
						cpl.Cut(Path.Combine(Settings.CurrentSettings.TempDir,tempFileName),false,out outVideo,out outAudio,out outChapter,this);

						foreach (string s in outVideo)
							videoFiles.Add(s);

						audioFiles.Add(outAudio);

						// the files must ALWAYS be added, otherwise they will not be deleted
						if (!KeepRunning) // check after Cut()
							break;
					}

					// nchster Name
					tempFileName=string.Format("Y{0}.mpv",Guid.NewGuid());
					cpl=new CutList();
					cpl.Add(CutPoint.LimitLeft(cp,Settings.CurrentSettings.SecCutPreview));
				}

				if (useRightMargin && KeepRunning)
				{
					cpl.Cut(Path.Combine(Settings.CurrentSettings.TempDir,tempFileName),false,out outVideo,out outAudio,out outChapter,this);

					foreach (string s in outVideo)
						videoFiles.Add(s);

					audioFiles.Add(outAudio);
				}

				Return(true,null);
			}
			catch (Exception ape)
			{
				try
				{
					// Letzte Dateien manuell lschen
					if (tempFileName!=string.Empty)
					{
						DirectoryInfo di=new DirectoryInfo(Settings.CurrentSettings.TempDir);
						FileInfo []fi=di.GetFiles(Path.ChangeExtension(tempFileName,".*"));
						foreach (FileInfo f in fi)
							f.Delete();
					}
				}
				catch
				{
				}
				Return(false,ape);
			}
		}
	}
}
