/*
Dialog to minimize Main window for Cuttermaran
Written by Elm (http://netcult.ch/elmue)
Date: 26-10-2006

----------------------------------------------------------------------------------
Using these conventions results in better readable code and less coding errors !
----------------------------------------------------------------------------------

     cName  for generic class definitions
     CName  for MFC     class definitions
     tName  for type    definitions
     eName  for enum    definitions
     kName  for struct  definitions

    e_Name  for enum variables
    E_Name  for enum constant values

    i_Name  for instances of classes
    h_Name  for handles

    T_Name  for Templates
    t_Name  for TCHAR or LPTSTR

    s_Name  for strings
   bs_Name  for BSTR
    f_Name  for function pointers
    k_Name  for contructs (struct)

    b_Name  bool,BOOL 1 Bit

   s8_Name    signed  8 Bit (char)
  s16_Name    signed 16 Bit (SHORT, WCHAR)
  s32_Name    signed 32 Bit (LONG, int)
  s64_Name    signed 64 Bit (LONGLONG)

   u8_Name  unsigned  8 Bit (BYTE)
  u16_Name  unsigned 16 bit (WORD)
  u32_Name  unsigned 32 Bit (DWORD, UINT)
  u64_Name  unsigned 64 Bit (ULONGLONG)

    d_Name  for double

  ----------------

    m_Name  for member variables of a class (e.g. ms32_Name for int member variable)
    g_Name  for global variables            (e.g. gu16_Name for global WORD)
    p_Name  for pointer                     (e.g.   ps_Name  *pointer to string)
   pp_Name  for pointer to pointer          (e.g.  ppd_Name **pointer to double)

*************************************************************************************/


using System;
using System.Threading;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;
using Coremoran.AVStreams;
using GUI.MainForm;

namespace GUI
{
	/// <summary>
	/// This class allows to open dialogs which behave as if they were modal.
	/// But they allow to minimize the owner form while the "modal" dialog is open 
	/// This is normally not possible because Windows closes a "real" modal dialog 
	/// when its owner becomes minimized!
	/// </summary>
	public class BaseDialog : GUIBaseForm
	{
		private BaseDlgOwner.WindowStateChangedHandler mi_WndStateChangedHandler;
		private StateController mi_State  = null;
		private BaseDlgOwner    mi_Owner  = null;
		private bool            mb_Closed = false;
		
		/// <summary>
		/// ATTENTCION: Do NOT use this constructor in your code !
		/// It exists only for Visual Studio, which otherwise would throw an exception
		/// when you try to edit the derived control in the dialog editor !!
		/// </summary>
		public BaseDialog()
		{
			mi_Owner = null;
		}

		public BaseDialog(BaseDlgOwner i_Owner)
		{
			mi_Owner = i_Owner;
			mi_State = CMMain.globalApplication.Controller;
			mi_WndStateChangedHandler = new BaseDlgOwner.WindowStateChangedHandler(OnOwnerWndStateChanged); 
		}
	
		protected override void OnCreateControl()
		{
			this.ControlBox    = true;
			this.MinimizeBox   = true;
			this.MaximizeBox   = false;
			this.ShowInTaskbar = false;

			base.OnCreateControl();
		}

		public new void Show()
		{
			throw new Exception("You cannot call Show()! Call ShowDialog() instead!");
		}

		// owner windows which are passed to ShowDialog must be ignored
		// The one and only valid owner is set in the constructor of the derived class
		public new DialogResult ShowDialog(IWin32Window owner)
		{
			return this.ShowDialog();
		}

		/// <summary>
		/// We imitate a modal dialog: This function does not return until the dialog closed
		/// </summary>
		public new DialogResult ShowDialog()
		{
			this.Owner = mi_Owner;

			if (mi_Owner == null)
				throw new Exception("You must set the owner of classes derived from BaseDialog!");

			mi_Owner.WindowStateChanged += mi_WndStateChangedHandler;

			// This forces the framework to create the dialog, but not yet display it
			// The handle must always exist for ActionObject.EnsureParentCreated()
			IntPtr h_Handle = this.Handle;

			// This MUST be called AFTER creating the handle and BEFORE Show()!!
			CenterToParent(); 

			// If the owner is minimized it must NOT be disabled 
			// otherwise the taskbar button would be inaccessible
			// If this dialog opens while the owner is minimized -> don't show this dialog
			if (mi_Owner.WindowState != FormWindowState.Minimized) 
				base.Show();

			// start the processing in the derived class
			OnStartDialog();

			mb_Closed = false;
			
			while (!mb_Closed) // Do the "modal loop"
			{
				Application.DoEvents();
				Thread.Sleep(50); // If Sleep is missing --> 99% CPU load !
			}

			mi_Owner.WindowStateChanged -= mi_WndStateChangedHandler;

			// After closing this dialog Windows sets the main window into background
			// We have to re-focus it here!
			if (mi_Owner.WindowState != FormWindowState.Minimized)
				mi_Owner.Activate();

			return this.DialogResult;
		}

		// called only if the dialog really shows on the screen
		protected override void OnLoad(EventArgs e)
		{
			// disable owner only if required!
			// If this dialog never opens, there is no need to disable the minimized owner
			// If the owner is minimized it must NOT be disabled 
			// otherwise the taskbar button would be inaccessible
			if (mi_Owner != null) mi_Owner.Enabled = false;

			base.OnLoad (e);
		}

		// ThreadControl calls Close() when finished
		public new void Close()
		{
			// re-enable the owner window
			// ATTENTION: This MUST be called before this dialog is destroyed!!!
			// DO not re-enable in OnClosed() instead which will not be called always!!
			mi_Owner.Enabled = true;

			// IMPORTANT: FIRST Enable owner THEN call base.Close()
			base.Close();
			mb_Closed = true; // set flag even if dialog was never visible
		}

		/// <summary>
		/// The user has minimized / normalized / maximized the main window
		/// </summary>
		protected virtual void OnOwnerWndStateChanged(FormWindowState e_State)
		{
			// While the main window is minimized it must be enabled
			// otherwise the taskbar button is not functional
			// But as the user normalizes / maximizes it, it must immediately be disabled again!
			bool b_Minimized = (e_State == FormWindowState.Minimized);
			mi_Owner.Enabled =  b_Minimized;

			CenterToParent();
			// this will bring the dialog on the screen if the owner was minimized up to now
			this.Visible = !b_Minimized;

			// If window minimized -> reduce CPU load to 40%
			// so the user can do other things while Cuttermaran is working
			BasicPacketStream.CpuLoad = b_Minimized ? 40 : 95;

			// remove the title bar's percent display when restoring a minimized window
			mi_State.SetStateFile(null); 
		}

		/// <summary>
		/// This function can be overridden.
		/// It is called when the processing in the derived class should start
		/// This can NOT be done in OnLoad() because this dialog will not appear
		/// on the sreen while its owner is minimized
		/// </summary>
		protected virtual void OnStartDialog()
		{
		}

		/// <summary>
		/// This function can be overridden.
		/// It is called when the user clicks the small "Close" button in the window's title bar
		/// </summary>
		protected virtual void OnCloseButton()
		{
		}

		/// <summary>
		/// Catch clicks on the control box of the dialog (Close and Minimize)
		/// </summary>
		protected override void WndProc(ref Message Msg)
		{
			const int WM_NCLBUTTONDOWN    = 0xA1; // Click into the dialog's title bar
			const int HTCLOSE             = 20;   // Close    Button
			const int HTMINBUTTON         = 8;    // Minimize Button

			// check this BEFORE base.WndProc()
			if (Msg.Msg == WM_NCLBUTTONDOWN)
			{
				switch ((int)Msg.WParam)
				{
						// The user clicked the small close button in the "modal" dialog
						// Do NOT close the window here! Let derived class decide what to do.
					case HTCLOSE:
						OnCloseButton();
						return;

						// The user clicked the Minimize button in the ThreadControl Window
						// --> minimize owner window
					case HTMINBUTTON:
						mi_Owner.WindowState = FormWindowState.Minimized;
						return;

					default: // Maximize etc.. --> do nothing
						return;
				}
			}
			base.WndProc(ref Msg);
		}
	}

	// ###########################################################################

	/// <summary>
	/// This is the window which owns BaseDialog
	/// When the user changes the main window's state the event is fired
	/// (Normal / Minimized / Maximized)
	/// </summary>
	public class BaseDlgOwner : System.Windows.Forms.Form
	{
		// This event is fired when the window state has changed 
		public  delegate void WindowStateChangedHandler(FormWindowState e_State);
		public  event         WindowStateChangedHandler WindowStateChanged;
		private FormWindowState me_LastWndState;

		protected override void WndProc(ref Message Msg)
		{
			const int WM_WINDOWPOSCHANGED = 0x47;
			base.WndProc(ref Msg);

			// check this AFTER base.WndProc()
			if (Msg.Msg == WM_WINDOWPOSCHANGED && me_LastWndState != WindowState)
			{
				me_LastWndState = WindowState;
				if (WindowStateChanged != null) // fire event
					WindowStateChanged(me_LastWndState);
			}
		}
	}
}
