/*
 * Cuttermaran - The Mpeg Cutter
 * 
 * Copyright (C) 2003-2006  Tobias Arnold (cuttermaran@gmx.net)
 * 
 * This file is part of cuttermaran, a free MPEG cut program.
 * See http://www.cuttermaran.de for updates.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

using System;
using System.IO;
using System.Drawing;
using System.Threading;
using System.Diagnostics;
using System.Windows.Forms;
using System.Runtime.InteropServices;

using Configuration;
using Configuration.data;
using DirectShowLib;

namespace GUI
{
	/// <summary>
	/// Summary description for DirectShowPlayer.
	/// </summary>
	public class DirectShowPlayer : IDisposable
	{
		enum GraphState
		{
			Stopped,
			Paused,
			Running,
			Exiting
		}

		#region Members

		// Current state of the graph (can change async)
		volatile private GraphState m_State = GraphState.Stopped;
		// graph builder interfaces
		private IFilterGraph2 m_graphBuilder;
		private IMediaControl m_mediaCtrl;
		private IMediaEvent m_mediaEvent;
#if DEBUG
		// Allow you to "Connect to remote graph" from GraphEdit
		private DsROTEntry m_DsRot;
#endif
		// Event that is called when a clip finishs playing
		public event DxPlayEvent StopPlay;
		public delegate void DxPlayEvent(object sender);

		// Event used by Media Event thread
		private ManualResetEvent m_mre;

		#endregion

		#region ctor

		public DirectShowPlayer(string filename,FilterMediaType fmt,Control destination)
		{
			try
			{
				// DirectShow Initialisieren
				m_graphBuilder = new FilterGraph() as IFilterGraph2;
				m_mediaEvent = m_graphBuilder as IMediaEvent;
				m_mediaCtrl = m_graphBuilder as IMediaControl;

#if DEBUG
				// Allows you to view the graph with GraphEdit File/Connect
				m_DsRot = new DsROTEntry(m_graphBuilder);
#endif
				BuildFiltergraph(filename,fmt);

				// Configure the Video Window if possible
				if (destination!=null)
				{
					IVideoWindow videoWindow = m_graphBuilder as IVideoWindow;
					ConfigureVideoWindow(videoWindow, destination);
				}

				// Get the event handle the graph will use to signal
				// when events occur
				IntPtr hEvent;
				int hr = m_mediaEvent.GetEventHandle(out hEvent);
				DsError.ThrowExceptionForHR(hr);

				// Wrap the graph event with a ManualResetEvent
				m_mre = new ManualResetEvent(false);
				m_mre.Handle = hEvent;

				// Create a new thread to wait for events
				Thread t = new Thread(new ThreadStart(this.EventWait));
				t.Name = "Media Event Thread";
				t.IsBackground=true;
				t.Start();

#if DEBUG
				// Double check to make sure we aren't releasing something
				// important.
				GC.Collect();
				GC.WaitForPendingFinalizers();
#endif
			}
			catch
			{
				Dispose();
				throw;
			}
		}

		#endregion

		#region RollUp

		#region IDisposable Members

		public void Dispose()
		{
			CloseInterfaces();
		}

		#endregion

		~DirectShowPlayer()
		{
			CloseInterfaces();
		}

		/// <summary>
		/// Shut down
		/// </summary>
		private void CloseInterfaces()
		{
			int hr;

			lock (this)
			{
				if (m_State != GraphState.Exiting)
				{
					m_State = GraphState.Exiting;

					// Release the thread (if the thread was started)
					if (m_mre != null)
					{
						m_mre.Set();
					}
				}

				if( m_mediaCtrl != null )
				{
					// Stop the graph
					hr = m_mediaCtrl.Stop();
					m_mediaCtrl = null;
				}

#if DEBUG
				if (m_DsRot != null)
				{
					m_DsRot.Dispose();
				}
#endif

				if (m_graphBuilder != null)
				{
					Marshal.ReleaseComObject(m_graphBuilder);
					m_graphBuilder = null;
				}
			}
			GC.Collect();
		}

		#endregion

		#region DirectShow Stuff

		// Configure the video window
		private void ConfigureVideoWindow(IVideoWindow videoWindow, Control hWin)
		{
			// Set the output window
			int hr = videoWindow.put_Owner( hWin.Handle );
			DsError.ThrowExceptionForHR( hr );

			// Set the window style
			hr = videoWindow.put_WindowStyle( (WindowStyle.Child | WindowStyle.ClipChildren | WindowStyle.ClipSiblings) );
			DsError.ThrowExceptionForHR( hr );

			// Make the window visible
			hr = videoWindow.put_Visible( OABool.True );
			DsError.ThrowExceptionForHR( hr );

			// Position the playing location
			Rectangle rc = hWin.ClientRectangle;
			hr = videoWindow.SetWindowPosition( 0, 0, rc.Right, rc.Bottom );
			DsError.ThrowExceptionForHR( hr );
			hWin.Resize+=new EventHandler(hWin_Resize);
		}

		#endregion

		#region Control

		// start playing
		public void Start()
		{
			// If we aren't already playing (or shutting down)
			if (m_State == GraphState.Stopped || m_State == GraphState.Paused)
			{
				int hr = m_mediaCtrl.Run();
				DsError.ThrowExceptionForHR( hr );

				m_State = GraphState.Running;
			}
		}

		public void Stop()
		{
			// Can only Stop when playing or paused
			if (m_State == GraphState.Running || m_State == GraphState.Paused)
			{
				int hr = m_mediaCtrl.Stop();
				DsError.ThrowExceptionForHR( hr );

				m_State = GraphState.Stopped;
			}
		}

		public void Pause()
		{
			// If we are playing
			if (m_State == GraphState.Running)
			{
				int hr = m_mediaCtrl.Pause();
				DsError.ThrowExceptionForHR( hr );

				m_State = GraphState.Paused;
			}
		}

		/// <summary>
		/// Position in seconds
		/// </summary>
		public double Position
		{
			get
			{
				IMediaPosition imp = m_graphBuilder as IMediaPosition;
				double pos;
				int hr = imp.get_CurrentPosition(out pos);
				DsError.ThrowExceptionForHR( hr );
				return pos;
			}
			set
			{
				IMediaPosition imp = m_graphBuilder as IMediaPosition;
				int hr = imp.put_CurrentPosition(value);
				DsError.ThrowExceptionForHR( hr );
			}
		}

		public double Duration
		{
			get
			{
				IMediaPosition imp = m_graphBuilder as IMediaPosition;
				double len;
				int hr = imp.get_Duration(out len);
				DsError.ThrowExceptionForHR( hr );
				return len;
			}
		}

		#endregion

		#region internal Event/Loop handling

		private void hWin_Resize(object sender, EventArgs e)
		{
			if (m_graphBuilder == null) // avoids crash
				return;
			IVideoWindow videoWindow = m_graphBuilder as IVideoWindow;
			Rectangle rc = ((Control)sender).ClientRectangle;
			int hr = videoWindow.SetWindowPosition( 0, 0, rc.Right, rc.Bottom );
			DsError.ThrowExceptionForHR( hr );
		}

		// Wait for events to happen.  This approach uses waiting on an event handle.
		// The nice thing about doing it this way is that you aren't in the windows message
		// loop, and don't have to worry about re-entrency or taking too long.  Plus, being
		// in a class as we are, we don't have access to the message loop.
		// Alternately, you can receive your events as windows messages.  See
		// IMediaEventEx.SetNotifyWindow.
		private void EventWait()
		{
			// Returned when GetEvent is called but there are no events
			const int E_ABORT = unchecked((int)0x80004004);

			int hr;
			int p1, p2;
			EventCode ec;

			do
			{
				// Wait for an event
				m_mre.WaitOne(-1, true);

				// Avoid contention for m_State
				lock(this)
				{
					// If we are not shutting down
					if (m_State != GraphState.Exiting)
					{
						// Read the event
						for (
							hr = m_mediaEvent.GetEvent(out ec, out p1, out p2, 0);
							hr >= 0;
							hr = m_mediaEvent.GetEvent(out ec, out p1, out p2, 0)
							)
						{
							// Write the event name to the debug window
							Debug.WriteLine(ec.ToString());

							// If the clip is finished playing
							if (ec == EventCode.Complete)
							{
								// Call Stop() to set state
								Stop();

								// Throw event
								if (StopPlay != null)
								{
									StopPlay(this);
								}
							}

							// Release any resources the message allocated
							hr = m_mediaEvent.FreeEventParams(ec, p1, p2);
							DsError.ThrowExceptionForHR(hr);
						}

						// If the error that exited the loop wasn't due to running out of events
						if (hr != E_ABORT)
						{
							DsError.ThrowExceptionForHR(hr);
						}
					}
					else
					{
						// We are shutting down
						break;
					}
				}
			} while (true);
		}

		#endregion

		#region FilterGraph Konstruktion

		[DllImport("OLE32.DLL", EntryPoint="CreateStreamOnHGlobal")]
		extern public static int CreateStreamOnHGlobal( int hGlobalMemHandle, bool
			fDeleteOnRelease, out UCOMIStream pOutStm);

		private void BuildFiltergraph(string sourceFile,FilterMediaType fmt)
		{
			int hr;
			FilterGraphData.MediaRow mr=Settings.CurrentSettings.GetFilter(fmt);

			if (mr==null) // DirectShow den Filtergraph aufbauen lassen
			{
				hr=m_graphBuilder.RenderFile(sourceFile,sourceFile);
				DsError.ThrowExceptionForHR(hr);
			}
			else
			{
				// Die Filter erzeugen:
				FilterGraphData.FILTERRow [] frs=mr.GetFILTERRows();
				IBaseFilter inputFileFilter;
				foreach (FilterGraphData.FILTERRow fr in frs)
				{
					if (new Guid(fr.clsid)==typeof(AsyncReader).GUID)
					{
						hr=m_graphBuilder.AddSourceFilter(sourceFile,fr.ID,out inputFileFilter);
						DsError.ThrowExceptionForHR(hr);
						continue; // Parameter wurde implizit gesetzt
					}

					IBaseFilter newFilter=(IBaseFilter)
						Activator.CreateInstance(Type.GetTypeFromCLSID(new Guid(fr.clsid)));

					IFileSourceFilter ifsf=newFilter as IFileSourceFilter;
					if (ifsf!=null)
					{
						FilterGraphData.PARAMRow param=FindParameter(fr.GetPARAMRows(),"src");
						if (param!=null)
						{
							hr=ifsf.Load(param.value,null);
							DsError.ThrowExceptionForHR(hr);
						}
					}

					IPersistStream ips=newFilter as IPersistStream;
					if (ips!=null)
					{
						FilterGraphData.PARAMRow param=FindParameter(fr.GetPARAMRows(),"data");
						if (param!=null)
						{
							UCOMIStream stream;
							CreateStreamOnHGlobal(0,true,out stream);
							try
							{
								int len=param.value.Length;
								byte [] content=new byte [len/2];
								for (int index=0;index<len;index+=2)
									content[index/2]=byte.Parse(param.value.Substring(index,2),System.Globalization.NumberStyles.HexNumber);
								stream.Write(content, content.Length, IntPtr.Zero); 
								// Stream auf 0 Positionieren
								IntPtr ip=new IntPtr(0);
								stream.Seek(0,(int)SeekOrigin.Begin,ip);

								hr=ips.Load(stream);
								DsError.ThrowExceptionForHR(hr);
							}
							finally
							{
								Marshal.ReleaseComObject(stream);
							}
						}
					}
					hr=m_graphBuilder.AddFilter(newFilter,fr.ID);
					DsError.ThrowExceptionForHR(hr);
				}

				// Pins verbinden
				foreach (FilterGraphData.connectRow cr in mr.GetconnectRows())
				{
					IPin from,to;
					IBaseFilter filter;

					hr=m_graphBuilder.FindFilterByName(cr.src,out filter);
					DsError.ThrowExceptionForHR(hr);
					hr=filter.FindPin(cr.srcpin,out from);
					DsError.ThrowExceptionForHR(hr);

					hr=m_graphBuilder.FindFilterByName(cr.dest,out filter);
					DsError.ThrowExceptionForHR(hr);
					hr=filter.FindPin(cr.destpin,out to);
					DsError.ThrowExceptionForHR(hr);

					hr=m_graphBuilder.Connect(from,to);
					DsError.ThrowExceptionForHR(hr);
				}
			}
		}

		private static FilterGraphData.PARAMRow FindParameter(FilterGraphData.PARAMRow [] rows,string val)
		{
			FilterGraphData.PARAMRow result=null;

			foreach (FilterGraphData.PARAMRow r in rows)
				if (r.name==val)
				{
					result=r;
					break;
				}

			return result;
		}

		#endregion
	}
}
