/*
 * Cuttermaran - The Mpeg Cutter
 * 
 * Copyright (C) 2003-2006  Tobias Arnold (cuttermaran@gmx.net)
 * 
 * This file is part of cuttermaran, a free MPEG cut program.
 * See http://www.cuttermaran.de for updates.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

using System;
using System.IO;
using System.Collections;
using System.Configuration;
using System.Windows.Forms;
using System.Drawing;
using System.Data;
using System.Reflection;
using System.Text;
using System.Diagnostics;
using System.Runtime.InteropServices;
using StringResources;
using GUI.MainForm;
using Coremoran.Common;
using Coremoran.AVStreams;
using Configuration;
using Configuration.data;

namespace GUI
{
	/// <summary>
	/// Summary description for GUIHelper.
	/// </summary>
	public class GUIHelper
	{
		static public ArrayList AudioMatchList(string path,string filename)
		{
			ArrayList matchlist=new ArrayList();
			string patterns=ConfigurationSettings.AppSettings["AudioDialog.Filter"];
			if (patterns==null || patterns==string.Empty)
				return matchlist;
			if (patterns.Split('|').Length!=2)
				return matchlist;
			patterns=patterns.Split('|')[1];

			foreach (string extension in patterns.Split(';'))
				matchlist.AddRange(Directory.GetFiles(path,"*"+filename+extension));

			return matchlist;
		}

		public static void ScaleControl(Control container,Control ctrl,decimal scaleX,decimal scaleY)
		{
			foreach (Control childCtrl in ctrl.Controls)
				ScaleControl(container,childCtrl,scaleX,scaleY);

			if (ctrl==container)
				ctrl.Font=new Font(ctrl.Font.FontFamily, ctrl.Font.Size*(float)scaleX,ctrl.Font.Style,ctrl.Font.Unit,ctrl.Font.GdiCharSet,ctrl.Font.GdiVerticalFont);
			ctrl.Location=new Point((int)(ctrl.Location.X*scaleX),(int)(ctrl.Location.Y*scaleY));
			ctrl.Size=new Size((int)(ctrl.Size.Width*scaleX),(int)(ctrl.Size.Height*scaleY));
		}

		public static void CenterDialog(Form form)
		{
			form.Location=new Point((SystemInformation.WorkingArea.Width-form.Size.Width)/2,
				(SystemInformation.WorkingArea.Height-form.Size.Height)/2);
		}

		public static FilterMediaType BasicPacketStream2FilterMediaType(BasicPacketStream bps)
		{
			FilterMediaType result=FilterMediaType.unknown;

			switch (bps.GetType().Name)
			{
				case "Mpeg12Video":
					result=((Mpeg12Video)bps).Sequence0.sequence_extension_exist?FilterMediaType.Mpeg2Video:FilterMediaType.Mpeg1Video;
					break;
				case "MpegAudio":
					result=FilterMediaType.MpegAudio;
					break;
				case "AC3Audio":
					result=FilterMediaType.AC3Audio;
					break;
				case "DTSAudio":
					result=FilterMediaType.DTSAudio;
					break;
				case "PCMAudio":
					result=FilterMediaType.PCMAudio;
					break;
			}

			return result;
		}

		static internal int CallExternalTool(ExternalToolProviderData.ConfigurationRow confRow,DataSet data,string xmlParameterFileName)
		{
			int encoderResult=-1;
			string relpath;
			if (confRow.ToolType=="encoder")
			{
				// Coremoran tries to start the encoder, but the user has disabled encoding
				if (!Settings.CurrentSettings.EncodeMode)
					throw new ApplicationException(Resource.CurrentResource[Resource.ENCODER_REQUIRED]);

				 relpath=@"Provider\encode";
			}
			else relpath=@"Provider\mux";

			if (confRow.IsAssembly)
			{
				// .NET Assembly in AppDomain laden
				AppDomain ad=AppDomain.CurrentDomain;
				ad.AppendPrivatePath(relpath);
				Assembly bridge=ad.Load(confRow.Executive);
				object connector=bridge.CreateInstance(confRow.ClassName);

				MethodInfo f_SetOwner    = connector.GetType().GetMethod("SetOwnerWindow");
				MethodInfo f_AbortAction = connector.GetType().GetMethod("AbortAction");
				MethodInfo f_GetProgress = connector.GetType().GetMethod("GetProgress");

				// Set owner window, AbortAction callback and GetProgress callback into provider
				if (f_SetOwner != null)
					f_SetOwner.Invoke(connector, new object[] {CMMain.globalApplication});
				if (f_AbortAction != null) // avoid exception
					ThreadControl.Instance.CallbackAbortAction = (ThreadControl.delProviderAbortAction) Delegate.CreateDelegate(typeof(ThreadControl.delProviderAbortAction), connector, "AbortAction", false);
				if (f_GetProgress != null) // avoid exception
					ThreadControl.Instance.CallbackGetProgress = (ThreadControl.delProviderGetProgress) Delegate.CreateDelegate(typeof(ThreadControl.delProviderGetProgress), connector, "GetProgress", false);
				
				// Display error message only once per provider
				if (f_SetOwner == null || f_AbortAction == null || f_GetProgress == null)
					Settings.DisplayOldProviderError(CMMain.globalApplication, bridge, confRow.Executive, confRow.DisplayName);

				MethodInfo encode=connector.GetType().GetMethod(confRow.ActionMethod);
				encoderResult=(int)encode.Invoke(connector,new object[] {data});
				ad.ClearPrivatePath();
			}
			else
			{
				string executableFile=Path.Combine(Path.Combine(Path.GetDirectoryName(Assembly.GetEntryAssembly().Location),relpath),confRow.Executive);
				data.WriteXml(xmlParameterFileName);

				StringBuilder parameter=new StringBuilder();

				if (!confRow.IsActionMethodNull())
					parameter.Append(confRow.ActionMethod+" ");

				parameter.Append(xmlParameterFileName);

				DataRow dr=data.Tables[0].Rows[0];
				// Start process hidden if main window is minimized
				bool b_Hidden = (bool)dr["Unattended"] ||
				                 CMMain.globalApplication.WindowState == FormWindowState.Minimized;
	
				mi_ExtProcess = StartProcessEx(executableFile, parameter.ToString(), b_Hidden);
				if (mi_ExtProcess != null)
				{
					ThreadControl.Instance.CallbackAbortAction = new ThreadControl.delProviderAbortAction(AbortExtProviderProcess);
					mi_ExtProcess.WaitForExit();
					encoderResult = mi_ExtProcess.ExitCode;
					mi_ExtProcess = null;
				}
			}
			return encoderResult;
		}

		/// <summary>
		/// Kills an external provider if the user clicks Cancel in ThreadControl
		/// </summary>
		private static Process mi_ExtProcess;
		public  static void AbortExtProviderProcess()
		{
			if (mi_ExtProcess != null)
				mi_ExtProcess.Kill();
		}

		// returns a string like "00:12:54,920  [P] 314" where "P" is the type of the picture
		// This function likes to reside in Coremoran.Common, but BasicVideoStream is unknown there
		public static string FormatFrameTimeAndType(int frame, BasicVideoStream video)
		{
			if (frame < 0)
				return "---";

			string type = "?";
			Picture pic = video.GetPicture(frame);
			if (pic != null) type = pic.PictureType;
			string space = (type == "I") ? "  " : " "; // "I" + space is as wide as "P" or "B"

			string res = Helper.FramesToTime(frame,video.FramesPerSecond).ToString("HH:mm:ss,fff");
			return res + string.Format("  [{0}]{1}{2}", type, space, frame);
		}

		// -------------- Start process hidden --------------------

		// used for Windows 98, ME
		[DllImport("ProcessStarter.dll", EntryPoint="StartProcessHiddenA", CharSet=CharSet.Ansi)]
		private static extern int StartProcessHiddenA(string s_Path, string s_CmdLine, bool b_WaitForExit, out int s32_ProcessID, out int s32_ExitCode);
		[DllImport("ProcessStarter.dll", EntryPoint="ExplainAPIErrorA", CharSet=CharSet.Ansi)]
		private static extern void ExplainAPIErrorA(int s32_ErrorCode, StringBuilder s_Buffer, int s32_BufSize);

		// used for Windows NT, 2000, XP...
		[DllImport("ProcessStarter.dll", EntryPoint="StartProcessHiddenW", CharSet=CharSet.Unicode)]
		private static extern int StartProcessHiddenW(string s_Path, string s_CmdLine, bool b_WaitForExit, out int s32_ProcessID, out int s32_ExitCode);
		[DllImport("ProcessStarter.dll", EntryPoint="ExplainAPIErrorW", CharSet=CharSet.Unicode)]
		private static extern void ExplainAPIErrorW(int s32_ErrorCode, StringBuilder s_Buffer, int s32_BufSize);

		/// <summary>
		/// This function can be used by any provider (TMPGEnc, ImagoMuxer...)
		/// Starts a process normally or hidden.
		/// b_Hidden=true: All toplevel windows which have a minimize button will be hidden.
		/// returns a Process instance or null if the process already exited
		/// </summary>
		static public Process StartProcessEx(string s_Path, string s_CmdLine, bool b_Hidden)
		{
			Debug.WriteLine(string.Format("Starting: {0}, CmdLine: {1}, Hidden {2}", s_Path, s_CmdLine, b_Hidden));

			if (!b_Hidden) // Start normally using .NET framework
				return Process.Start(s_Path, s_CmdLine);

			// Ansi for Windows 98 and ME, otherwise Unicode (NT, 2000, XP...)
			bool  b_Unicode = (Environment.OSVersion.Platform == PlatformID.Win32NT);
			int s32_ProcessID;
			int s32_Err;
			int s32_ExitCode; // not used here because b_WaitForExit = false

			if (b_Unicode) s32_Err = StartProcessHiddenW(s_Path, s_CmdLine, false, out s32_ProcessID, out s32_ExitCode);
			else           s32_Err = StartProcessHiddenA(s_Path, s_CmdLine, false, out s32_ProcessID, out s32_ExitCode);

			if (s32_Err != 0)
			{
				StringBuilder s_Error = new StringBuilder(2000);
				if (b_Unicode) ExplainAPIErrorW(s32_Err, s_Error, s_Error.Capacity);
				else           ExplainAPIErrorA(s32_Err, s_Error, s_Error.Capacity);

				throw new Exception(s_Error +"\n"+ s_Path);
			}
		
			try   { return Process.GetProcessById(s32_ProcessID); }
			catch { return null; } // may fail if process has already exited
		}
	}
}
