/*
 * Cuttermaran - The Mpeg Cutter
 * 
 * Copyright (C) 2003-2005  Tobias Arnold (cuttermaran@gmx.net)
 * 
 * This file is part of cuttermaran, a free MPEG cut program.
 * See http://www.cuttermaran.de for updates.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

using System;
using System.IO;
using System.Drawing;
using System.Windows.Forms;
using System.Collections;
using System.Collections.Specialized;
using System.Configuration;

using Coremoran.AVStreams;
using GUI.data;
using GUI.Actions;
using GUI.MainForm;
using StringResources;
using Configuration;
using Coremoran.Common;
using Coremoran.Algorithms;
 
namespace GUI
{
	// Event Delegates
	public delegate void SetDefaultsDelegate(bool b_NewProject);
	public delegate void AudioVideoOpenedDelegate(bool videoOpened, bool audioOpened, bool reOpened);
	public delegate void VideoPositionUpdatedDelegate(int newPosition);
	public delegate void CurrentCutModifiedDelegate();
	public delegate void CurrentCutOutModifiedDelegate();
	public delegate void CutPointModifiedDelegate(int selectedCutIndex);
	public delegate void CutListSelChangedDelegate(ArrayList selectedIndices);
	public delegate void SettingsChangedDelegate();
	public delegate void LanguageChangedDelegate();
	public delegate void GuiEnabledDelegate(Control sender, bool isRunning);
	public delegate void MarkerChangedDelegate();

	public enum Navigation
	{
		NextIFrame,
		NextPFrame,
		NextBFrame,
		NextCustomStep,
		PrevIFrame,
		PrevPFrame,
		PrevBFrame,
		PrevCustomStep
	}

	/// <summary>
	/// Summary description for StateController.
	/// </summary>
	public class StateController
	{
		#region Fields

		private CutList cuts=null;
		private StateData.CmdArgsDataTable commands=null;
		private bool modified=false;
		private string StateFile=string.Empty;
		private Hashtable videoMarker;
		
		#endregion

		#region .ctor

		public StateController()
		{
			InitializeApplication(true);
			CutPointModified+=new CutPointModifiedDelegate(OnCutPointModified);
		}

		#endregion

		#region Methods

		// Laden des State aus der cpf-Datei
		public bool LoadProject(Stream cpfFile)
		{
			InitializeApplication(true); // reset GUI

			LoadProjectAction lpa=new LoadProjectAction(cpfFile);
			ThreadControl tc=new ThreadControl(lpa);
			if (tc.ShowDialog()!=DialogResult.OK)
			{
				if (BasicPacketStream.KeepRunning) // Cancling is not an error!
					MessageBox.Show(Resource.CurrentResource[Resource.PROJECT_ERROR],Resource.CurrentResource[Resource.ERROR]);
				return false;
			}
			
			StateData data=lpa.State;
			Hashtable video_streams=lpa.VideoStreams;
			Hashtable audio_streams=lpa.AudioStreams;
			videoMarker=lpa.VideoMarker;

			// Commandos bertragen
			commands=data.CmdArgs;

			// Cut Commands in die Settings bertragen
			if (commands!=null && commands.Count>0)
			{
				if (!commands[0].IsmuxResultNull())
					Settings.CurrentSettings.MuxingMode=commands[0].muxResult;
				if (!commands[0].IsNTSCDropFrameNull())
					Settings.CurrentSettings.UseNTSCDropFrame=commands[0].NTSCDropFrame;
				if (!commands[0].IscreateInfoNull())
					Settings.CurrentSettings.CreateInfoFile=commands[0].createInfo;
				if (!commands[0].IsFirstSequenceNull())
					Settings.CurrentSettings.FirstSequence=commands[0].FirstSequence;
				if (!commands[0].IsSeqEndCodeNull())
					Settings.CurrentSettings.SequenceEndCode=commands[0].SeqEndCode;
				if (!commands[0].IsmakeDVDCompliantNull())
					Settings.CurrentSettings.CreateDVDCompliantStream=commands[0].makeDVDCompliant;
				if (!commands[0].IsmuxFolderNull())
				{
					ArrayList muxPath=new ArrayList();
					muxPath.Add(commands[0].muxFolder);
					Settings.CurrentSettings.MRUMuxPathEntries=muxPath;
				}
				if (!commands[0].IsRecalcBitrateNull())
					Settings.CurrentSettings.RecalcBitrate=commands[0].RecalcBitrate;
			}

			// Schnitte erzeugen
			foreach (StateData.CutElementsRow cer in data.CutElements)
			{
				CutPoint cp=new CutPoint();
				cp.VideoStream=(BasicVideoStream)video_streams[cer.refVideoFile];
				cp.StartFrame=cer.StartPosition;
				cp.EndFrame=cer.EndPosition;

				// ggf. Punkte modifizieren
				if (cp.StartFrame<0) cp.StartFrame=0;
				if (cp.StartFrame>=cp.VideoStream.PicturesCount) cp.StartFrame=cp.VideoStream.PicturesCount-1;
				if (cp.EndFrame<0) cp.EndFrame=0;
				if (cp.EndFrame>=cp.VideoStream.PicturesCount) cp.EndFrame=cp.VideoStream.PicturesCount-1;
				if (CmdSnapToCutPoints)
				{
					if (cp.VideoStream.GetPicture(cp.StartFrame).picture_coding_type!=(short)FrameType.IFrame)
						cp.StartFrame=((Mpeg12Video)cp.VideoStream).CalculateNextFramePos(cp.StartFrame,FrameType.IFrame);
					if (cp.VideoStream.GetPicture(cp.EndFrame).picture_coding_type==(short)FrameType.BFrame)
						cp.EndFrame=((Mpeg12Video)cp.VideoStream).CalculateNextFramePos(cp.EndFrame,FrameType.PFrame);
				}

				// zugehrige Audiofiles
				ArrayList cutAudios=new ArrayList();
				foreach (StateData.cutAudioFilesRow cafr in cer.GetcutAudioFilesRows())
					cutAudios.Add(audio_streams[cafr.refAudioFile]);
				cp.AudioStreams=(ArrayList)cutAudios.Clone();

				// zugehrige Kapitel
				foreach (StateData.chapterFramesRow cfr in cer.GetchapterFramesRows())
					cp.AddChapter(cfr.FrameNr);

				// Bild erzeugen
				Bitmap bmp=null;
				bool dummyNewBMPcreated;

				if (VideoDecoder.GetPicture(cp.VideoStream,ConversionType.RGB24,cer.EndPosition,ref bmp,out dummyNewBMPcreated))
					cp.CutOutFrame=bmp;
				Cuts.Add(cp);
			}

			// Analyze all cutpoints and find out the video <--> audio relations!!
			// i_Relations[VideoStream] = ArrayList[AudioStream1, AudioStream2..]
			Hashtable i_Relations = new Hashtable();
			foreach (CutPoint i_Cut in Cuts)
			{
				if (!i_Relations.ContainsKey(i_Cut.VideoStream))
					 i_Relations.Add(i_Cut.VideoStream, new ArrayList());

				ArrayList i_Audios = (ArrayList) i_Relations[i_Cut.VideoStream];

				foreach (BasicAudioStream i_Audio in i_Cut.AudioStreams)
				{
					if (!i_Audios.Contains(i_Audio))
						 i_Audios.Add(i_Audio);
				}
			}

			// Load all video files into the GUI and create a StreamInfoCtrl for each video
			// and all its audio files
			bool b_First = true;
			foreach (Mpeg12Video i_Video in i_Relations.Keys)
			{
				if (!b_First)
					CMMain.globalApplication.AddNewVideoTab();
					
				b_First = false;
				VideoStream = i_Video;
				ArrayList i_AudioList = (ArrayList) i_Relations[i_Video];
				foreach (BasicAudioStream i_Audio in i_AudioList)
				{
					AddAudioStream(i_Audio);
				}

				if (AudioVideoOpened != null) // must be called for each new VideoTab
					AudioVideoOpened(true, true, false);
			}

			if (Cuts.Count>0)
				FireCutListModifiedEvent(0,true);

			GoToVideoPositionForce(0);

			// Namen merken fr spteres Save!
			if (cpfFile is FileStream)
			{
				SetStateFile(((FileStream)cpfFile).Name);
				SetModified(false);
			}
			return true;
		}

		/// <summary>
		/// Sets this.StateFileName and sets the title text of the main window CMMain
		/// which is normally "Cuttermaran 1.69 beta - C:\Videos\MyProject.cpf*"
		/// If s_File == null, only the titlebar of CMMain is updated
		/// </summary>
		public void SetStateFile(string s_File)
		{
			if (s_File == null) // no file specified -> use the current one
				s_File = StateFile;
			else 
				StateFile = s_File;

			if (!Settings.CurrentSettings.DisplayFullPathInformations)
				s_File = Path.GetFileName(s_File);

			if (s_File.Length == 0)
				s_File = Resource.CurrentResource[Resource.NONAME];

			string s_Title = string.Format("{0} - {1}{2}", ConfigurationSettings.AppSettings["Version.Text"],
				                           s_File, (IsModified ? "*" : ""));

			CMMain.globalApplication.Text = s_Title;
		}

		/// <summary>
		/// If the main window is minimized and an Action like cutting is executed
		/// the title of CMMain is used to display the progress on Cuttermaran's 
		/// taskbar button like " 56%"
		/// </summary>
		public void SetTaskbarProgress(string s_Percent)
		{
			// display percent value only in titlebar if main window is minimized
			if (CMMain.globalApplication.WindowState == FormWindowState.Minimized)
			{
				CMMain.globalApplication.Text = " " + s_Percent;
			}
		}

		public void SaveSate(string FileName)
		{
			StateData data=new StateData();
			Hashtable video_streams=new Hashtable(new CaseInsensitiveHashCodeProvider(), new CaseInsensitiveComparer());

			// --- used Video
			int ID=0;
			video_streams[this.VideoStream.StreamName]=ID++;
			foreach(CutPoint cut in this.Cuts)
				if (!video_streams.ContainsKey(cut.VideoName))
					video_streams[cut.VideoName]=ID++;

			foreach (DictionaryEntry de in video_streams)
			{
				StateData.usedVideoFilesRow uvfr=data.usedVideoFiles.NewusedVideoFilesRow();
				uvfr.FileID=(int)de.Value;
				uvfr.FileName=(string)de.Key;
				data.usedVideoFiles.AddusedVideoFilesRow(uvfr);

				// add marker
				SortedList marker=(SortedList)videoMarker[((string)de.Key).ToLower()];
				if (marker!=null)
					foreach (int pos in marker.Keys)
						data.VideoMarker.AddVideoMarkerRow(pos,uvfr);
			}

			// --- used Audio
			// do not set ID=0 here -> so every file gets a unique number
			Hashtable audio_streams=new Hashtable();
			// Add only audio files to hashtable, which are used in any cut, 
			// ignore additional audio files: they would not be loaded when opening a CPF file
			foreach(CutPoint cut in this.Cuts)
				foreach (BasicAudioStream bas in cut.AudioStreams)
					if (!audio_streams.ContainsKey(bas)) // Bugfix
						audio_streams[bas]=ID++;

			foreach (DictionaryEntry de in audio_streams)
				data.usedAudioFiles.AddusedAudioFilesRow((int)de.Value,((BasicAudioStream)de.Key).StreamName,((BasicAudioStream)de.Key).StartDelay);

			// <CurrentFiles> XML entry is ONLY stored for downward compatibility (< version 1.69)
			// so older versions which require the CurrentFiles entry can load the CPF file
			// Since version 1.69 the CurrentFiles entry will be ignored when loading CPF files
			StateData.CurrentFilesRow cfr=data.CurrentFiles.AddCurrentFilesRow((int)video_streams[this.VideoStream.StreamName]);
			foreach(BasicAudioStream bas in this.AudioStreams)
			{
				// IMPORTANT: if there is an additional audio file loaded, 
				// which is not used in any cut --> do NOT store it !!!
				if (audio_streams.ContainsKey(bas))
					data.currentAudioFiles.AddcurrentAudioFilesRow((int)audio_streams[bas],cfr);
			}

			// add cutpoints
			foreach (CutPoint cut in this.Cuts)
			{
				StateData.CutElementsRow cer=data.CutElements.AddCutElementsRow((int)video_streams[cut.VideoName],cut.StartFrame,cut.EndFrame);
				// related audio files
				foreach (BasicAudioStream bas in cut.AudioStreams)
					data.cutAudioFiles.AddcutAudioFilesRow((int)audio_streams[bas],cer);
				// related chapter lists
				foreach (int FrameNr in cut.ChapterItems)
					data.chapterFrames.AddchapterFramesRow(FrameNr,cer);
			}

			data.WriteXml(FileName);
			SetStateFile (FileName);
			SetModified(false);
		}

		/// <summary>
		/// This is called after the user switched to another video tab
		/// </summary>
		public void SetActiveVideoTab(Control i_SelectedCtrl)
		{
			StreamInfoCtrl.SetActiveVideoTab(i_SelectedCtrl);

			if (VideoStream == null)
			{
				InitializeApplication(false); // disable GUI if no video loaded
				return;
			}

			if (AudioVideoOpened!=null)
				AudioVideoOpened(true, true, true);

			GoToVideoPositionForce(CurrentVideoPosition);
		}

		#endregion

		#region Properties

		public Mpeg12Video VideoStream
		{
			get { return (Mpeg12Video)StreamInfoCtrl.Streams.VideoStream; }
			set
			{
				StreamInfoCtrl.Streams.VideoStream = value;
				StreamInfoCtrl.Streams.AudioStreams.Clear();
				StreamInfoCtrl.Streams.CurrentDecoder= (value==null)? null: VideoDecoder.Create(value, ConversionType.RGB24);
			}
		}

		public ArrayList AudioStreams
		{
			get { return StreamInfoCtrl.Streams.AudioStreams; }
			set { StreamInfoCtrl.Streams.AudioStreams=value; }
		}

        public BasicAudioStream AudioStream(int index)
		{
			return (BasicAudioStream)AudioStreams[index];
		}

		public void AddAudioStream(BasicAudioStream audioStream)
		{
			AudioStreams.Add(audioStream);
		}

		/// <summary>
		/// returns a hashtable with the interesting stream points found
		/// in the video file and all audio files
		/// </summary>
		public Hashtable InterestingStreamPoints
		{
			get
			{
				Hashtable i_Points = new Hashtable();
				if (VideoStream != null)
				{
					foreach (DictionaryEntry dic in VideoStream.InterestingStreamPoints)
						i_Points[dic.Key] = dic.Value;
				}

				foreach (BasicAudioStream bas in AudioStreams)
				{
					foreach (DictionaryEntry dic in bas.InterestingStreamPoints)
						i_Points[dic.Key] = dic.Value;
				}
				return i_Points;
			}
		}

		public CutList Cuts
		{
			get
			{
				return cuts;
			}
			set
			{
				cuts=value;
			}
		}

		public bool IsModified
		{
			get { return modified; }
		}

		private void SetModified(bool val)
		{
			modified=val;
			SetStateFile(null);
		}

		public VideoDecoder CurrentDecoder
		{
			get { return StreamInfoCtrl.Streams.CurrentDecoder; }
		}

		public string StateFileName
		{
			get
			{
				return StateFile;
			}
		}

		#endregion

		#region Command-Line Properties

		public string CmdOutFile
		{
			get
			{
				if (commands!=null && commands.Count>0 && !commands[0].IsOutFileNull())
					return commands[0].OutFile;
				else
					return "";
			}
		}

		public bool CmdCut
		{
			get
			{
				if (commands!=null && commands.Count>0 && !commands[0].IscutNull() && CmdOutFile!="")
					return commands[0].cut;
				else
					return false;
			}
		}

		public bool CmdCutAudio
		{
			get
			{
				if (commands!=null && commands.Count>0 && !commands[0].IscutAudioNull() 
						&& CmdOutFile!="" && !CmdCut)
					return commands[0].cutAudio;
				else
					return false;
			}
		}

		public bool CmdUnattended
		{
			get
			{
				if (commands!=null && commands.Count>0 && !commands[0].IsunattendedNull() 
						&& (CmdCut || CmdCutAudio))
					return commands[0].unattended;
				else
					return false;
			}
		}

		public bool CmdCloseApp
		{
			get
			{
				if (commands!=null && commands.Count>0 && !commands[0].IscloseAppNull() 
						&& (CmdCut || CmdCutAudio))
					return commands[0].closeApp;
				else
					return false;
			}
		}

		private bool CmdSnapToCutPoints
		{
			get
			{
				if (commands!=null && commands.Count>0 && !commands[0].IssnapToCutPointsNull())
					return commands[0].snapToCutPoints;
				else
					return false;
			}
		}

		#endregion

		#region Initialisierung

		/// <summary>
		/// b_NewProject=true  used for initialization at program start and menu "New Project"
		/// b_NewProject=false used for initialization when loading a new video file
		/// </summary>
		public event SetDefaultsDelegate SetDefaults;

		/// <summary>
		/// b_NewProject=true  used for initialization at program start and menu "New Project"
		/// b_NewProject=false used for initialization when loading a new video file
		/// </summary>
		public void InitializeApplication(bool b_NewProject)
		{
			if (b_NewProject)
			{
				// remove all video tabs but the last one
				CMMain.globalApplication.RemoveAllVideoTabs();

				cuts=new CutList();
				commands=null;
				modified=false;
				SetStateFile("");
				SetModified(false);
				videoMarker=new Hashtable();
			}

			// reset all controls
			if (SetDefaults!=null)
				SetDefaults(b_NewProject);
		}

		#endregion

		#region Video-/Audiostreamhandling

		/// <summary>
		/// One event for Audio and Video avoids multiple drawing in the controls (e.g. ScrollCtrl) 
		/// It is fire after both: video and audio files have been opened!!
		/// The Interesting StreamPoints can only be painted into the Scrollbar after
		/// the video file AND all audio files have been loaded -> THEN this event is fired.
		/// </summary>
		public event AudioVideoOpenedDelegate  AudioVideoOpened;

		/// <summary>
		/// This function allows to load a video and multiple audio files 
		/// but opens only ONE ThreadControl dialog to display the progress of all files!
		/// This function can also load audio only and video only
		/// </summary>
		public DialogResult LoadAudioAndVideo(string    videoFile,  // =null to load audio only
		                                      ArrayList audioFiles) // =null to load video only
		{
			LoadAVstreamAction lavs = new LoadAVstreamAction(videoFile, audioFiles);
			ThreadControl tc = new ThreadControl(lavs);
			DialogResult res = tc.ShowDialog();

			if (lavs.VideoStream != null) // if successfully loaded video file
			{
				this.VideoStream=(Mpeg12Video)lavs.VideoStream;
				if (Settings.CurrentSettings.RecalcSourceBitrate)
					this.VideoStream.RecalcBitrate();
				CurrentCut=new CutPoint(); // der alte Schnitt ist sicher nicht mehr gltig
			}

			foreach (BasicAudioStream audioStream in lavs.AudioStreams)
			{
				// if successfully loaded audio files
				this.AddAudioStream(audioStream);
			}
			
			// Even if no file was opened at all: the event must be fired ALWAYS!
			if (AudioVideoOpened!=null)
				AudioVideoOpened(true, true, false);

			GoToVideoPositionForce(0);
			return res;
		}

		public void AdjustSelectedAudioDelay()
		{
			if (SelectedAudioIndex!=-1)
			{
				BasicAudioStream bsa=this.AudioStream(SelectedAudioIndex);
				AudioStartDelay asd=new AudioStartDelay();
				asd.AudioFileName=Path.GetFileName(bsa.StreamName);
				asd.StartDelay=bsa.StartDelay;
				if (asd.ShowDialog()==DialogResult.OK)
				{
					bsa.StartDelay=asd.StartDelay;
					FireCutListModifiedEvent(-1,true);
					if (AudioVideoOpened!=null)
						AudioVideoOpened(false, true, true);
				}
			}
		}

		public int SelectedAudioIndex
		{
			get
			{
				if (AudioStreams.Count>0)
				{
					int Idx = StreamInfoCtrl.Streams.SelectedAudioIndex;
					if (Idx < 0 || Idx >= AudioStreams.Count)
						return 0; // Ist der Index falsch, den ersten zurckgeben
					else
						return Idx;
				}
				else
					return -1;
			}
			set
			{
				StreamInfoCtrl.Streams.SelectedAudioIndex=value;
			}
		}

		public void MoveSelectedAudioUp()
		{
			if (SelectedAudioIndex>0)
			{
				BasicAudioStream p=this.AudioStream(SelectedAudioIndex);
				this.AudioStreams[SelectedAudioIndex]=this.AudioStreams[SelectedAudioIndex-1];
				this.AudioStreams[SelectedAudioIndex-1]=p;
				--SelectedAudioIndex;
				if (AudioVideoOpened!=null)
					AudioVideoOpened(false, true, true);
			}
		}

		public void MoveSelectedAudioDown()
		{
			if (SelectedAudioIndex<this.AudioStreams.Count-1)
			{
				BasicAudioStream p=this.AudioStream(SelectedAudioIndex);
				this.AudioStreams[SelectedAudioIndex]=this.AudioStreams[SelectedAudioIndex+1];
				this.AudioStreams[SelectedAudioIndex+1]=p;
				++SelectedAudioIndex;
				if (AudioVideoOpened!=null)
					AudioVideoOpened(false, true, true);
			}
		}

		public void DeleteSelectedAudio()
		{
			if (SelectedAudioIndex!=-1)
			{
				AudioStreams.RemoveAt(SelectedAudioIndex);
				if (AudioVideoOpened!=null)
					AudioVideoOpened(false, true, false);
			}
		}

		#endregion

		#region Video Positionshandling

		public int CurrentVideoPosition
		{
			get { return StreamInfoCtrl.Streams.CurrentVideoPosition; }
		}

		public event VideoPositionUpdatedDelegate VideoPositionUpdated;

		/// <summary>
		/// Forces a GoToVideoPosition. This is important if the video
		/// has changed but the position is the same (e.g. position = 0)
		/// </summary>
		private void GoToVideoPositionForce(int position)
		{
			StreamInfoCtrl.Streams.CurrentVideoPosition=-1;
			GoToVideoPosition(position);
		}

		public void GoToVideoPosition(int position)
		{
			if (VideoStream == null)
				return;
		
			position = Math.Max(0, Math.Min(position, VideoStream.PicturesCount-1));
			if (position == CurrentVideoPosition)
				return;
			
			StreamInfoCtrl.Streams.CurrentVideoPosition = position;

			if (VideoPositionUpdated!=null)
				VideoPositionUpdated(position); // fire event
		}

		public void MoveToPicture(Navigation nav)
		{
			switch(nav)
			{
				case Navigation.NextIFrame:
					GoToVideoPosition(this.VideoStream.CalculateNextFramePos(CurrentVideoPosition,FrameType.IFrame));
					break;
				case Navigation.NextPFrame:
					GoToVideoPosition(this.VideoStream.CalculateNextFramePos(CurrentVideoPosition,FrameType.PFrame));
					break;
				case Navigation.NextBFrame:
					GoToVideoPosition(this.VideoStream.CalculateNextFramePos(CurrentVideoPosition,FrameType.BFrame));
					break;
				case Navigation.PrevIFrame:
					GoToVideoPosition(this.VideoStream.CalculatePrevFramePos(CurrentVideoPosition,FrameType.IFrame));
					break;
				case Navigation.PrevPFrame:
					GoToVideoPosition(this.VideoStream.CalculatePrevFramePos(CurrentVideoPosition,FrameType.PFrame));
					break;
				case Navigation.PrevBFrame:
					GoToVideoPosition(this.VideoStream.CalculatePrevFramePos(CurrentVideoPosition,FrameType.BFrame));
					break;
				case Navigation.NextCustomStep:
					GoToVideoPosition(CurrentVideoPosition+Settings.CurrentSettings.CustomPlacementStep);
					break;
				case Navigation.PrevCustomStep:
					GoToVideoPosition(CurrentVideoPosition-Settings.CurrentSettings.CustomPlacementStep);
					break;
			}
		}

		/// <summary>
		/// This event is fired while a video is playing back
		/// </summary>
		public event VideoPositionUpdatedDelegate  VideoPlayPositionUpdated;

		public void UpdateVideoPlayPosition(int currentPosition)
		{
			if (VideoPlayPositionUpdated != null)
				VideoPlayPositionUpdated(currentPosition);
		}

		#endregion

		#region Handling der Schnittpunkte

		public event CurrentCutModifiedDelegate CurrentCutModified;

		public CutPoint CurrentCut
		{
			get { return StreamInfoCtrl.Streams.CurrentCut; }
			set 
			{ 
				StreamInfoCtrl.Streams.CurrentCut = value;
				if (CurrentCutModified!=null)
					CurrentCutModified();
			}
		}

		public void SetCurrentCutStartFrame(int frameNr)
		{
			CurrentCut.StartFrame=frameNr;
			if (CurrentCutModified!=null)
				CurrentCutModified();
		}

		public void SetCurrentCutEndFrame(int frameNr)
		{
			CurrentCut.EndFrame=frameNr;
			if (CurrentCutModified!=null)
				CurrentCutModified();
		}

		public event CurrentCutOutModifiedDelegate CurrentCutOutModified;

		public CutPoint CurrentCutOut
		{
			get { return StreamInfoCtrl.Streams.CurrentCutOut; }
			set 
			{ 
				StreamInfoCtrl.Streams.CurrentCutOut = value;
				if (CurrentCutOutModified!=null)
					CurrentCutOutModified();
			}
		}

		public void SetCurrentCutOutEndFrame(int frameNr)
		{
			CurrentCutOut.EndFrame=frameNr;
			Bitmap bitm=CurrentCutOut.CutOutFrame;
			bool newBMPcreated;
			VideoDecoder.GetPicture(CurrentCutOut.VideoStream,ConversionType.RGB24,frameNr,ref bitm,out newBMPcreated);

			if (CurrentCutOutModified!=null)
				CurrentCutOutModified();

			// Durch die CutOut Frame Korrektur kann sich auch
			// die CutList und die Chapterlist ndern
			if (CurrentCut==CurrentCutOut)
			{
				if (CurrentCutModified!=null) // Cut ist der Aktuelle
					CurrentCutModified();
			}
			else
			{
				// Position ermitteln
				int position=-1;
				for (int index=0;index<Cuts.Count;++index)
					if (Cuts[index]==CurrentCutOut)
						position=index;
				FireCutListModifiedEvent(position,true); // Cut ist aus der Liste
			}
		}

		public void AutoCreateChapters()
		{
			if (this.Cuts.Count>0)
			{
				this.Cuts.AutoCreateChapters(Settings.CurrentSettings.AutoChaptersValue);
				FireCutListModifiedEvent(-1,false); // Kapitel sind Teil der Cutlist!
			}
		}

		public void RemoveChapter(CutPoint cp,int frameNr)
		{
			cp.RemoveChapter(frameNr);
			FireCutListModifiedEvent(-1,false); // Kapitel sind Teil der Cutlist!
		}

		#endregion

		#region Schnittliste

		// Wird gefeuert, wenn sich irgendetwas an einem Schnitt in der Schnittliste gendert hat,
		// z.B. CutOutModifikation, Positionsupdate oder Kapitel etc.
		public event CutPointModifiedDelegate CutPointModified;

		private void FireCutListModifiedEvent(int selectedCutIndex,bool refresh)
		{
			if (refresh)
				this.Cuts.RecalculateAudioCuts();
			if (CutPointModified!=null) // Cut ist aus der Liste
				CutPointModified(selectedCutIndex);
		}

		public void AddToCutList(CutPoint cp)
		{
			int insertPos = cp.LastCutListPosition;
			if (insertPos < 0 || insertPos >= this.cuts.Count) // avoids crash!!
			{
				this.Cuts.Add(cp); // append
				insertPos = Cuts.Count-1;
			}
			else this.Cuts.Insert(insertPos, cp);

			cp.LastCutListPosition = -1; // position is no longer valid
			CurrentCut=new CutPoint();   // create new cut
			FireCutListModifiedEvent(insertPos,true);
		}

		public bool MoveCutUp(int selectedCutIndex)
		{
			if (selectedCutIndex>0 && selectedCutIndex<this.Cuts.Count)
			{
				CutPoint p=this.Cuts[selectedCutIndex];
				this.Cuts[selectedCutIndex]=this.Cuts[selectedCutIndex-1];
				this.Cuts[selectedCutIndex-1]=p;

				FireCutListModifiedEvent(selectedCutIndex-1,true);

				return true;
			}
			else
				return false;
		}

		public bool MoveCutDown(int selectedCutIndex)
		{
			if (selectedCutIndex>=0 && selectedCutIndex<this.Cuts.Count-1)
			{
				CutPoint p=this.Cuts[selectedCutIndex];
				this.Cuts[selectedCutIndex]=this.Cuts[selectedCutIndex+1];
				this.Cuts[selectedCutIndex+1]=p;

				FireCutListModifiedEvent(selectedCutIndex+1,true);

				return true;
			}
			else
				return false;
		}

		public bool EditCut(int selectedCutIndex)
		{
			if (selectedCutIndex>=0 && selectedCutIndex<this.Cuts.Count)
			{
				if (VideoStream.StreamName.ToLower()!=this.Cuts[selectedCutIndex].VideoStream.StreamName.ToLower())
					if (MarkerChanged!=null)
						MarkerChanged();

				VideoStream =(Mpeg12Video)this.Cuts[selectedCutIndex].VideoStream;
				AudioStreams=(ArrayList)  this.Cuts[selectedCutIndex].AudioStreams.Clone();
				if (AudioVideoOpened!=null)
					AudioVideoOpened(true, true, true);
				
				CurrentCut=this.Cuts[selectedCutIndex]; // Der aktuelle Punkt ...
				CurrentCut.LastCutListPosition=selectedCutIndex; // fr ein spteres Wiedereinfgen
				DeleteCut(selectedCutIndex); // wirft ntige Events!!!
				CurrentCutOut=CurrentCut;
				GoToVideoPositionForce(CurrentCut.StartFrame);
				return true;
			}
			else
				return false;
		}

		public void DuplicateCut(int selectedCutIndex)
		{
			if (selectedCutIndex>=0 && selectedCutIndex<this.Cuts.Count)
			{
				CutPoint newCutPoint=new CutPoint();
				CutPoint currentPoint=this.Cuts[selectedCutIndex];
				newCutPoint.VideoStream=currentPoint.VideoStream;
				newCutPoint.StartFrame=currentPoint.StartFrame;
				newCutPoint.EndFrame=currentPoint.EndFrame;
				newCutPoint.CutOutFrame=(Bitmap)currentPoint.CutOutFrame.Clone();
				newCutPoint.AudioStreams=(ArrayList)currentPoint.AudioStreams.Clone();
				newCutPoint.LastCutListPosition=selectedCutIndex+1;
				AddToCutList(newCutPoint);
			}
		}

		public bool DeleteCut(int selectedCutIndex)
		{
			if (selectedCutIndex < 0 || selectedCutIndex >= this.Cuts.Count)
				return false;
			
			this.Cuts.RemoveAt(selectedCutIndex);

			if (selectedCutIndex>0)
				CurrentCutOut=Cuts[selectedCutIndex-1];
			else
				CurrentCutOut=new CutPoint();

			FireCutListModifiedEvent(selectedCutIndex-1,true);
			return true;
		}

		/// <summary>
		/// This is fired when the selection in the cutlist has changed
		/// (Multiple selection is possible)
		/// </summary>
		public event CutListSelChangedDelegate CutListSelectionChanged;

		public void ReportCutListSelection(ArrayList selectedIndices)
		{
			if (CutListSelectionChanged != null)
				CutListSelectionChanged(selectedIndices);
		}

		/// <summary>
		/// Checks if the current video file is used in the cutlist.
		/// If so asks the user if he wants to delete all related cuts from the cutlist
		/// returns true  to proceed
		/// retruns false to abort
		/// </summary>
		public bool CheckVideoInUse()
		{
			if (VideoStream == null)
				return true;

			string s_VideoFile = VideoStream.StreamName;

			bool b_InUse = false;
			foreach (CutPoint i_Cut in this.Cuts)
			{
				if (string.Compare(i_Cut.VideoStream.StreamName, s_VideoFile, true) == 0)
				{
					b_InUse = true;
					break;
				}
			}

			if (!b_InUse)
				return true;

			string s_Msg = string.Format(Resource.CurrentResource[Resource.VIDEO_USED_IN_CUTLIST], Path.GetFileName(s_VideoFile));
			
			DialogResult Res = MessageBox.Show(s_Msg, ConfigurationSettings.AppSettings["Version.Text"],
			                                   MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button2);
			if (Res == DialogResult.Cancel)
				return false;

			// delete related cuts
			int Sel=0;
			int C=0;
			while (C<this.Cuts.Count)
			{
				CutPoint i_Cut = this.Cuts[C];
				if (string.Compare(i_Cut.VideoStream.StreamName, s_VideoFile, true) == 0)
				{
					CurrentCutOut=null;
					this.Cuts.RemoveAt(C); // do NOT call DeleteCut(C) here
					Sel = C-1;
				}
				else C++;
			}

			// This event will also set the new CutOut
			FireCutListModifiedEvent(Math.Max(0,Sel), true);
			return true;
		}

		#endregion

		#region Kapitelliste

		public void CreateChapter(int frameNr)
		{
			if (CurrentCut.StartFrame!=-1)
				CurrentCut.AddChapter(frameNr);

			// nachsehen, in welchen Cuts dieser Frame vorhanden ist und dort auch setzten
			foreach (CutPoint cp in this.Cuts)
				if (0 == string.Compare(cp.VideoName, VideoStream.StreamName, true) && cp.StartFrame<=frameNr && frameNr<=cp.EndFrame)
					cp.AddChapter(frameNr);

			FireCutListModifiedEvent(-1,false);
		}

		#endregion

		#region Enable Gui

		public event GuiEnabledDelegate GuiEnabled;

		public void EnableGui(Control sender, bool b_Enable)
		{
			if (GuiEnabled!=null)
				GuiEnabled(sender, b_Enable);
		}

		#endregion

		#region Actions

		public event SettingsChangedDelegate SettingsChanged;

		public void CallSettings(Settings.GUIPresetData currentGUIdata)
		{
			SettingsDialog settings=new SettingsDialog(currentGUIdata);
			if (settings.ShowDialog()==DialogResult.OK)
			{
				SetModified(IsModified); // Trick, damit die Pfadinfo auch in der Titlebar aktualisiert wird
				if (SettingsChanged!=null)
					SettingsChanged();
			}
		}

		public void CallQuickJump()
		{
			using (QuickJump2 qj=new QuickJump2(VideoStream,CurrentVideoPosition,Settings.CurrentSettings.SecQuickJumpStep))
			{
				if (qj.ShowDialog()==DialogResult.OK)
					GoToVideoPosition(qj.NewPosition);

				// Marker auch bernehmen wenn "Schlieen" geklickt wurde
				foreach (int marker in qj.Marker)
					SetMarker(marker);
			}
		}

		public void ProcessCut(CutList actionList,string videoDestFileName,bool shutdown,bool unattended)
		{
			if (Settings.CurrentSettings.EncodeMode && Settings.CurrentSettings.UsedEncoderInfo==null)
				MessageBox.Show(Resource.CurrentResource[Resource.NO_ENCODER],Resource.CurrentResource[Resource.ERROR]);
			else
			{
				ProcessCutAction pca=new ProcessCutAction(actionList,videoDestFileName,false,this);
				ThreadControl tc=new ThreadControl(pca);
				if (tc.ShowDialog()==DialogResult.OK)
				{
					// Mssen wir noch muxen?
					if (Settings.CurrentSettings.MuxingMode)
					{
						pca.VideoFiles.Add(Path.Combine(Settings.CurrentSettings.MuxOutputFolder,Path.GetFileNameWithoutExtension(pca.VideoFiles[0])+".mpg"));
						MuxOutputAction moa=new MuxOutputAction(pca.VideoFiles,pca.AudioFiles,pca.ChapterFiles,((Mpeg12Video)actionList[0].VideoStream).Sequence0.sequence_extension_exist,this);
						ThreadControl mux_tc=new ThreadControl(moa);
						if (mux_tc.ShowDialog()==DialogResult.OK)
						{
							if (Settings.CurrentSettings.OnMuxDeleteES)
							{
								foreach (string filename in pca.VideoFiles)
								{
									if (Path.GetExtension(filename).ToLower()!=".mpg")
									{
										File.Delete(filename);
										File.Delete(filename+".info");
									}
								}
								foreach (string filename in pca.AudioFiles)
									File.Delete(filename);
							}
							// No annoying message on success!
							// The user gets an error if something went wrong
							// If all was OK -> no disturbing message!
						}
					}
					else
					{
						// No annoying message on success!
						// The user gets an error if something went wrong
						// If all was OK -> no disturbing message!
					}
					if (shutdown)
					{
						SetModified(false); // Projekt NICHT speichern
						GUI.MainForm.CMMain.globalApplication.Close();
					}
				}
			}
		}

		public void CutAudioOnly(string CutFileNameWOExt)
		{
			ProcessCutAction pca=new ProcessCutAction(this.Cuts,CutFileNameWOExt,true,this);
			ThreadControl tc=new ThreadControl(pca);
			tc.ShowDialog();
			// No annoying message on success!
			// The user gets an error if something went wrong
			// If all was OK -> no disturbing message!
		}

		public void PreviewAll()
		{
			using (PlayBack play=new PlayBack())
			{
				CutList cl=new CutList();
				foreach (CutPoint cp in this.Cuts)
					cl.Add(cp);

				if (this.CurrentCut.StartFrame!=-1 && this.CurrentCut.EndFrame!=-1)
				{
					this.CurrentCut.AudioStreams=(ArrayList)this.AudioStreams.Clone();
					cl.Add(this.CurrentCut);
					play.addCurrentCut=true;
				}

				play.VideoFilesType=GUIHelper.BasicPacketStream2FilterMediaType(cl[0].VideoStream);
				for (int aIndex=0;aIndex<cl[0].AudioStreams.Count;++aIndex)
					play.AudioFilesTypes.Add(GUIHelper.BasicPacketStream2FilterMediaType((BasicPacketStream)cl[0].AudioStreams[aIndex]));

				GeneratePreviewAction gpa=new GeneratePreviewAction(play.VideoFiles,play.AudioFiles,this,true,true,cl);
				ThreadControl tc=new ThreadControl(gpa);
				if (tc.ShowDialog()==DialogResult.OK)
					play.ShowDialog(CMMain.globalApplication);	// los gehts

				DeletePreviewTempFiles(play);
			}
		}

		private void DeletePreviewTempFiles(PlayBack play)
		{
			// Tempfiles wieder lschen
			// Hier mu getrickst werden, da u.U. ein doppeltes Delete ausgefhrt wird.
			// Ist der Rechner schnell kann es dadurch zu einer UnauthorizedAccessException kommen kann.
			ArrayList delName=new ArrayList();
			foreach (string filename in play.VideoFiles)
				foreach (string f in Directory.GetFiles(Path.GetDirectoryName(filename),Path.GetFileNameWithoutExtension(filename)+".*"))
					if (!delName.Contains(f.ToLower()))
						delName.Add(f.ToLower());
			foreach (StringCollection audioArray in play.AudioFiles)
				foreach (string filename in audioArray)
					if (filename!=string.Empty && !delName.Contains(filename.ToLower()))
						delName.Add(filename.ToLower());

			// Jetzt wird erst gelscht!
			foreach (string filename in delName)
				File.Delete(filename);
		}

		public void PreviewSelectedCut(int selectedCutIndex)
		{
			using (PlayBack play=new PlayBack())
			{
				play.firstCutNr=selectedCutIndex;
				play.addCurrentCut=this.CurrentCut.StartFrame!=-1 && this.CurrentCut.EndFrame!=-1 && play.firstCutNr==this.Cuts.Count-1;
				play.showLeftMargin=false;
				play.showRightMargin=false;
				CutList tempCuts=new CutList();
				if (play.firstCutNr==0)
					play.showLeftMargin=true;
				if (play.firstCutNr==this.Cuts.Count-1 &&!play.addCurrentCut)
					play.showRightMargin=true;
				if (play.firstCutNr>0)
					tempCuts.Add(this.Cuts[play.firstCutNr-1]);
				tempCuts.Add(this.Cuts[play.firstCutNr]);
				if (play.firstCutNr<this.Cuts.Count-1)
					tempCuts.Add(this.Cuts[play.firstCutNr+1]);
				if (play.addCurrentCut)
				{
					// AudioStreams dazu
					this.CurrentCut.AudioStreams=(ArrayList)this.AudioStreams.Clone();
					tempCuts.Add(this.CurrentCut);
				}

				play.VideoFilesType=GUIHelper.BasicPacketStream2FilterMediaType(tempCuts[0].VideoStream);
				for (int aIndex=0;aIndex<tempCuts[0].AudioStreams.Count;++aIndex)
					play.AudioFilesTypes.Add(GUIHelper.BasicPacketStream2FilterMediaType((BasicPacketStream)tempCuts[0].AudioStreams[aIndex]));

				GeneratePreviewAction gpa=new GeneratePreviewAction(play.VideoFiles,play.AudioFiles,
					this,play.showLeftMargin,play.showRightMargin,tempCuts);
				ThreadControl tc=new ThreadControl(gpa);
				if (tc.ShowDialog()==DialogResult.OK)
					play.ShowDialog(CMMain.globalApplication); // Los gehts

				DeletePreviewTempFiles(play);
			}
		}

		#endregion

		#region Event Handler

		private void OnCutPointModified(int selectedCutIndex)
		{
			SetModified(true);
		}

		#endregion

		#region Marker

		public event MarkerChangedDelegate MarkerChanged;

		private SortedList CurrentMakerList()
		{
			if (VideoStream == null)
				return new SortedList();

			if (!videoMarker.Contains(VideoStream.StreamName.ToLower()))
				videoMarker[VideoStream.StreamName.ToLower()]=new SortedList();
			return (SortedList)videoMarker[VideoStream.StreamName.ToLower()];
		}

		public void SetMarker(int position)
		{
			SortedList marker=CurrentMakerList();
			if (!marker.Contains(position))
			{
				marker.Add(position,null);
				if (MarkerChanged!=null)
					MarkerChanged();
			}
		}

		public void RemoveMarker(int index)
		{
			SortedList marker=CurrentMakerList();
			if (index>=0 && index<marker.Count)
			{
				marker.RemoveAt(index);
				if (MarkerChanged!=null)
					MarkerChanged();
			}
		}

		public void InvokeMarker(int index)
		{
			SortedList marker=CurrentMakerList();
			if (index>=0 && index<marker.Count)
				GoToVideoPosition((int)marker.GetKey(index));
		}

		public ICollection AllMarkers
		{
			get
			{
				SortedList marker=CurrentMakerList();
				return marker.Keys;
			}
		}

		#endregion
	}
}
