/*
 * Cuttermaran - The Mpeg Cutter
 * 
 * Copyright (C) 2003-2005  Tobias Arnold (cuttermaran@gmx.net)
 * 
 * This file is part of cuttermaran, a free MPEG cut program.
 * See http://www.cuttermaran.de for updates.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

using System;
using System.IO;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using Configuration;
using Coremoran.AVStreams;
using StringResources;
using Coremoran.Common;
using Coremoran.Algorithms;

namespace GUI.MainForm
{
	/// <summary>
	/// Summary description for StreamInfoCtrl.
	/// </summary>
	public class StreamInfoCtrl : MainBaseControl
	{
		#region embedded class cStreamSet

		/// <summary>
		/// Stores all information about the current video and audio streams
		/// </summary>
		public class cStreamSet
		{
			public BasicVideoStream VideoStream;
			public VideoDecoder     CurrentDecoder;
			public ArrayList        AudioStreams;
			public CutPoint         CurrentCut;
			public CutPoint         CurrentCutOut;
			public int              CurrentVideoPosition;
			public int              SelectedAudioIndex;

			public cStreamSet()
			{
				Reset();
			}

			public void Reset()
			{
				if (VideoStream != null) 
					((Mpeg12Video)VideoStream).Dispose();

				if (CurrentDecoder != null)
					CurrentDecoder.CloseVideo();

				VideoStream    = null;
				CurrentDecoder = null;
				AudioStreams   = new ArrayList();
				CurrentCut     = new CutPoint();
				CurrentCutOut  = new CutPoint();
				CurrentVideoPosition = -1;
				SelectedAudioIndex   = -1;
			}
		}

		#endregion

		// ------ !STATIC! ------
		static private StreamInfoCtrl mi_SelectedTab = null;
		// ATTENTION: mi_StreamSet will be accessed BEFORE the first StreamInfoCtrl instance is ceated
		static private cStreamSet     mi_StreamSet   = new cStreamSet();
		
		// ----- !Instance! -----
		private        cStreamSet     mi_MyStreams   = new cStreamSet();

		#region ctor / Dispose

		public StreamInfoCtrl()
		{
			// Store the very first created video tab
			if (mi_SelectedTab == null) 
			{
				mi_SelectedTab = this;
				mi_StreamSet   = mi_MyStreams;

				// register static event handlers ONLY ONCE !!!
				State.SetDefaults          += new SetDefaultsDelegate(OnSetDefaults);
				State.AudioVideoOpened     += new AudioVideoOpenedDelegate(OnAudioVideoOpened);
				State.VideoPositionUpdated += new VideoPositionUpdatedDelegate(OnVideoPositionUpdated);
				State.SettingsChanged      += new SettingsChangedDelegate(OnSettingsChanged);
			}

			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			mi_MyStreams.Reset(); // Dispose

			if (disposing)
			{
				if (components != null)
					components.Dispose();
			}
			base.Dispose(disposing);
		}

		#endregion

		#region Component Designer generated code

		private System.Windows.Forms.Panel panelControl;
		private System.Windows.Forms.Button butOpenVideo;
		private System.Windows.Forms.Button butOpenAudio;
		private System.Windows.Forms.Panel panelVideo;
		private System.Windows.Forms.Label lblVideoFileName;
		private System.Windows.Forms.Label label8;
		private System.Windows.Forms.Label lblVideoLength;
		private System.Windows.Forms.Label label21;
		private System.Windows.Forms.Label lblResolution;
		private System.Windows.Forms.Label label10;
		private System.Windows.Forms.Label lblRatio;
		private System.Windows.Forms.Label label11;
		private System.Windows.Forms.Label lblFrameRate;
		private System.Windows.Forms.Label label12;
		private System.Windows.Forms.Label lblBitrate;
		private System.Windows.Forms.Label label13;
		private System.Windows.Forms.Label lblVBVBuffer;
		private System.Windows.Forms.Label label14;
		private System.Windows.Forms.Panel panelAudio;
		private System.Windows.Forms.ListView listViewAudioStream;
		private System.Windows.Forms.ColumnHeader columnHeader1;
		private System.Windows.Forms.ColumnHeader columnHeader2;
		private System.Windows.Forms.ColumnHeader columnHeader3;
		private System.Windows.Forms.ColumnHeader columnHeader4;
		private System.Windows.Forms.ColumnHeader columnHeader5;
		private System.Windows.Forms.ColumnHeader columnHeader6;
		private System.Windows.Forms.ColumnHeader columnHeader10;
		private System.Windows.Forms.OpenFileDialog openVideoDialog;
		private System.Windows.Forms.OpenFileDialog openAudioDialog;
		private System.Windows.Forms.ContextMenu cmAudio;
		private System.Windows.Forms.MenuItem menuAdjustDelay;
		private System.Windows.Forms.MenuItem menuItem9;
		private System.Windows.Forms.MenuItem menuRemoveAudio;
		private System.Windows.Forms.Button butMoveAudioDown;
		private System.Windows.Forms.Button butMoveAudioUp;
		private System.Windows.Forms.ToolTip stdToolTip;
		private System.Windows.Forms.Button btnCloseTab;
		private System.Windows.Forms.Button btnNewTab;
		private System.ComponentModel.IContainer components;

		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(StreamInfoCtrl));
			System.Configuration.AppSettingsReader configurationAppSettings = new System.Configuration.AppSettingsReader();
			this.panelControl = new System.Windows.Forms.Panel();
			this.btnCloseTab = new System.Windows.Forms.Button();
			this.butOpenAudio = new System.Windows.Forms.Button();
			this.butMoveAudioUp = new System.Windows.Forms.Button();
			this.butMoveAudioDown = new System.Windows.Forms.Button();
			this.btnNewTab = new System.Windows.Forms.Button();
			this.butOpenVideo = new System.Windows.Forms.Button();
			this.panelVideo = new System.Windows.Forms.Panel();
			this.lblVideoFileName = new System.Windows.Forms.Label();
			this.label8 = new System.Windows.Forms.Label();
			this.lblVideoLength = new System.Windows.Forms.Label();
			this.label21 = new System.Windows.Forms.Label();
			this.lblResolution = new System.Windows.Forms.Label();
			this.label10 = new System.Windows.Forms.Label();
			this.lblRatio = new System.Windows.Forms.Label();
			this.label11 = new System.Windows.Forms.Label();
			this.lblFrameRate = new System.Windows.Forms.Label();
			this.label12 = new System.Windows.Forms.Label();
			this.lblBitrate = new System.Windows.Forms.Label();
			this.label13 = new System.Windows.Forms.Label();
			this.lblVBVBuffer = new System.Windows.Forms.Label();
			this.label14 = new System.Windows.Forms.Label();
			this.panelAudio = new System.Windows.Forms.Panel();
			this.listViewAudioStream = new System.Windows.Forms.ListView();
			this.columnHeader1 = new System.Windows.Forms.ColumnHeader();
			this.columnHeader2 = new System.Windows.Forms.ColumnHeader();
			this.columnHeader3 = new System.Windows.Forms.ColumnHeader();
			this.columnHeader4 = new System.Windows.Forms.ColumnHeader();
			this.columnHeader5 = new System.Windows.Forms.ColumnHeader();
			this.columnHeader6 = new System.Windows.Forms.ColumnHeader();
			this.columnHeader10 = new System.Windows.Forms.ColumnHeader();
			this.openVideoDialog = new System.Windows.Forms.OpenFileDialog();
			this.openAudioDialog = new System.Windows.Forms.OpenFileDialog();
			this.cmAudio = new System.Windows.Forms.ContextMenu();
			this.menuAdjustDelay = new System.Windows.Forms.MenuItem();
			this.menuItem9 = new System.Windows.Forms.MenuItem();
			this.menuRemoveAudio = new System.Windows.Forms.MenuItem();
			this.stdToolTip = new System.Windows.Forms.ToolTip(this.components);
			this.panelControl.SuspendLayout();
			this.panelVideo.SuspendLayout();
			this.panelAudio.SuspendLayout();
			this.SuspendLayout();
			// 
			// panelControl
			// 
			this.panelControl.AccessibleDescription = resources.GetString("panelControl.AccessibleDescription");
			this.panelControl.AccessibleName = resources.GetString("panelControl.AccessibleName");
			this.panelControl.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("panelControl.Anchor")));
			this.panelControl.AutoScroll = ((bool)(resources.GetObject("panelControl.AutoScroll")));
			this.panelControl.AutoScrollMargin = ((System.Drawing.Size)(resources.GetObject("panelControl.AutoScrollMargin")));
			this.panelControl.AutoScrollMinSize = ((System.Drawing.Size)(resources.GetObject("panelControl.AutoScrollMinSize")));
			this.panelControl.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("panelControl.BackgroundImage")));
			this.panelControl.Controls.Add(this.btnCloseTab);
			this.panelControl.Controls.Add(this.butOpenAudio);
			this.panelControl.Controls.Add(this.butMoveAudioUp);
			this.panelControl.Controls.Add(this.butMoveAudioDown);
			this.panelControl.Controls.Add(this.btnNewTab);
			this.panelControl.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("panelControl.Dock")));
			this.panelControl.Enabled = ((bool)(resources.GetObject("panelControl.Enabled")));
			this.panelControl.Font = ((System.Drawing.Font)(resources.GetObject("panelControl.Font")));
			this.panelControl.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("panelControl.ImeMode")));
			this.panelControl.Location = ((System.Drawing.Point)(resources.GetObject("panelControl.Location")));
			this.panelControl.Name = "panelControl";
			this.panelControl.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("panelControl.RightToLeft")));
			this.panelControl.Size = ((System.Drawing.Size)(resources.GetObject("panelControl.Size")));
			this.panelControl.TabIndex = ((int)(resources.GetObject("panelControl.TabIndex")));
			this.panelControl.Text = resources.GetString("panelControl.Text");
			this.stdToolTip.SetToolTip(this.panelControl, resources.GetString("panelControl.ToolTip"));
			this.panelControl.Visible = ((bool)(resources.GetObject("panelControl.Visible")));
			// 
			// btnCloseTab
			// 
			this.btnCloseTab.AccessibleDescription = resources.GetString("btnCloseTab.AccessibleDescription");
			this.btnCloseTab.AccessibleName = resources.GetString("btnCloseTab.AccessibleName");
			this.btnCloseTab.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("btnCloseTab.Anchor")));
			this.btnCloseTab.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("btnCloseTab.BackgroundImage")));
			this.btnCloseTab.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("btnCloseTab.Dock")));
			this.btnCloseTab.Enabled = ((bool)(resources.GetObject("btnCloseTab.Enabled")));
			this.btnCloseTab.FlatStyle = ((System.Windows.Forms.FlatStyle)(resources.GetObject("btnCloseTab.FlatStyle")));
			this.btnCloseTab.Font = ((System.Drawing.Font)(resources.GetObject("btnCloseTab.Font")));
			this.btnCloseTab.Image = ((System.Drawing.Image)(resources.GetObject("btnCloseTab.Image")));
			this.btnCloseTab.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("btnCloseTab.ImageAlign")));
			this.btnCloseTab.ImageIndex = ((int)(resources.GetObject("btnCloseTab.ImageIndex")));
			this.btnCloseTab.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("btnCloseTab.ImeMode")));
			this.btnCloseTab.Location = ((System.Drawing.Point)(resources.GetObject("btnCloseTab.Location")));
			this.btnCloseTab.Name = "btnCloseTab";
			this.btnCloseTab.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("btnCloseTab.RightToLeft")));
			this.btnCloseTab.Size = ((System.Drawing.Size)(resources.GetObject("btnCloseTab.Size")));
			this.btnCloseTab.TabIndex = ((int)(resources.GetObject("btnCloseTab.TabIndex")));
			this.btnCloseTab.Text = resources.GetString("btnCloseTab.Text");
			this.btnCloseTab.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("btnCloseTab.TextAlign")));
			this.stdToolTip.SetToolTip(this.btnCloseTab, resources.GetString("btnCloseTab.ToolTip"));
			this.btnCloseTab.Visible = ((bool)(resources.GetObject("btnCloseTab.Visible")));
			this.btnCloseTab.Click += new System.EventHandler(this.btnCloseTab_Click);
			// 
			// butOpenAudio
			// 
			this.butOpenAudio.AccessibleDescription = resources.GetString("butOpenAudio.AccessibleDescription");
			this.butOpenAudio.AccessibleName = resources.GetString("butOpenAudio.AccessibleName");
			this.butOpenAudio.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("butOpenAudio.Anchor")));
			this.butOpenAudio.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("butOpenAudio.BackgroundImage")));
			this.butOpenAudio.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("butOpenAudio.Dock")));
			this.butOpenAudio.Enabled = ((bool)(resources.GetObject("butOpenAudio.Enabled")));
			this.butOpenAudio.FlatStyle = ((System.Windows.Forms.FlatStyle)(resources.GetObject("butOpenAudio.FlatStyle")));
			this.butOpenAudio.Font = ((System.Drawing.Font)(resources.GetObject("butOpenAudio.Font")));
			this.butOpenAudio.Image = ((System.Drawing.Image)(resources.GetObject("butOpenAudio.Image")));
			this.butOpenAudio.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("butOpenAudio.ImageAlign")));
			this.butOpenAudio.ImageIndex = ((int)(resources.GetObject("butOpenAudio.ImageIndex")));
			this.butOpenAudio.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("butOpenAudio.ImeMode")));
			this.butOpenAudio.Location = ((System.Drawing.Point)(resources.GetObject("butOpenAudio.Location")));
			this.butOpenAudio.Name = "butOpenAudio";
			this.butOpenAudio.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("butOpenAudio.RightToLeft")));
			this.butOpenAudio.Size = ((System.Drawing.Size)(resources.GetObject("butOpenAudio.Size")));
			this.butOpenAudio.TabIndex = ((int)(resources.GetObject("butOpenAudio.TabIndex")));
			this.butOpenAudio.Text = resources.GetString("butOpenAudio.Text");
			this.butOpenAudio.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("butOpenAudio.TextAlign")));
			this.stdToolTip.SetToolTip(this.butOpenAudio, resources.GetString("butOpenAudio.ToolTip"));
			this.butOpenAudio.Visible = ((bool)(resources.GetObject("butOpenAudio.Visible")));
			this.butOpenAudio.Click += new System.EventHandler(this.butOpenAudio_Click);
			// 
			// butMoveAudioUp
			// 
			this.butMoveAudioUp.AccessibleDescription = resources.GetString("butMoveAudioUp.AccessibleDescription");
			this.butMoveAudioUp.AccessibleName = resources.GetString("butMoveAudioUp.AccessibleName");
			this.butMoveAudioUp.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("butMoveAudioUp.Anchor")));
			this.butMoveAudioUp.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("butMoveAudioUp.BackgroundImage")));
			this.butMoveAudioUp.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("butMoveAudioUp.Dock")));
			this.butMoveAudioUp.Enabled = ((bool)(resources.GetObject("butMoveAudioUp.Enabled")));
			this.butMoveAudioUp.FlatStyle = ((System.Windows.Forms.FlatStyle)(resources.GetObject("butMoveAudioUp.FlatStyle")));
			this.butMoveAudioUp.Font = ((System.Drawing.Font)(resources.GetObject("butMoveAudioUp.Font")));
			this.butMoveAudioUp.Image = ((System.Drawing.Image)(resources.GetObject("butMoveAudioUp.Image")));
			this.butMoveAudioUp.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("butMoveAudioUp.ImageAlign")));
			this.butMoveAudioUp.ImageIndex = ((int)(resources.GetObject("butMoveAudioUp.ImageIndex")));
			this.butMoveAudioUp.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("butMoveAudioUp.ImeMode")));
			this.butMoveAudioUp.Location = ((System.Drawing.Point)(resources.GetObject("butMoveAudioUp.Location")));
			this.butMoveAudioUp.Name = "butMoveAudioUp";
			this.butMoveAudioUp.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("butMoveAudioUp.RightToLeft")));
			this.butMoveAudioUp.Size = ((System.Drawing.Size)(resources.GetObject("butMoveAudioUp.Size")));
			this.butMoveAudioUp.TabIndex = ((int)(resources.GetObject("butMoveAudioUp.TabIndex")));
			this.butMoveAudioUp.Text = resources.GetString("butMoveAudioUp.Text");
			this.butMoveAudioUp.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("butMoveAudioUp.TextAlign")));
			this.stdToolTip.SetToolTip(this.butMoveAudioUp, resources.GetString("butMoveAudioUp.ToolTip"));
			this.butMoveAudioUp.Visible = ((bool)(resources.GetObject("butMoveAudioUp.Visible")));
			this.butMoveAudioUp.Click += new System.EventHandler(this.butMoveAudioUp_Click);
			// 
			// butMoveAudioDown
			// 
			this.butMoveAudioDown.AccessibleDescription = resources.GetString("butMoveAudioDown.AccessibleDescription");
			this.butMoveAudioDown.AccessibleName = resources.GetString("butMoveAudioDown.AccessibleName");
			this.butMoveAudioDown.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("butMoveAudioDown.Anchor")));
			this.butMoveAudioDown.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("butMoveAudioDown.BackgroundImage")));
			this.butMoveAudioDown.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("butMoveAudioDown.Dock")));
			this.butMoveAudioDown.Enabled = ((bool)(resources.GetObject("butMoveAudioDown.Enabled")));
			this.butMoveAudioDown.FlatStyle = ((System.Windows.Forms.FlatStyle)(resources.GetObject("butMoveAudioDown.FlatStyle")));
			this.butMoveAudioDown.Font = ((System.Drawing.Font)(resources.GetObject("butMoveAudioDown.Font")));
			this.butMoveAudioDown.Image = ((System.Drawing.Image)(resources.GetObject("butMoveAudioDown.Image")));
			this.butMoveAudioDown.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("butMoveAudioDown.ImageAlign")));
			this.butMoveAudioDown.ImageIndex = ((int)(resources.GetObject("butMoveAudioDown.ImageIndex")));
			this.butMoveAudioDown.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("butMoveAudioDown.ImeMode")));
			this.butMoveAudioDown.Location = ((System.Drawing.Point)(resources.GetObject("butMoveAudioDown.Location")));
			this.butMoveAudioDown.Name = "butMoveAudioDown";
			this.butMoveAudioDown.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("butMoveAudioDown.RightToLeft")));
			this.butMoveAudioDown.Size = ((System.Drawing.Size)(resources.GetObject("butMoveAudioDown.Size")));
			this.butMoveAudioDown.TabIndex = ((int)(resources.GetObject("butMoveAudioDown.TabIndex")));
			this.butMoveAudioDown.Text = resources.GetString("butMoveAudioDown.Text");
			this.butMoveAudioDown.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("butMoveAudioDown.TextAlign")));
			this.stdToolTip.SetToolTip(this.butMoveAudioDown, resources.GetString("butMoveAudioDown.ToolTip"));
			this.butMoveAudioDown.Visible = ((bool)(resources.GetObject("butMoveAudioDown.Visible")));
			this.butMoveAudioDown.Click += new System.EventHandler(this.butMoveAudioDown_Click);
			// 
			// btnNewTab
			// 
			this.btnNewTab.AccessibleDescription = resources.GetString("btnNewTab.AccessibleDescription");
			this.btnNewTab.AccessibleName = resources.GetString("btnNewTab.AccessibleName");
			this.btnNewTab.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("btnNewTab.Anchor")));
			this.btnNewTab.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("btnNewTab.BackgroundImage")));
			this.btnNewTab.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("btnNewTab.Dock")));
			this.btnNewTab.Enabled = ((bool)(resources.GetObject("btnNewTab.Enabled")));
			this.btnNewTab.FlatStyle = ((System.Windows.Forms.FlatStyle)(resources.GetObject("btnNewTab.FlatStyle")));
			this.btnNewTab.Font = ((System.Drawing.Font)(resources.GetObject("btnNewTab.Font")));
			this.btnNewTab.Image = ((System.Drawing.Image)(resources.GetObject("btnNewTab.Image")));
			this.btnNewTab.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("btnNewTab.ImageAlign")));
			this.btnNewTab.ImageIndex = ((int)(resources.GetObject("btnNewTab.ImageIndex")));
			this.btnNewTab.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("btnNewTab.ImeMode")));
			this.btnNewTab.Location = ((System.Drawing.Point)(resources.GetObject("btnNewTab.Location")));
			this.btnNewTab.Name = "btnNewTab";
			this.btnNewTab.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("btnNewTab.RightToLeft")));
			this.btnNewTab.Size = ((System.Drawing.Size)(resources.GetObject("btnNewTab.Size")));
			this.btnNewTab.TabIndex = ((int)(resources.GetObject("btnNewTab.TabIndex")));
			this.btnNewTab.Text = resources.GetString("btnNewTab.Text");
			this.btnNewTab.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("btnNewTab.TextAlign")));
			this.stdToolTip.SetToolTip(this.btnNewTab, resources.GetString("btnNewTab.ToolTip"));
			this.btnNewTab.Visible = ((bool)(resources.GetObject("btnNewTab.Visible")));
			this.btnNewTab.Click += new System.EventHandler(this.btnNewTab_Click);
			// 
			// butOpenVideo
			// 
			this.butOpenVideo.AccessibleDescription = resources.GetString("butOpenVideo.AccessibleDescription");
			this.butOpenVideo.AccessibleName = resources.GetString("butOpenVideo.AccessibleName");
			this.butOpenVideo.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("butOpenVideo.Anchor")));
			this.butOpenVideo.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("butOpenVideo.BackgroundImage")));
			this.butOpenVideo.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("butOpenVideo.Dock")));
			this.butOpenVideo.Enabled = ((bool)(resources.GetObject("butOpenVideo.Enabled")));
			this.butOpenVideo.FlatStyle = ((System.Windows.Forms.FlatStyle)(resources.GetObject("butOpenVideo.FlatStyle")));
			this.butOpenVideo.Font = ((System.Drawing.Font)(resources.GetObject("butOpenVideo.Font")));
			this.butOpenVideo.Image = ((System.Drawing.Image)(resources.GetObject("butOpenVideo.Image")));
			this.butOpenVideo.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("butOpenVideo.ImageAlign")));
			this.butOpenVideo.ImageIndex = ((int)(resources.GetObject("butOpenVideo.ImageIndex")));
			this.butOpenVideo.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("butOpenVideo.ImeMode")));
			this.butOpenVideo.Location = ((System.Drawing.Point)(resources.GetObject("butOpenVideo.Location")));
			this.butOpenVideo.Name = "butOpenVideo";
			this.butOpenVideo.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("butOpenVideo.RightToLeft")));
			this.butOpenVideo.Size = ((System.Drawing.Size)(resources.GetObject("butOpenVideo.Size")));
			this.butOpenVideo.TabIndex = ((int)(resources.GetObject("butOpenVideo.TabIndex")));
			this.butOpenVideo.Text = resources.GetString("butOpenVideo.Text");
			this.butOpenVideo.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("butOpenVideo.TextAlign")));
			this.stdToolTip.SetToolTip(this.butOpenVideo, resources.GetString("butOpenVideo.ToolTip"));
			this.butOpenVideo.Visible = ((bool)(resources.GetObject("butOpenVideo.Visible")));
			this.butOpenVideo.Click += new System.EventHandler(this.butOpenVideo_Click);
			// 
			// panelVideo
			// 
			this.panelVideo.AccessibleDescription = resources.GetString("panelVideo.AccessibleDescription");
			this.panelVideo.AccessibleName = resources.GetString("panelVideo.AccessibleName");
			this.panelVideo.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("panelVideo.Anchor")));
			this.panelVideo.AutoScroll = ((bool)(resources.GetObject("panelVideo.AutoScroll")));
			this.panelVideo.AutoScrollMargin = ((System.Drawing.Size)(resources.GetObject("panelVideo.AutoScrollMargin")));
			this.panelVideo.AutoScrollMinSize = ((System.Drawing.Size)(resources.GetObject("panelVideo.AutoScrollMinSize")));
			this.panelVideo.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("panelVideo.BackgroundImage")));
			this.panelVideo.Controls.Add(this.lblVideoFileName);
			this.panelVideo.Controls.Add(this.label8);
			this.panelVideo.Controls.Add(this.lblVideoLength);
			this.panelVideo.Controls.Add(this.label21);
			this.panelVideo.Controls.Add(this.lblResolution);
			this.panelVideo.Controls.Add(this.label10);
			this.panelVideo.Controls.Add(this.lblRatio);
			this.panelVideo.Controls.Add(this.label11);
			this.panelVideo.Controls.Add(this.lblFrameRate);
			this.panelVideo.Controls.Add(this.label12);
			this.panelVideo.Controls.Add(this.lblBitrate);
			this.panelVideo.Controls.Add(this.label13);
			this.panelVideo.Controls.Add(this.lblVBVBuffer);
			this.panelVideo.Controls.Add(this.label14);
			this.panelVideo.Controls.Add(this.butOpenVideo);
			this.panelVideo.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("panelVideo.Dock")));
			this.panelVideo.Enabled = ((bool)(resources.GetObject("panelVideo.Enabled")));
			this.panelVideo.Font = ((System.Drawing.Font)(resources.GetObject("panelVideo.Font")));
			this.panelVideo.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("panelVideo.ImeMode")));
			this.panelVideo.Location = ((System.Drawing.Point)(resources.GetObject("panelVideo.Location")));
			this.panelVideo.Name = "panelVideo";
			this.panelVideo.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("panelVideo.RightToLeft")));
			this.panelVideo.Size = ((System.Drawing.Size)(resources.GetObject("panelVideo.Size")));
			this.panelVideo.TabIndex = ((int)(resources.GetObject("panelVideo.TabIndex")));
			this.panelVideo.Text = resources.GetString("panelVideo.Text");
			this.stdToolTip.SetToolTip(this.panelVideo, resources.GetString("panelVideo.ToolTip"));
			this.panelVideo.Visible = ((bool)(resources.GetObject("panelVideo.Visible")));
			// 
			// lblVideoFileName
			// 
			this.lblVideoFileName.AccessibleDescription = resources.GetString("lblVideoFileName.AccessibleDescription");
			this.lblVideoFileName.AccessibleName = resources.GetString("lblVideoFileName.AccessibleName");
			this.lblVideoFileName.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("lblVideoFileName.Anchor")));
			this.lblVideoFileName.AutoSize = ((bool)(resources.GetObject("lblVideoFileName.AutoSize")));
			this.lblVideoFileName.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("lblVideoFileName.Dock")));
			this.lblVideoFileName.Enabled = ((bool)(resources.GetObject("lblVideoFileName.Enabled")));
			this.lblVideoFileName.Font = ((System.Drawing.Font)(resources.GetObject("lblVideoFileName.Font")));
			this.lblVideoFileName.Image = ((System.Drawing.Image)(resources.GetObject("lblVideoFileName.Image")));
			this.lblVideoFileName.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("lblVideoFileName.ImageAlign")));
			this.lblVideoFileName.ImageIndex = ((int)(resources.GetObject("lblVideoFileName.ImageIndex")));
			this.lblVideoFileName.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("lblVideoFileName.ImeMode")));
			this.lblVideoFileName.Location = ((System.Drawing.Point)(resources.GetObject("lblVideoFileName.Location")));
			this.lblVideoFileName.Name = "lblVideoFileName";
			this.lblVideoFileName.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("lblVideoFileName.RightToLeft")));
			this.lblVideoFileName.Size = ((System.Drawing.Size)(resources.GetObject("lblVideoFileName.Size")));
			this.lblVideoFileName.TabIndex = ((int)(resources.GetObject("lblVideoFileName.TabIndex")));
			this.lblVideoFileName.Text = resources.GetString("lblVideoFileName.Text");
			this.lblVideoFileName.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("lblVideoFileName.TextAlign")));
			this.stdToolTip.SetToolTip(this.lblVideoFileName, resources.GetString("lblVideoFileName.ToolTip"));
			this.lblVideoFileName.Visible = ((bool)(resources.GetObject("lblVideoFileName.Visible")));
			// 
			// label8
			// 
			this.label8.AccessibleDescription = resources.GetString("label8.AccessibleDescription");
			this.label8.AccessibleName = resources.GetString("label8.AccessibleName");
			this.label8.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("label8.Anchor")));
			this.label8.AutoSize = ((bool)(resources.GetObject("label8.AutoSize")));
			this.label8.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("label8.Dock")));
			this.label8.Enabled = ((bool)(resources.GetObject("label8.Enabled")));
			this.label8.Font = ((System.Drawing.Font)(resources.GetObject("label8.Font")));
			this.label8.Image = ((System.Drawing.Image)(resources.GetObject("label8.Image")));
			this.label8.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("label8.ImageAlign")));
			this.label8.ImageIndex = ((int)(resources.GetObject("label8.ImageIndex")));
			this.label8.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("label8.ImeMode")));
			this.label8.Location = ((System.Drawing.Point)(resources.GetObject("label8.Location")));
			this.label8.Name = "label8";
			this.label8.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("label8.RightToLeft")));
			this.label8.Size = ((System.Drawing.Size)(resources.GetObject("label8.Size")));
			this.label8.TabIndex = ((int)(resources.GetObject("label8.TabIndex")));
			this.label8.Text = resources.GetString("label8.Text");
			this.label8.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("label8.TextAlign")));
			this.stdToolTip.SetToolTip(this.label8, resources.GetString("label8.ToolTip"));
			this.label8.Visible = ((bool)(resources.GetObject("label8.Visible")));
			// 
			// lblVideoLength
			// 
			this.lblVideoLength.AccessibleDescription = resources.GetString("lblVideoLength.AccessibleDescription");
			this.lblVideoLength.AccessibleName = resources.GetString("lblVideoLength.AccessibleName");
			this.lblVideoLength.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("lblVideoLength.Anchor")));
			this.lblVideoLength.AutoSize = ((bool)(resources.GetObject("lblVideoLength.AutoSize")));
			this.lblVideoLength.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("lblVideoLength.Dock")));
			this.lblVideoLength.Enabled = ((bool)(resources.GetObject("lblVideoLength.Enabled")));
			this.lblVideoLength.Font = ((System.Drawing.Font)(resources.GetObject("lblVideoLength.Font")));
			this.lblVideoLength.Image = ((System.Drawing.Image)(resources.GetObject("lblVideoLength.Image")));
			this.lblVideoLength.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("lblVideoLength.ImageAlign")));
			this.lblVideoLength.ImageIndex = ((int)(resources.GetObject("lblVideoLength.ImageIndex")));
			this.lblVideoLength.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("lblVideoLength.ImeMode")));
			this.lblVideoLength.Location = ((System.Drawing.Point)(resources.GetObject("lblVideoLength.Location")));
			this.lblVideoLength.Name = "lblVideoLength";
			this.lblVideoLength.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("lblVideoLength.RightToLeft")));
			this.lblVideoLength.Size = ((System.Drawing.Size)(resources.GetObject("lblVideoLength.Size")));
			this.lblVideoLength.TabIndex = ((int)(resources.GetObject("lblVideoLength.TabIndex")));
			this.lblVideoLength.Text = resources.GetString("lblVideoLength.Text");
			this.lblVideoLength.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("lblVideoLength.TextAlign")));
			this.stdToolTip.SetToolTip(this.lblVideoLength, resources.GetString("lblVideoLength.ToolTip"));
			this.lblVideoLength.Visible = ((bool)(resources.GetObject("lblVideoLength.Visible")));
			// 
			// label21
			// 
			this.label21.AccessibleDescription = resources.GetString("label21.AccessibleDescription");
			this.label21.AccessibleName = resources.GetString("label21.AccessibleName");
			this.label21.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("label21.Anchor")));
			this.label21.AutoSize = ((bool)(resources.GetObject("label21.AutoSize")));
			this.label21.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("label21.Dock")));
			this.label21.Enabled = ((bool)(resources.GetObject("label21.Enabled")));
			this.label21.Font = ((System.Drawing.Font)(resources.GetObject("label21.Font")));
			this.label21.Image = ((System.Drawing.Image)(resources.GetObject("label21.Image")));
			this.label21.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("label21.ImageAlign")));
			this.label21.ImageIndex = ((int)(resources.GetObject("label21.ImageIndex")));
			this.label21.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("label21.ImeMode")));
			this.label21.Location = ((System.Drawing.Point)(resources.GetObject("label21.Location")));
			this.label21.Name = "label21";
			this.label21.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("label21.RightToLeft")));
			this.label21.Size = ((System.Drawing.Size)(resources.GetObject("label21.Size")));
			this.label21.TabIndex = ((int)(resources.GetObject("label21.TabIndex")));
			this.label21.Text = resources.GetString("label21.Text");
			this.label21.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("label21.TextAlign")));
			this.stdToolTip.SetToolTip(this.label21, resources.GetString("label21.ToolTip"));
			this.label21.Visible = ((bool)(resources.GetObject("label21.Visible")));
			// 
			// lblResolution
			// 
			this.lblResolution.AccessibleDescription = resources.GetString("lblResolution.AccessibleDescription");
			this.lblResolution.AccessibleName = resources.GetString("lblResolution.AccessibleName");
			this.lblResolution.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("lblResolution.Anchor")));
			this.lblResolution.AutoSize = ((bool)(resources.GetObject("lblResolution.AutoSize")));
			this.lblResolution.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("lblResolution.Dock")));
			this.lblResolution.Enabled = ((bool)(resources.GetObject("lblResolution.Enabled")));
			this.lblResolution.Font = ((System.Drawing.Font)(resources.GetObject("lblResolution.Font")));
			this.lblResolution.Image = ((System.Drawing.Image)(resources.GetObject("lblResolution.Image")));
			this.lblResolution.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("lblResolution.ImageAlign")));
			this.lblResolution.ImageIndex = ((int)(resources.GetObject("lblResolution.ImageIndex")));
			this.lblResolution.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("lblResolution.ImeMode")));
			this.lblResolution.Location = ((System.Drawing.Point)(resources.GetObject("lblResolution.Location")));
			this.lblResolution.Name = "lblResolution";
			this.lblResolution.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("lblResolution.RightToLeft")));
			this.lblResolution.Size = ((System.Drawing.Size)(resources.GetObject("lblResolution.Size")));
			this.lblResolution.TabIndex = ((int)(resources.GetObject("lblResolution.TabIndex")));
			this.lblResolution.Text = resources.GetString("lblResolution.Text");
			this.lblResolution.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("lblResolution.TextAlign")));
			this.stdToolTip.SetToolTip(this.lblResolution, resources.GetString("lblResolution.ToolTip"));
			this.lblResolution.Visible = ((bool)(resources.GetObject("lblResolution.Visible")));
			// 
			// label10
			// 
			this.label10.AccessibleDescription = resources.GetString("label10.AccessibleDescription");
			this.label10.AccessibleName = resources.GetString("label10.AccessibleName");
			this.label10.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("label10.Anchor")));
			this.label10.AutoSize = ((bool)(resources.GetObject("label10.AutoSize")));
			this.label10.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("label10.Dock")));
			this.label10.Enabled = ((bool)(resources.GetObject("label10.Enabled")));
			this.label10.Font = ((System.Drawing.Font)(resources.GetObject("label10.Font")));
			this.label10.Image = ((System.Drawing.Image)(resources.GetObject("label10.Image")));
			this.label10.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("label10.ImageAlign")));
			this.label10.ImageIndex = ((int)(resources.GetObject("label10.ImageIndex")));
			this.label10.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("label10.ImeMode")));
			this.label10.Location = ((System.Drawing.Point)(resources.GetObject("label10.Location")));
			this.label10.Name = "label10";
			this.label10.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("label10.RightToLeft")));
			this.label10.Size = ((System.Drawing.Size)(resources.GetObject("label10.Size")));
			this.label10.TabIndex = ((int)(resources.GetObject("label10.TabIndex")));
			this.label10.Text = resources.GetString("label10.Text");
			this.label10.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("label10.TextAlign")));
			this.stdToolTip.SetToolTip(this.label10, resources.GetString("label10.ToolTip"));
			this.label10.Visible = ((bool)(resources.GetObject("label10.Visible")));
			// 
			// lblRatio
			// 
			this.lblRatio.AccessibleDescription = resources.GetString("lblRatio.AccessibleDescription");
			this.lblRatio.AccessibleName = resources.GetString("lblRatio.AccessibleName");
			this.lblRatio.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("lblRatio.Anchor")));
			this.lblRatio.AutoSize = ((bool)(resources.GetObject("lblRatio.AutoSize")));
			this.lblRatio.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("lblRatio.Dock")));
			this.lblRatio.Enabled = ((bool)(resources.GetObject("lblRatio.Enabled")));
			this.lblRatio.Font = ((System.Drawing.Font)(resources.GetObject("lblRatio.Font")));
			this.lblRatio.Image = ((System.Drawing.Image)(resources.GetObject("lblRatio.Image")));
			this.lblRatio.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("lblRatio.ImageAlign")));
			this.lblRatio.ImageIndex = ((int)(resources.GetObject("lblRatio.ImageIndex")));
			this.lblRatio.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("lblRatio.ImeMode")));
			this.lblRatio.Location = ((System.Drawing.Point)(resources.GetObject("lblRatio.Location")));
			this.lblRatio.Name = "lblRatio";
			this.lblRatio.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("lblRatio.RightToLeft")));
			this.lblRatio.Size = ((System.Drawing.Size)(resources.GetObject("lblRatio.Size")));
			this.lblRatio.TabIndex = ((int)(resources.GetObject("lblRatio.TabIndex")));
			this.lblRatio.Text = resources.GetString("lblRatio.Text");
			this.lblRatio.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("lblRatio.TextAlign")));
			this.stdToolTip.SetToolTip(this.lblRatio, resources.GetString("lblRatio.ToolTip"));
			this.lblRatio.Visible = ((bool)(resources.GetObject("lblRatio.Visible")));
			// 
			// label11
			// 
			this.label11.AccessibleDescription = resources.GetString("label11.AccessibleDescription");
			this.label11.AccessibleName = resources.GetString("label11.AccessibleName");
			this.label11.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("label11.Anchor")));
			this.label11.AutoSize = ((bool)(resources.GetObject("label11.AutoSize")));
			this.label11.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("label11.Dock")));
			this.label11.Enabled = ((bool)(resources.GetObject("label11.Enabled")));
			this.label11.Font = ((System.Drawing.Font)(resources.GetObject("label11.Font")));
			this.label11.Image = ((System.Drawing.Image)(resources.GetObject("label11.Image")));
			this.label11.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("label11.ImageAlign")));
			this.label11.ImageIndex = ((int)(resources.GetObject("label11.ImageIndex")));
			this.label11.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("label11.ImeMode")));
			this.label11.Location = ((System.Drawing.Point)(resources.GetObject("label11.Location")));
			this.label11.Name = "label11";
			this.label11.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("label11.RightToLeft")));
			this.label11.Size = ((System.Drawing.Size)(resources.GetObject("label11.Size")));
			this.label11.TabIndex = ((int)(resources.GetObject("label11.TabIndex")));
			this.label11.Text = resources.GetString("label11.Text");
			this.label11.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("label11.TextAlign")));
			this.stdToolTip.SetToolTip(this.label11, resources.GetString("label11.ToolTip"));
			this.label11.Visible = ((bool)(resources.GetObject("label11.Visible")));
			// 
			// lblFrameRate
			// 
			this.lblFrameRate.AccessibleDescription = resources.GetString("lblFrameRate.AccessibleDescription");
			this.lblFrameRate.AccessibleName = resources.GetString("lblFrameRate.AccessibleName");
			this.lblFrameRate.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("lblFrameRate.Anchor")));
			this.lblFrameRate.AutoSize = ((bool)(resources.GetObject("lblFrameRate.AutoSize")));
			this.lblFrameRate.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("lblFrameRate.Dock")));
			this.lblFrameRate.Enabled = ((bool)(resources.GetObject("lblFrameRate.Enabled")));
			this.lblFrameRate.Font = ((System.Drawing.Font)(resources.GetObject("lblFrameRate.Font")));
			this.lblFrameRate.Image = ((System.Drawing.Image)(resources.GetObject("lblFrameRate.Image")));
			this.lblFrameRate.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("lblFrameRate.ImageAlign")));
			this.lblFrameRate.ImageIndex = ((int)(resources.GetObject("lblFrameRate.ImageIndex")));
			this.lblFrameRate.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("lblFrameRate.ImeMode")));
			this.lblFrameRate.Location = ((System.Drawing.Point)(resources.GetObject("lblFrameRate.Location")));
			this.lblFrameRate.Name = "lblFrameRate";
			this.lblFrameRate.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("lblFrameRate.RightToLeft")));
			this.lblFrameRate.Size = ((System.Drawing.Size)(resources.GetObject("lblFrameRate.Size")));
			this.lblFrameRate.TabIndex = ((int)(resources.GetObject("lblFrameRate.TabIndex")));
			this.lblFrameRate.Text = resources.GetString("lblFrameRate.Text");
			this.lblFrameRate.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("lblFrameRate.TextAlign")));
			this.stdToolTip.SetToolTip(this.lblFrameRate, resources.GetString("lblFrameRate.ToolTip"));
			this.lblFrameRate.Visible = ((bool)(resources.GetObject("lblFrameRate.Visible")));
			// 
			// label12
			// 
			this.label12.AccessibleDescription = resources.GetString("label12.AccessibleDescription");
			this.label12.AccessibleName = resources.GetString("label12.AccessibleName");
			this.label12.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("label12.Anchor")));
			this.label12.AutoSize = ((bool)(resources.GetObject("label12.AutoSize")));
			this.label12.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("label12.Dock")));
			this.label12.Enabled = ((bool)(resources.GetObject("label12.Enabled")));
			this.label12.Font = ((System.Drawing.Font)(resources.GetObject("label12.Font")));
			this.label12.Image = ((System.Drawing.Image)(resources.GetObject("label12.Image")));
			this.label12.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("label12.ImageAlign")));
			this.label12.ImageIndex = ((int)(resources.GetObject("label12.ImageIndex")));
			this.label12.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("label12.ImeMode")));
			this.label12.Location = ((System.Drawing.Point)(resources.GetObject("label12.Location")));
			this.label12.Name = "label12";
			this.label12.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("label12.RightToLeft")));
			this.label12.Size = ((System.Drawing.Size)(resources.GetObject("label12.Size")));
			this.label12.TabIndex = ((int)(resources.GetObject("label12.TabIndex")));
			this.label12.Text = resources.GetString("label12.Text");
			this.label12.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("label12.TextAlign")));
			this.stdToolTip.SetToolTip(this.label12, resources.GetString("label12.ToolTip"));
			this.label12.Visible = ((bool)(resources.GetObject("label12.Visible")));
			// 
			// lblBitrate
			// 
			this.lblBitrate.AccessibleDescription = resources.GetString("lblBitrate.AccessibleDescription");
			this.lblBitrate.AccessibleName = resources.GetString("lblBitrate.AccessibleName");
			this.lblBitrate.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("lblBitrate.Anchor")));
			this.lblBitrate.AutoSize = ((bool)(resources.GetObject("lblBitrate.AutoSize")));
			this.lblBitrate.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("lblBitrate.Dock")));
			this.lblBitrate.Enabled = ((bool)(resources.GetObject("lblBitrate.Enabled")));
			this.lblBitrate.Font = ((System.Drawing.Font)(resources.GetObject("lblBitrate.Font")));
			this.lblBitrate.Image = ((System.Drawing.Image)(resources.GetObject("lblBitrate.Image")));
			this.lblBitrate.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("lblBitrate.ImageAlign")));
			this.lblBitrate.ImageIndex = ((int)(resources.GetObject("lblBitrate.ImageIndex")));
			this.lblBitrate.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("lblBitrate.ImeMode")));
			this.lblBitrate.Location = ((System.Drawing.Point)(resources.GetObject("lblBitrate.Location")));
			this.lblBitrate.Name = "lblBitrate";
			this.lblBitrate.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("lblBitrate.RightToLeft")));
			this.lblBitrate.Size = ((System.Drawing.Size)(resources.GetObject("lblBitrate.Size")));
			this.lblBitrate.TabIndex = ((int)(resources.GetObject("lblBitrate.TabIndex")));
			this.lblBitrate.Text = resources.GetString("lblBitrate.Text");
			this.lblBitrate.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("lblBitrate.TextAlign")));
			this.stdToolTip.SetToolTip(this.lblBitrate, resources.GetString("lblBitrate.ToolTip"));
			this.lblBitrate.Visible = ((bool)(resources.GetObject("lblBitrate.Visible")));
			// 
			// label13
			// 
			this.label13.AccessibleDescription = resources.GetString("label13.AccessibleDescription");
			this.label13.AccessibleName = resources.GetString("label13.AccessibleName");
			this.label13.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("label13.Anchor")));
			this.label13.AutoSize = ((bool)(resources.GetObject("label13.AutoSize")));
			this.label13.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("label13.Dock")));
			this.label13.Enabled = ((bool)(resources.GetObject("label13.Enabled")));
			this.label13.Font = ((System.Drawing.Font)(resources.GetObject("label13.Font")));
			this.label13.Image = ((System.Drawing.Image)(resources.GetObject("label13.Image")));
			this.label13.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("label13.ImageAlign")));
			this.label13.ImageIndex = ((int)(resources.GetObject("label13.ImageIndex")));
			this.label13.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("label13.ImeMode")));
			this.label13.Location = ((System.Drawing.Point)(resources.GetObject("label13.Location")));
			this.label13.Name = "label13";
			this.label13.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("label13.RightToLeft")));
			this.label13.Size = ((System.Drawing.Size)(resources.GetObject("label13.Size")));
			this.label13.TabIndex = ((int)(resources.GetObject("label13.TabIndex")));
			this.label13.Text = resources.GetString("label13.Text");
			this.label13.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("label13.TextAlign")));
			this.stdToolTip.SetToolTip(this.label13, resources.GetString("label13.ToolTip"));
			this.label13.Visible = ((bool)(resources.GetObject("label13.Visible")));
			// 
			// lblVBVBuffer
			// 
			this.lblVBVBuffer.AccessibleDescription = resources.GetString("lblVBVBuffer.AccessibleDescription");
			this.lblVBVBuffer.AccessibleName = resources.GetString("lblVBVBuffer.AccessibleName");
			this.lblVBVBuffer.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("lblVBVBuffer.Anchor")));
			this.lblVBVBuffer.AutoSize = ((bool)(resources.GetObject("lblVBVBuffer.AutoSize")));
			this.lblVBVBuffer.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("lblVBVBuffer.Dock")));
			this.lblVBVBuffer.Enabled = ((bool)(resources.GetObject("lblVBVBuffer.Enabled")));
			this.lblVBVBuffer.Font = ((System.Drawing.Font)(resources.GetObject("lblVBVBuffer.Font")));
			this.lblVBVBuffer.Image = ((System.Drawing.Image)(resources.GetObject("lblVBVBuffer.Image")));
			this.lblVBVBuffer.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("lblVBVBuffer.ImageAlign")));
			this.lblVBVBuffer.ImageIndex = ((int)(resources.GetObject("lblVBVBuffer.ImageIndex")));
			this.lblVBVBuffer.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("lblVBVBuffer.ImeMode")));
			this.lblVBVBuffer.Location = ((System.Drawing.Point)(resources.GetObject("lblVBVBuffer.Location")));
			this.lblVBVBuffer.Name = "lblVBVBuffer";
			this.lblVBVBuffer.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("lblVBVBuffer.RightToLeft")));
			this.lblVBVBuffer.Size = ((System.Drawing.Size)(resources.GetObject("lblVBVBuffer.Size")));
			this.lblVBVBuffer.TabIndex = ((int)(resources.GetObject("lblVBVBuffer.TabIndex")));
			this.lblVBVBuffer.Text = resources.GetString("lblVBVBuffer.Text");
			this.lblVBVBuffer.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("lblVBVBuffer.TextAlign")));
			this.stdToolTip.SetToolTip(this.lblVBVBuffer, resources.GetString("lblVBVBuffer.ToolTip"));
			this.lblVBVBuffer.Visible = ((bool)(resources.GetObject("lblVBVBuffer.Visible")));
			// 
			// label14
			// 
			this.label14.AccessibleDescription = resources.GetString("label14.AccessibleDescription");
			this.label14.AccessibleName = resources.GetString("label14.AccessibleName");
			this.label14.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("label14.Anchor")));
			this.label14.AutoSize = ((bool)(resources.GetObject("label14.AutoSize")));
			this.label14.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("label14.Dock")));
			this.label14.Enabled = ((bool)(resources.GetObject("label14.Enabled")));
			this.label14.Font = ((System.Drawing.Font)(resources.GetObject("label14.Font")));
			this.label14.Image = ((System.Drawing.Image)(resources.GetObject("label14.Image")));
			this.label14.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("label14.ImageAlign")));
			this.label14.ImageIndex = ((int)(resources.GetObject("label14.ImageIndex")));
			this.label14.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("label14.ImeMode")));
			this.label14.Location = ((System.Drawing.Point)(resources.GetObject("label14.Location")));
			this.label14.Name = "label14";
			this.label14.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("label14.RightToLeft")));
			this.label14.Size = ((System.Drawing.Size)(resources.GetObject("label14.Size")));
			this.label14.TabIndex = ((int)(resources.GetObject("label14.TabIndex")));
			this.label14.Text = resources.GetString("label14.Text");
			this.label14.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("label14.TextAlign")));
			this.stdToolTip.SetToolTip(this.label14, resources.GetString("label14.ToolTip"));
			this.label14.Visible = ((bool)(resources.GetObject("label14.Visible")));
			// 
			// panelAudio
			// 
			this.panelAudio.AccessibleDescription = resources.GetString("panelAudio.AccessibleDescription");
			this.panelAudio.AccessibleName = resources.GetString("panelAudio.AccessibleName");
			this.panelAudio.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("panelAudio.Anchor")));
			this.panelAudio.AutoScroll = ((bool)(resources.GetObject("panelAudio.AutoScroll")));
			this.panelAudio.AutoScrollMargin = ((System.Drawing.Size)(resources.GetObject("panelAudio.AutoScrollMargin")));
			this.panelAudio.AutoScrollMinSize = ((System.Drawing.Size)(resources.GetObject("panelAudio.AutoScrollMinSize")));
			this.panelAudio.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("panelAudio.BackgroundImage")));
			this.panelAudio.Controls.Add(this.listViewAudioStream);
			this.panelAudio.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("panelAudio.Dock")));
			this.panelAudio.Enabled = ((bool)(resources.GetObject("panelAudio.Enabled")));
			this.panelAudio.Font = ((System.Drawing.Font)(resources.GetObject("panelAudio.Font")));
			this.panelAudio.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("panelAudio.ImeMode")));
			this.panelAudio.Location = ((System.Drawing.Point)(resources.GetObject("panelAudio.Location")));
			this.panelAudio.Name = "panelAudio";
			this.panelAudio.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("panelAudio.RightToLeft")));
			this.panelAudio.Size = ((System.Drawing.Size)(resources.GetObject("panelAudio.Size")));
			this.panelAudio.TabIndex = ((int)(resources.GetObject("panelAudio.TabIndex")));
			this.panelAudio.Text = resources.GetString("panelAudio.Text");
			this.stdToolTip.SetToolTip(this.panelAudio, resources.GetString("panelAudio.ToolTip"));
			this.panelAudio.Visible = ((bool)(resources.GetObject("panelAudio.Visible")));
			// 
			// listViewAudioStream
			// 
			this.listViewAudioStream.AccessibleDescription = resources.GetString("listViewAudioStream.AccessibleDescription");
			this.listViewAudioStream.AccessibleName = resources.GetString("listViewAudioStream.AccessibleName");
			this.listViewAudioStream.Alignment = ((System.Windows.Forms.ListViewAlignment)(resources.GetObject("listViewAudioStream.Alignment")));
			this.listViewAudioStream.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("listViewAudioStream.Anchor")));
			this.listViewAudioStream.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("listViewAudioStream.BackgroundImage")));
			this.listViewAudioStream.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																								  this.columnHeader1,
																								  this.columnHeader2,
																								  this.columnHeader3,
																								  this.columnHeader4,
																								  this.columnHeader5,
																								  this.columnHeader6,
																								  this.columnHeader10});
			this.listViewAudioStream.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("listViewAudioStream.Dock")));
			this.listViewAudioStream.Enabled = ((bool)(resources.GetObject("listViewAudioStream.Enabled")));
			this.listViewAudioStream.Font = ((System.Drawing.Font)(resources.GetObject("listViewAudioStream.Font")));
			this.listViewAudioStream.FullRowSelect = true;
			this.listViewAudioStream.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
			this.listViewAudioStream.HideSelection = false;
			this.listViewAudioStream.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("listViewAudioStream.ImeMode")));
			this.listViewAudioStream.LabelWrap = ((bool)(resources.GetObject("listViewAudioStream.LabelWrap")));
			this.listViewAudioStream.Location = ((System.Drawing.Point)(resources.GetObject("listViewAudioStream.Location")));
			this.listViewAudioStream.MultiSelect = false;
			this.listViewAudioStream.Name = "listViewAudioStream";
			this.listViewAudioStream.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("listViewAudioStream.RightToLeft")));
			this.listViewAudioStream.Size = ((System.Drawing.Size)(resources.GetObject("listViewAudioStream.Size")));
			this.listViewAudioStream.TabIndex = ((int)(resources.GetObject("listViewAudioStream.TabIndex")));
			this.listViewAudioStream.Text = resources.GetString("listViewAudioStream.Text");
			this.stdToolTip.SetToolTip(this.listViewAudioStream, resources.GetString("listViewAudioStream.ToolTip"));
			this.listViewAudioStream.View = System.Windows.Forms.View.Details;
			this.listViewAudioStream.Visible = ((bool)(resources.GetObject("listViewAudioStream.Visible")));
			this.listViewAudioStream.KeyDown += new System.Windows.Forms.KeyEventHandler(this.listViewAudioStream_KeyDown);
			this.listViewAudioStream.SelectedIndexChanged += new System.EventHandler(this.listViewAudioStream_SelectedIndexChanged);
			// 
			// columnHeader1
			// 
			this.columnHeader1.Text = resources.GetString("columnHeader1.Text");
			this.columnHeader1.TextAlign = ((System.Windows.Forms.HorizontalAlignment)(resources.GetObject("columnHeader1.TextAlign")));
			this.columnHeader1.Width = ((int)(resources.GetObject("columnHeader1.Width")));
			// 
			// columnHeader2
			// 
			this.columnHeader2.Text = resources.GetString("columnHeader2.Text");
			this.columnHeader2.TextAlign = ((System.Windows.Forms.HorizontalAlignment)(resources.GetObject("columnHeader2.TextAlign")));
			this.columnHeader2.Width = ((int)(resources.GetObject("columnHeader2.Width")));
			// 
			// columnHeader3
			// 
			this.columnHeader3.Text = resources.GetString("columnHeader3.Text");
			this.columnHeader3.TextAlign = ((System.Windows.Forms.HorizontalAlignment)(resources.GetObject("columnHeader3.TextAlign")));
			this.columnHeader3.Width = ((int)(resources.GetObject("columnHeader3.Width")));
			// 
			// columnHeader4
			// 
			this.columnHeader4.Text = resources.GetString("columnHeader4.Text");
			this.columnHeader4.TextAlign = ((System.Windows.Forms.HorizontalAlignment)(resources.GetObject("columnHeader4.TextAlign")));
			this.columnHeader4.Width = ((int)(resources.GetObject("columnHeader4.Width")));
			// 
			// columnHeader5
			// 
			this.columnHeader5.Text = resources.GetString("columnHeader5.Text");
			this.columnHeader5.TextAlign = ((System.Windows.Forms.HorizontalAlignment)(resources.GetObject("columnHeader5.TextAlign")));
			this.columnHeader5.Width = ((int)(resources.GetObject("columnHeader5.Width")));
			// 
			// columnHeader6
			// 
			this.columnHeader6.Text = resources.GetString("columnHeader6.Text");
			this.columnHeader6.TextAlign = ((System.Windows.Forms.HorizontalAlignment)(resources.GetObject("columnHeader6.TextAlign")));
			this.columnHeader6.Width = ((int)(resources.GetObject("columnHeader6.Width")));
			// 
			// columnHeader10
			// 
			this.columnHeader10.Text = resources.GetString("columnHeader10.Text");
			this.columnHeader10.TextAlign = ((System.Windows.Forms.HorizontalAlignment)(resources.GetObject("columnHeader10.TextAlign")));
			this.columnHeader10.Width = ((int)(resources.GetObject("columnHeader10.Width")));
			// 
			// openVideoDialog
			// 
			this.openVideoDialog.Filter = ((string)(configurationAppSettings.GetValue("VideoDialog.Filter", typeof(string))));
			this.openVideoDialog.Title = resources.GetString("openVideoDialog.Title");
			// 
			// openAudioDialog
			// 
			this.openAudioDialog.Filter = ((string)(configurationAppSettings.GetValue("AudioDialog.Filter", typeof(string))));
			this.openAudioDialog.Title = resources.GetString("openAudioDialog.Title");
			// 
			// cmAudio
			// 
			this.cmAudio.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					this.menuAdjustDelay,
																					this.menuItem9,
																					this.menuRemoveAudio});
			this.cmAudio.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("cmAudio.RightToLeft")));
			this.cmAudio.Popup += new System.EventHandler(this.cmAudio_Popup);
			// 
			// menuAdjustDelay
			// 
			this.menuAdjustDelay.Enabled = ((bool)(resources.GetObject("menuAdjustDelay.Enabled")));
			this.menuAdjustDelay.Index = 0;
			this.menuAdjustDelay.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("menuAdjustDelay.Shortcut")));
			this.menuAdjustDelay.ShowShortcut = ((bool)(resources.GetObject("menuAdjustDelay.ShowShortcut")));
			this.menuAdjustDelay.Text = resources.GetString("menuAdjustDelay.Text");
			this.menuAdjustDelay.Visible = ((bool)(resources.GetObject("menuAdjustDelay.Visible")));
			this.menuAdjustDelay.Click += new System.EventHandler(this.menuAdjustDelay_Click);
			// 
			// menuItem9
			// 
			this.menuItem9.Enabled = ((bool)(resources.GetObject("menuItem9.Enabled")));
			this.menuItem9.Index = 1;
			this.menuItem9.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("menuItem9.Shortcut")));
			this.menuItem9.ShowShortcut = ((bool)(resources.GetObject("menuItem9.ShowShortcut")));
			this.menuItem9.Text = resources.GetString("menuItem9.Text");
			this.menuItem9.Visible = ((bool)(resources.GetObject("menuItem9.Visible")));
			// 
			// menuRemoveAudio
			// 
			this.menuRemoveAudio.Enabled = ((bool)(resources.GetObject("menuRemoveAudio.Enabled")));
			this.menuRemoveAudio.Index = 2;
			this.menuRemoveAudio.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("menuRemoveAudio.Shortcut")));
			this.menuRemoveAudio.ShowShortcut = ((bool)(resources.GetObject("menuRemoveAudio.ShowShortcut")));
			this.menuRemoveAudio.Text = resources.GetString("menuRemoveAudio.Text");
			this.menuRemoveAudio.Visible = ((bool)(resources.GetObject("menuRemoveAudio.Visible")));
			this.menuRemoveAudio.Click += new System.EventHandler(this.menuRemoveAudio_Click);
			// 
			// StreamInfoCtrl
			// 
			this.AccessibleDescription = resources.GetString("$this.AccessibleDescription");
			this.AccessibleName = resources.GetString("$this.AccessibleName");
			this.AutoScroll = ((bool)(resources.GetObject("$this.AutoScroll")));
			this.AutoScrollMargin = ((System.Drawing.Size)(resources.GetObject("$this.AutoScrollMargin")));
			this.AutoScrollMinSize = ((System.Drawing.Size)(resources.GetObject("$this.AutoScrollMinSize")));
			this.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("$this.BackgroundImage")));
			this.ContextMenu = this.cmAudio;
			this.Controls.Add(this.panelAudio);
			this.Controls.Add(this.panelVideo);
			this.Controls.Add(this.panelControl);
			this.Enabled = ((bool)(resources.GetObject("$this.Enabled")));
			this.Font = ((System.Drawing.Font)(resources.GetObject("$this.Font")));
			this.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("$this.ImeMode")));
			this.Location = ((System.Drawing.Point)(resources.GetObject("$this.Location")));
			this.Name = "StreamInfoCtrl";
			this.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("$this.RightToLeft")));
			this.Size = ((System.Drawing.Size)(resources.GetObject("$this.Size")));
			this.stdToolTip.SetToolTip(this, resources.GetString("$this.ToolTip"));
			this.Load += new System.EventHandler(this.StreamInfoCtrl_Load);
			this.panelControl.ResumeLayout(false);
			this.panelVideo.ResumeLayout(false);
			this.panelAudio.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		#region ControlEvents

		private void StreamInfoCtrl_Load(object sender, System.EventArgs e)
		{
			// InitialDirectories setzen
			openVideoDialog.InitialDirectory=Settings.CurrentSettings.OpenDialogs_InitialDirectory;
		}

		private void butOpenVideo_Click(object sender, System.EventArgs e)
		{
			// Ask user to delete relates cuts from cutlist
			if (!State.CheckVideoInUse())
				return;

			if (openVideoDialog.ShowDialog() != DialogResult.OK)
				return;
			
			// Pfad sichern
			if (openVideoDialog.FileName!="")
				Settings.CurrentSettings.OpenDialogs_InitialDirectory=Path.GetDirectoryName(openVideoDialog.FileName);

			string video=Path.GetFileNameWithoutExtension(openVideoDialog.FileName);
			ArrayList matchlist=GUIHelper.AudioMatchList(Path.GetDirectoryName(openVideoDialog.FileName),video);

			// We will load another video file -> reset GUI
			State.InitializeApplication(false);
			
			if (State.LoadAudioAndVideo(openVideoDialog.FileName, matchlist)==DialogResult.OK)
			{
				if (matchlist.Count==0)
					butOpenAudio_Click(sender,e);
			}
		}

		private void butOpenAudio_Click(object sender, System.EventArgs e)
		{
			if (openVideoDialog.FileName!="")
				openAudioDialog.InitialDirectory=Path.GetDirectoryName(openVideoDialog.FileName);
			else
				openAudioDialog.InitialDirectory=openVideoDialog.InitialDirectory;

			openAudioDialog.FileName="";
			if (openAudioDialog.ShowDialog() == DialogResult.OK)
			{
				ArrayList audioFiles = new ArrayList();
				audioFiles.Add(openAudioDialog.FileName);
				State.LoadAudioAndVideo(null, audioFiles);
			}
		}

		private void butMoveAudioUp_Click(object sender, System.EventArgs e)
		{
			State.MoveSelectedAudioUp();
		}

		private void butMoveAudioDown_Click(object sender, System.EventArgs e)
		{
			State.MoveSelectedAudioDown();
		}

		private void btnNewTab_Click(object sender, System.EventArgs e)
		{
			CMMain.globalApplication.AddNewVideoTab();
		}

		private void btnCloseTab_Click(object sender, System.EventArgs e)
		{
			CMMain.globalApplication.RemoveVideoTab(this);
		}

		#endregion

		#region Static Event-Handlers

		/// <summary>
		/// ----- !STATIC! -----
		/// This is called after the user switched to another video tab
		/// </summary>
		static public void SetActiveVideoTab(Control i_SelectedCtrl)
		{
			// enable the selected StreamInfoCtrl
			mi_SelectedTab = (StreamInfoCtrl) i_SelectedCtrl;
			// activate the streams from the StreamInfoCtrl which is ENABLED
			mi_StreamSet = mi_SelectedTab.mi_MyStreams;
		}

		// ----- !STATIC! -----
		static private void OnSetDefaults(bool b_NewProject)
		{
			mi_StreamSet.Reset();

			mi_SelectedTab.Enabled=true;

			mi_SelectedTab.lblVideoFileName.Text=string.Empty;
			mi_SelectedTab.lblVideoLength.Text=string.Empty;
			mi_SelectedTab.lblVBVBuffer.Text=string.Empty;
			mi_SelectedTab.lblResolution.Text=string.Empty;
			mi_SelectedTab.lblRatio.Text=string.Empty;
			mi_SelectedTab.lblFrameRate.Text=string.Empty;
			mi_SelectedTab.lblBitrate.Text=string.Empty;

			mi_SelectedTab.butOpenAudio.    Enabled=false;
			mi_SelectedTab.butMoveAudioUp.  Enabled=false;
			mi_SelectedTab.butMoveAudioDown.Enabled=false;

			mi_SelectedTab.listViewAudioStream.Items.Clear();
		}

		// ----- !STATIC! -----
		static private void OnAudioVideoOpened(bool videoOpened, bool audioOpened, bool reOpened)
		{
			if (videoOpened && State.VideoStream != null)
			{
				if (!Settings.CurrentSettings.DisplayFullPathInformations)
					mi_SelectedTab.lblVideoFileName.Text=Path.GetFileName(State.VideoStream.StreamName);
				else
					mi_SelectedTab.lblVideoFileName.Text=State.VideoStream.StreamName;
			
				mi_SelectedTab.butOpenAudio.Enabled=true;
				mi_SelectedTab.lblVideoLength.Text=Helper.FramesToTimeString(State.VideoStream.PicturesCount,State.VideoStream.FramesPerSecond);
				mi_SelectedTab.UpdateVideoInformations();
			}

			// Update Audio List ALWAYS even if no Audiofiles were added
			mi_SelectedTab.RebuildAudioStreamsListView();
			mi_SelectedTab.UpdateAudioInformations();
		}

		// ----- !STATIC! -----
		static private void OnSettingsChanged()
		{
			OnAudioVideoOpened(true, true, true);
		}

		// ----- !STATIC! -----
		static private void OnVideoPositionUpdated(int newPosition)
		{
			mi_SelectedTab.UpdateVideoInformations();
			mi_SelectedTab.UpdateAudioInformations();
		}

		#endregion

		#region Audio List Methods

		private void listViewAudioStream_KeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
		{
			if (e.KeyData==Keys.Delete)
				State.DeleteSelectedAudio();
		}

		private void listViewAudioStream_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			if (listViewAudioStream.SelectedIndices.Count>0)
				State.SelectedAudioIndex=listViewAudioStream.SelectedIndices[0];
			else
				State.SelectedAudioIndex=-1;
		}

		#endregion

		#region ContextMenu

		private void menuRemoveAudio_Click(object sender, System.EventArgs e)
		{
			State.DeleteSelectedAudio();
		}

		private void menuAdjustDelay_Click(object sender, System.EventArgs e)
		{
			State.AdjustSelectedAudioDelay();
		}

		private void cmAudio_Popup(object sender, System.EventArgs e)
		{
			bool menuVisible=listViewAudioStream.SelectedItems.Count!=0;

			foreach (MenuItem mi in cmAudio.MenuItems)
				mi.Visible=menuVisible;
		}

		#endregion

		#region DrawInformations

		private void UpdateVideoInformations()
		{
			// Videostreaminfos an die aktuelle Pos. anpassen
			Sequence seq=null;
			if (State.CurrentVideoPosition>=0)
			{
				Mpeg12Object pic=State.VideoStream.GetPicture(State.CurrentVideoPosition);
				if (pic!=null)
					seq=(Sequence)State.VideoStream.GetPrevMpeg12Object(typeof(Sequence),pic);
			}
			if (seq==null)
				seq=State.VideoStream.Sequence0;
			
			lblResolution.Text=string.Format("{0}x{1}{2}",seq.horizontal_size_value,seq.vertical_size_value,seq.progressive_sequence?'p':'i');
			
			if (seq.sequence_extension_exist)
				lblRatio.Text=Mpeg12VideoConst.Mpeg2RatioInfo[seq.aspect_ratio_information];
			else
				lblRatio.Text=Mpeg12VideoConst.Mpeg1RatioInfo[seq.aspect_ratio_information];
			
			if (seq.frame_rate_code>=1 && seq.frame_rate_code<=8)
				lblFrameRate.Text=State.VideoStream.FramesPerSecond.ToString("0.##")+" fps";
			else
				lblFrameRate.Text=Resource.CurrentResource[Resource.UNKNOWN];
			
			// If the sequence seq does not contain a valid bitrate, we display 
			// the average bitrate of the whole stream. This is better than 
			// displaying 15000 kB/s which is !definitely! wrong.
			decimal bitRate = 0;
			if (seq.bit_rate_value != 0x3ffff && seq.BitrateKbps != 15000)
				bitRate = seq.BitrateKbps;
			else 
				bitRate = State.VideoStream.AverageBitrateKBps;

			// switch to a precision of 100 Bytes/sec below 100 kBps
			string s_Format = (bitRate < 100) ? "0.0" : "0";
			lblBitrate.Text=bitRate.ToString(s_Format) + " kBit/s";

			lblVBVBuffer.Text=seq.vbv_buffer_size_value.ToString()+" kWords";
		}

		private void UpdateAudioInformations()
		{
			if (State.AudioStreams.Count>0 && listViewAudioStream.Items.Count>0)
			{
				for (int index=0;index<State.AudioStreams.Count;++index)
				{
					int audio_index=0;
					if (State.CurrentVideoPosition>=0 && State.CurrentVideoPosition!=int.MaxValue)
					{
						audio_index=State.AudioStream(index).GetLessOrSame(State.CurrentVideoPosition*1000m/State.VideoStream.FramesPerSecond);
					}
					AudioType row=State.AudioStream(index).GetSample(audio_index);
					ListViewItem lvi=listViewAudioStream.Items[index];
					lvi.SubItems[2].Text=row.Description;
					lvi.SubItems[3].Text=string.Format("{0} kBit/s",row.Bitrate/1000);
					lvi.SubItems[4].Text=string.Format("{0} Hz",row.Samplerate);
					string modeText=row.Mode;
					if (modeText.StartsWith("~"))
						modeText=Resource.CurrentResource[modeText];
					lvi.SubItems[5].Text=modeText;
					lvi.SubItems[6].Text=string.Format("{0} ms",State.AudioStream(index).StartDelay);
				}
			}
		}

		private void RebuildAudioStreamsListView()
		{
			listViewAudioStream.Items.Clear();
			for (int index=0;index<State.AudioStreams.Count;++index)
			{
				string name,length;
				BasicAudioStream bsa=State.AudioStream(index);
				if (!Settings.CurrentSettings.DisplayFullPathInformations)
					name=Path.GetFileName(bsa.StreamName);
				else
					name=bsa.StreamName;
				length=new DateTime().AddMilliseconds((double)(bsa.Length)).ToString("HH:mm:ss,fff");
				ListViewItem lvi=new ListViewItem(new string[] {name,length,"","","","",""});
				listViewAudioStream.Items.Add(lvi);
			}
			// Den gewnschten Eintrag selektieren
			if (listViewAudioStream.Items.Count>0)
			{
				listViewAudioStream.Items[State.SelectedAudioIndex].Selected=true;
				butMoveAudioUp.Enabled=butMoveAudioDown.Enabled=true;
			}
			else
				butMoveAudioUp.Enabled=butMoveAudioDown.Enabled=false;
		}

		#endregion

		#region Properties

		// ----- !STATIC! -----
		static public cStreamSet Streams
		{
			get { return mi_StreamSet; }
		}

		// ----- !Instance! -----
		/// <summary>
		/// Returns the path of the videostream of this StreamInfoCtrl or "" if not yet loaded
		/// </summary>
		public string VideoFile
		{
			get
			{
				if (mi_MyStreams.VideoStream == null)
					return "";
				else
					return mi_MyStreams.VideoStream.StreamName;
			}
		}

		public override object PersistData
		{
			get
			{
				ArrayList colWidth=new ArrayList();
				foreach (ColumnHeader ch in listViewAudioStream.Columns)
					colWidth.Add(ch.Width);
				return colWidth;
			}
			set
			{
				if (value is ArrayList && ((ArrayList)value).Count==listViewAudioStream.Columns.Count)
				{
					for (int index=0;index<listViewAudioStream.Columns.Count;++index)
						listViewAudioStream.Columns[index].Width=(int)((ArrayList)value)[index];
				}
			}
		}

		#endregion
	}
}
