/*
TabManager for MagicLibrary
Written by Elm (http://netcult.ch/elmue)
Date: 19-11-2006

----------------------------------------------------------------------------------
Using these conventions results in better readable code and less coding errors !
----------------------------------------------------------------------------------

     cName  for generic class definitions
     CName  for MFC     class definitions
     tName  for type    definitions
     eName  for enum    definitions
     kName  for struct  definitions

    e_Name  for enum variables
    E_Name  for enum constant values

    i_Name  for instances of classes
    h_Name  for handles

    T_Name  for Templates
    t_Name  for TCHAR or LPTSTR

    s_Name  for strings
   bs_Name  for BSTR
    f_Name  for function pointers
    k_Name  for contructs (struct)

    b_Name  bool,BOOL 1 Bit

   s8_Name    signed  8 Bit (char)
  s16_Name    signed 16 Bit (SHORT, WCHAR)
  s32_Name    signed 32 Bit (LONG, int)
  s64_Name    signed 64 Bit (LONGLONG)

   u8_Name  unsigned  8 Bit (BYTE)
  u16_Name  unsigned 16 bit (WORD)
  u32_Name  unsigned 32 Bit (DWORD, UINT)
  u64_Name  unsigned 64 Bit (ULONGLONG)

    d_Name  for double

  ----------------

    m_Name  for member variables of a class (e.g. ms32_Name for int member variable)
    g_Name  for global variables            (e.g. gu16_Name for global WORD)
    p_Name  for pointer                     (e.g.   ps_Name  *pointer to string)
   pp_Name  for pointer to pointer          (e.g.  ppd_Name **pointer to double)

*************************************************************************************/

using System;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using StringResources;
using Crownwood.Magic.Common;
using Crownwood.Magic.Docking;


namespace GUI.MainForm
{
	/// <summary>
	/// Manages multiple tabs in a Crownwood TabControl
	/// Used to display multiple video tabs
	/// </summary>
	public class TabManager
	{
		private DockingManager mi_Manager;
		private Type           mt_HostedCtrl;
		private bool           mb_BlockTabManager = false;
		private ArrayList      mi_TabList = new ArrayList();
		private string         ms_TabTitle;

		/// <summary>
		/// Fired after the user has selected another tab
		/// </summary>
		public event         SelectionChangedDelegate  SelectionChanged;
		public delegate void SelectionChangedDelegate(Control i_SelectedCtrl);

		// Constructor
		public TabManager(DockingManager i_Manager,      // The GUI Master
		                  string         s_TabTitle)     // The tab title in the form "Video {0}"
		{
			mi_Manager  = i_Manager;
			ms_TabTitle = s_TabTitle;

			mi_Manager.ContentShown  += new DockingManager.ContentHandler(OnContentShown);
			mi_Manager.ContentHidden += new DockingManager.ContentHandler(OnContentHidden);
		}

		/// <summary>
		/// Returns the current count of tabs
		/// </summary>
		public int Count
		{
			get { return mi_TabList.Count; }
		}

		/// <summary>
		/// returns the Content at the given index
		/// </summary>
		public Content GetTabContent(int s32_Index)
		{
			if (s32_Index < 0 || s32_Index >= mi_TabList.Count)
				return null;
			else
				return (Content) mi_TabList[s32_Index];
		}

		protected void OnContentShown(Content i_Cont, EventArgs ev)
		{
			ManageTabs(i_Cont, true);
		}

		protected void OnContentHidden(Content i_Cont, EventArgs ev)
		{
			ManageTabs(i_Cont, false);
		}

		/// <summary>
		/// Show or hide (but not remove) and rename tabs
		/// ATTENTION ATTENTION ATTENTION ATTENTION ATTENTION ATTENTION ATTENTION 
		/// Do not modify this function!!
		/// It has been tested under all possible conditions. 
		/// Any modification will have unwanted side effects!!
		/// This function must behave correctly after adding/removing tabs, after
		/// switching to another GUI preset, the tab order must be preserved,
		/// toggle of docking/floating must function and the tabs must be renamed properly.
		/// </summary>
		protected void ManageTabs(Content i_Cont, bool b_Show)
		{
			if (i_Cont.Control.GetType() != mt_HostedCtrl)
				return;

			if (!mb_BlockTabManager)
			{
				mb_BlockTabManager = true;

				if (!b_Show) // a mt_HostedCtrl has been hidden -> hide all other tabs
				{
					// ATTENTION: Do NOT start loop with T=0; here otherwise 
					// a double click to invoke floating mode will fail
					for (int T=1; T<mi_TabList.Count; T++)
					{
						// Do NOT set reorder=true !!! (this would shuffle the tab order)
						mi_Manager.HideContent((Content)mi_TabList[T], true, false);
					}
				}
				else // a mt_HostedCtrl has been shown -> show and rename all tabs
				{
					// Do NOT use FirstTabContent.ParentWindowContent here !!!
					WindowContentTabbed i_Tab = (WindowContentTabbed) i_Cont.ParentWindowContent;

					// i_Tab is not always the same instance so we must add the handler each time anew
					i_Tab.TabControl.SelectionChanged -= new EventHandler(OnTabSelectionChanged);
					i_Tab.TabControl.SelectionChanged += new EventHandler(OnTabSelectionChanged);
					i_Tab.Contents.Inserted -= new Crownwood.Magic.Collections.CollectionChange(OnTabContentInserted);
					i_Tab.Contents.Inserted += new Crownwood.Magic.Collections.CollectionChange(OnTabContentInserted);

					// show tabs
					for (int T=0; T<mi_TabList.Count; T++)
					{
						Content i_addCont = (Content) mi_TabList[T];
						// IMPORTANT: The small HIDE button at top right of the title bar 
						// in the tab MUST be disabled anew here after loading XML presets!!
						i_addCont.CloseButton = false;
						mi_Manager.AddContentToWindowContent(i_addCont, i_Tab);
					}
				}
			}

			RenameTabs(); // after adding tabs AND after deleting tabs (renumber)

			mb_BlockTabManager = false;
		}

		protected void OnTabContentInserted(int index, object value)
		{
			// Required after switching into floating mode (tabs are removed and added anew)
			RenameTabs();
		}

		/// <summary>
		/// Renames tabs into "Video 1", "Video 2", etc..
		/// This must be called ALWAYS: on adding AND on removing tabs
		/// </summary>
		protected void RenameTabs()
		{
			WindowContentTabbed i_Tab = (WindowContentTabbed) GetTabContent(0).ParentWindowContent;
			if (i_Tab == null)
				return;
			
			for (int T=0; T<i_Tab.TabControl.TabPages.Count; T++)
			{
				// ATTENTION: (setting TabPage.Title = "Video 1")
				// DO NOT MODIFY Content.Title as this is required when saving magic 
				// XML data. Content.Title must always stay "WINDOW_STREAM_INFO"
				i_Tab.TabControl.TabPages[T].Title = string.Format(ms_TabTitle, T+1);
			}
		}

		/// <summary>
		/// Executed when the user clicked on a tab
		/// </summary>
		protected void OnTabSelectionChanged(object sender, EventArgs e)
		{
			if (SelectionChanged != null)
			{
				Crownwood.Magic.Controls.TabControl i_Tab = (Crownwood.Magic.Controls.TabControl) sender;
				if (i_Tab.SelectedTab != null) // fire event
					SelectionChanged(i_Tab.SelectedTab.Control);
			}
		}

		/// <summary>
		/// This must be called after the caller has added a new tab
		/// </summary>
		public void AddedNewTab(Content i_NewContent, bool b_SelectNewTab)
		{
			// IMPORTANT: The small HIDE button at top right of the title bar 
			// in the tab MUST be disabled!!
			i_NewContent.CloseButton = false; 

			mi_TabList.Add(i_NewContent);
			mt_HostedCtrl = i_NewContent.Control.GetType();

			if (b_SelectNewTab) // select the last (recently added) tab
				SelectTab(this.Count -1);
		}

		/// <summary>
		/// returns the index of the tab which holds the desired control or -1 if not found
		/// </summary>
		public int FindTab(Control i_Ctrl)
		{
			for (int T=0; T<mi_TabList.Count; T++)
			{
				Content i_Cont = (Content)mi_TabList[T];
				if (i_Cont.Control == i_Ctrl)
					return T;
			}
			return -1;
		}

		/// <summary>
		/// This will remove a tab and dispose its Control
		/// </summary>
		public void RemoveTab(Control i_Ctrl)
		{
			// Never remove the LAST tab!
			if (mi_TabList.Count <= 1)
				return;

			int Pos = FindTab(i_Ctrl);
			if (Pos < 0)
				return;

			mb_BlockTabManager = true;

			Content i_Cont = (Content)mi_TabList[Pos];

			mi_TabList.RemoveAt(Pos);
			mi_Manager.HideContent(i_Cont);
			mi_Manager.Contents.Remove(i_Cont);
			// Important: This will close the video streams
			i_Cont.Control.Dispose();

			mb_BlockTabManager = false;
		}

		/// <summary>
		/// Remove all tabs but the last one and dispose their Controls
		/// </summary>
		public void RemoveAllTabs()
		{
			while (mi_TabList.Count > 1)
			{
				RemoveTab(GetTabContent(0).Control);
			}
		}

		public void SelectTab(int s32_Index)
		{
			if (s32_Index < 0 || s32_Index >= this.Count)
				return;

			WindowContentTabbed i_Tab = (WindowContentTabbed) GetTabContent(0).ParentWindowContent;
			if (i_Tab != null) // Tab invisible ?
				i_Tab.TabControl.SelectedIndex = s32_Index;
		}
	}
}
