/*
Video Scroller for Cuttermaran
Written by Elm (http://netcult.ch/elmue)
Date: 15-10-2006

----------------------------------------------------------------------------------
Using these conventions results in better readable code and less coding errors !
----------------------------------------------------------------------------------

     cName  for generic class definitions
     CName  for MFC     class definitions
     tName  for type    definitions
     eName  for enum    definitions
     kName  for struct  definitions

    e_Name  for enum variables
    E_Name  for enum constant values

    i_Name  for instances of classes
    h_Name  for handles

    T_Name  for Templates
    t_Name  for TCHAR or LPTSTR

    s_Name  for strings
   bs_Name  for BSTR
    f_Name  for function pointers
    k_Name  for contructs (struct)

    b_Name  bool,BOOL 1 Bit

   s8_Name    signed  8 Bit (char)
  s16_Name    signed 16 Bit (SHORT, WCHAR)
  s32_Name    signed 32 Bit (LONG, int)
  s64_Name    signed 64 Bit (LONGLONG)

   u8_Name  unsigned  8 Bit (BYTE)
  u16_Name  unsigned 16 bit (WORD)
  u32_Name  unsigned 32 Bit (DWORD, UINT)
  u64_Name  unsigned 64 Bit (ULONGLONG)

    d_Name  for double

  ----------------

    m_Name  for member variables of a class (e.g. ms32_Name for int member variable)
    g_Name  for global variables            (e.g. gu16_Name for global WORD)
    p_Name  for pointer                     (e.g.   ps_Name  *pointer to string)
   pp_Name  for pointer to pointer          (e.g.  ppd_Name **pointer to double)

*************************************************************************************/


using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;

namespace GUI.MainForm.VideoScroller
{
	/// <summary>
	/// An owner drawn horizontal scrollbar which can replace System.Windows.Forms.HScrollBar
	/// Additional functionality: the background color can be set in coloured blocks 
	/// FIRST of all you must set Minimum and Maximum !!
	/// This scrollbar will always reach its Maximum 
	/// (while HScrollBar reaches only Maximum - LargeChange + 1)
	/// </summary>
	public class cBackScrollBarH : Control
	{
		#region struct kBackgr

		// defines one block of coloured background -> stored in arraylist of blocks
		protected struct kBackgr
		{
			public int  s32_Start;  // a value between cBackScrollBarH.Minimum and Maximum
			public int  s32_End;    // a value between cBackScrollBarH.Minimum and Maximum
			public Color  i_Color;

			// Constructor
			public kBackgr(int Start, int End, Color Col)
			{
				s32_Start = Start;
				s32_End   = End;
				  i_Color = Col;
			}
		}

		#endregion

		private Bitmap     mi_BackgrBmp   = null;  // the scrollbar's background
		private ArrayList  mi_BackgrList  = null;  // stores kBackgr
		private int      ms32_Minimum     = 0;
		private int      ms32_Maximum     = 0;
		private int      ms32_SmallChange = 1;
		private int      ms32_LargeChange = 10;
		private int      ms32_Value       = 0;
		private int      ms32_PtrPos      = 0;
		private int      ms32_RepeatDelay = 60;
		private bool       mb_SmallDown   = false; // button up / down
		private bool       mb_SmallUp     = false;
		private bool       mb_LargeUp     = false;
		private bool       mb_LargeDown   = false;
		private bool       mb_Dragging    = false;
		private int      ms32_BkgLeft     = 0;
		private int      ms32_BkgWidth    = 0;
		private double   mdbl_BkgFactor   = 0;
		private double   mdbl_PtrFactor   = 0;
		private System.Windows.Forms.Timer mi_RepeatTimer = null;

		#region Construction

		public cBackScrollBarH()
		{
			this.Name = "cBackScrollBarH";
			this.Text = "cBackScrollBarH"; // invisible
			// The background color of the scrollbar is a little brighter than the dialog background
			this.BackColor = ControlPaint.Light(SystemColors.Control);
			
			mi_RepeatTimer = new System.Windows.Forms.Timer();
			mi_RepeatTimer.Tick += new EventHandler(OnRepeatTimer);
		}

		protected override void OnCreateControl()
		{
			base.OnCreateControl();
			// The following makes the control draw its background
			// in WM_PAINT instead of WM_ERASEBACKGROUND
			SetStyle(ControlStyles.UserPaint,            true);
			SetStyle(ControlStyles.AllPaintingInWmPaint, true);
			// This avaoids flickering of the slider when it moves
			SetStyle(ControlStyles.DoubleBuffer,         true);
		}

		#endregion

		#region Properties & Events

		/// <summary>
		/// This event will ONLY be fired when the USER moved the scrollbar
		/// It will NOT be fired when program code modified Value!
		/// </summary>
		public event ScrollEventHandler Scroll = null;

		private void FireScrollEvent(ScrollEventType i_Type)
		{
			if (Scroll != null) 
				Scroll(this, new ScrollEventArgs(i_Type, Value));
		}

		/// <summary>
		/// Is fired when the user moved the scrollbar OR 
		/// when C# code modified the Value property
		/// The same behaviour as for HScrollBar
		/// </summary>
		public event EventHandler ValueChanged = null;

		public int Minimum
		{
			get { return ms32_Minimum; }
			set 
			{ 
				ms32_Minimum = value;
				ResetBackgroundColors();
			}
		}

		/// <summary>
		/// The scrollbar will always reach its Maximum 
		/// (HScrollBar reaches only Maximum - LargeChange + 1)
		/// </summary>
		public int Maximum
		{
			get { return ms32_Maximum; }
			set 
			{ 
				ms32_Maximum = value;
				ResetBackgroundColors();
			}
		}

		/// <summary>
		/// defines the current scrollbar position between Minimum and Maximum
		/// Has to be called AFTER setting Minimum and Maximum !
		/// The scrollbar will always reach its Maximum 
		/// (HScrollBar reaches only Maximum - LargeChange + 1)
		/// </summary>
		public int Value
		{
			get { return ms32_Value; }
			set 
			{ 
				int s32_OldValue = ms32_Value;
				ms32_Value = Math.Max(ms32_Minimum, Math.Min(ms32_Maximum, value)); 
				
				// do all the painting stuff only if the slider has moved
				if (CalcPtrPos(ms32_Value) != ms32_PtrPos)
					Invalidate();

				if (s32_OldValue != ms32_Value && ValueChanged != null)
				{
					// Fire the event
					ValueChanged(this, EventArgs.Empty);
				}

				if (ms32_Value == ms32_Minimum) FireScrollEvent(ScrollEventType.First);
				if (ms32_Value == ms32_Maximum) FireScrollEvent(ScrollEventType.Last);
			}
		}

		/// <summary>
		/// Calculate the slider position from the Value
		/// </summary>
		private int CalcPtrPos(int s32_Value)
		{
			return (int)(mdbl_PtrFactor * (s32_Value - Minimum)) + ms32_BkgLeft;
		}

		/// <summary>
		/// Size of the step when clicking the buttons at the end of the scrollbar
		/// </summary>
		public int SmallChange
		{
			get { return ms32_SmallChange;  }
			set { ms32_SmallChange = value; }
		}

		/// <summary>
		/// Size of the step when clicking onto an area of the scrollbar where is no button
		/// </summary>
		public int LargeChange
		{
			get { return ms32_LargeChange;  }
			set { ms32_LargeChange = value; }
		}

		/// <summary>
		/// Sets the timer delay to fire the event when the user
		/// holds down a scrollbar end button
		/// </summary>
		public int RepeatDelay
		{
			set { ms32_RepeatDelay = value; }
		}

		#endregion

		/// <summary>
		/// Removes all blocks of defined background colors
		/// </summary>
		public void ResetBackgroundColors()
		{
			mi_BackgrList = null;
			mi_BackgrBmp  = null;
		}

		/// <summary>
		/// Store a block of background color and its position and size in the ArrayList
		/// Has to be called AFTER setting Minimum and Maximum !
		/// After adding the last color you must call Invalidate()
		/// </summary>
		public void AddBackgroundColor(int s32_StartPos, int s32_EndPos, Color i_Color, bool b_Append)
		{
			if (s32_StartPos > s32_EndPos || s32_StartPos < Minimum || s32_EndPos > Maximum)
				throw new Exception("Invalid values when setting scrollbar background colors!");

			if (mi_BackgrList == null)
				mi_BackgrList = new ArrayList();

			if (b_Append) mi_BackgrList.Add   (   new kBackgr(s32_StartPos, s32_EndPos, i_Color));
			else          mi_BackgrList.Insert(0, new kBackgr(s32_StartPos, s32_EndPos, i_Color));
			mi_BackgrBmp = null;
		}

		#region Painting

		protected override void OnSizeChanged(EventArgs ev)
		{
			base.OnSizeChanged(ev);
			mi_BackgrBmp = null;
			Invalidate();
		}

		/// <summary>
		/// Only once create a bitmap which contains the background of the scrollbar
		/// to increase drawing speed
		/// </summary>
		protected void CreateBackgroundBitmap(Graphics i_GraphDC)
		{
			if (mi_BackgrBmp != null)
				return;

			// ----- Calculate constants ------

			// the end buttons on the left and on the right are square, so Width == Height
			// Start of Background bitmap
			ms32_BkgLeft  = this.Height;
			// Width of Background bitmap
			ms32_BkgWidth = Math.Max(1, this.Width - 2* this.Height);
			// Total width of scrollable area (where the slider can move)
			int s32_PtrWidth = Math.Max(0, ms32_BkgWidth - 10);

			mdbl_BkgFactor = 0;
			mdbl_PtrFactor = 0;

			int s32_ScrollRange = Math.Max(1, Maximum - Minimum);

			mdbl_BkgFactor = (double)ms32_BkgWidth / (double)s32_ScrollRange;
			mdbl_PtrFactor = (double) s32_PtrWidth / (double)s32_ScrollRange;

			// ----- Create bitmap ------

			mi_BackgrBmp = new Bitmap(ms32_BkgWidth, this.Height, i_GraphDC);

			Graphics i_GraphBmp = Graphics.FromImage(mi_BackgrBmp);
			
			// First erase the whole background of the scrollbar
			Brush i_BkgBrush = new SolidBrush(this.BackColor);
			i_GraphBmp.FillRectangle(i_BkgBrush, 0, 0, ms32_BkgWidth, this.Height);
			i_BkgBrush.Dispose();

			if (mi_BackgrList == null || mdbl_BkgFactor <= 0)
				return;

			// now draw the coloured blocks
			foreach (kBackgr k_Backgr in mi_BackgrList)
			{
				int s32_BkgStart = (int)(mdbl_BkgFactor * (k_Backgr.s32_Start - Minimum));
				int s32_BkgEnd   = (int)(mdbl_BkgFactor * (k_Backgr.s32_End   - Minimum));

				Brush i_ColBrush = new SolidBrush(k_Backgr.i_Color);
				i_GraphBmp.FillRectangle(i_ColBrush, s32_BkgStart, 0, s32_BkgEnd-s32_BkgStart, this.Height);
				i_ColBrush.Dispose();
			}
		}

		/// <summary>
		/// This will be executed in WM_PAINT !!
		/// The background is painted in the highest possible speed
		/// because we simply draw an image into the DC
		/// </summary>
		protected override void OnPaintBackground(PaintEventArgs ev)
		{
			CreateBackgroundBitmap(ev.Graphics);
			ev.Graphics.DrawImage (mi_BackgrBmp, ms32_BkgLeft, 0, ms32_BkgWidth, this.Height);
		}

		/// <summary>
		/// Paint the scrollbar buttons
		/// </summary>
		protected override void OnPaint(PaintEventArgs ev)
		{
			int s32_PosL = 0;
			int s32_PosR = this.Width - this.Height;
			ButtonState e_Left  = mb_SmallDown ? ButtonState.Pushed : ButtonState.Normal;
			ButtonState e_Right = mb_SmallUp   ? ButtonState.Pushed : ButtonState.Normal;
			if (!Enabled) 
			{
				e_Left  = ButtonState.Inactive;
				e_Right = ButtonState.Inactive;
			}
			
			ControlPaint.DrawScrollButton(ev.Graphics, s32_PosL, 0, Height, Height, ScrollButton.Left,  e_Left);
			ControlPaint.DrawScrollButton(ev.Graphics, s32_PosR, 0, Height, Height, ScrollButton.Right, e_Right);

			if (mdbl_PtrFactor <= 0)
				return;

			ms32_PtrPos = CalcPtrPos(Value);
			ButtonState i_State = Enabled ? ButtonState.Normal : ButtonState.Flat;
			ControlPaint.DrawButton(ev.Graphics, ms32_PtrPos, 0, 10, Height, i_State);
		}

		#endregion

		#region Mouse Events

		protected override void OnMouseDown(MouseEventArgs ev)
		{
			// The left end button has been clicked
			if (ev.X < this.Height) 
			{
				mb_SmallDown = true;
				Invalidate(); // Push button
			}
			// The right end button has been clicked
			else if (ev.X > this.Width - this.Height)
			{
				mb_SmallUp = true;
				Invalidate(); // Push button
			}
			// Click between Slider and left end button
			else if (ev.X < ms32_PtrPos) 
			{
				mb_LargeDown = true;
			}
			// Click between Slider and right end button
			else if (ev.X > ms32_PtrPos + 10)
			{
				mb_LargeUp = true;
			}
			// Click on the slider itself
			else
			{
				mb_Dragging = true;
				return;
			}

			AdjustValue(0);

			mi_RepeatTimer.Interval = 400; // delay before repetition starts
			mi_RepeatTimer.Start();
		}

		/// <summary>
		/// Dragging the slider
		/// </summary>
		protected override void OnMouseMove(MouseEventArgs ev)
		{
			if (!mb_Dragging || mdbl_PtrFactor <= 0)
				return;

			// This is a kind of rounding: if Maximum and Minimum are very near to each other
			// the slider will jump to the nearest legal slider position
			int X = ev.X +  (int)(mdbl_PtrFactor / 2) - 5;
			int s32_Value = (int)((double)(X - ms32_BkgLeft) / mdbl_PtrFactor) + Minimum;
			
			if (s32_Value != Value) // avoid flickering
				AdjustValue(s32_Value);
		}

		protected override void OnMouseUp(MouseEventArgs ev)
		{
			mi_RepeatTimer.Stop();

			if (mb_Dragging || mb_SmallDown || mb_SmallUp || mb_LargeUp || mb_LargeDown) 
				FireScrollEvent(ScrollEventType.EndScroll);

			mb_SmallDown = false;
			mb_SmallUp   = false;
			mb_LargeUp   = false;
			mb_LargeDown = false;
			mb_Dragging  = false;
			
			Invalidate(); // Release buttons
		}

		private void AdjustValue(int s32_NewValue)
		{
			if (mb_Dragging)  Value  = s32_NewValue;
			if (mb_SmallUp)   Value += SmallChange;
			if (mb_SmallDown) Value -= SmallChange;
			if (mb_LargeUp)   Value += LargeChange;
			if (mb_LargeDown) Value -= LargeChange;

			if (mb_Dragging)  FireScrollEvent(ScrollEventType.ThumbTrack);
			if (mb_SmallUp)   FireScrollEvent(ScrollEventType.SmallIncrement);
			if (mb_SmallDown) FireScrollEvent(ScrollEventType.SmallDecrement);
			if (mb_LargeUp)   FireScrollEvent(ScrollEventType.LargeIncrement);
			if (mb_LargeDown) FireScrollEvent(ScrollEventType.LargeDecrement);
		}

		/// <summary>
		/// Is fired while the mouse button is down
		/// </summary>
		private void OnRepeatTimer(object sender, EventArgs e)
		{
			mi_RepeatTimer.Stop(); // IMPORTANT !!
			
			AdjustValue(0);

			if (Value <= Minimum || Value >= Maximum)
				return;

			mi_RepeatTimer.Interval = ms32_RepeatDelay;
			mi_RepeatTimer.Start(); // RESTART here!!
		}

		#endregion
	}
}
