/*
Video Scroller for Cuttermaran
Written by Elm (http://netcult.ch/elmue)
Date: 15-10-2006

----------------------------------------------------------------------------------
Using these conventions results in better readable code and less coding errors !
----------------------------------------------------------------------------------

     cName  for generic class definitions
     CName  for MFC     class definitions
     tName  for type    definitions
     eName  for enum    definitions
     kName  for struct  definitions

    e_Name  for enum variables
    E_Name  for enum constant values

    i_Name  for instances of classes
    h_Name  for handles

    T_Name  for Templates
    t_Name  for TCHAR or LPTSTR

    s_Name  for strings
   bs_Name  for BSTR
    f_Name  for function pointers
    k_Name  for contructs (struct)

    b_Name  bool,BOOL 1 Bit

   s8_Name    signed  8 Bit (char)
  s16_Name    signed 16 Bit (SHORT, WCHAR)
  s32_Name    signed 32 Bit (LONG, int)
  s64_Name    signed 64 Bit (LONGLONG)

   u8_Name  unsigned  8 Bit (BYTE)
  u16_Name  unsigned 16 bit (WORD)
  u32_Name  unsigned 32 Bit (DWORD, UINT)
  u64_Name  unsigned 64 Bit (ULONGLONG)

    d_Name  for double

  ----------------

    m_Name  for member variables of a class (e.g. ms32_Name for int member variable)
    g_Name  for global variables            (e.g. gu16_Name for global WORD)
    p_Name  for pointer                     (e.g.   ps_Name  *pointer to string)
   pp_Name  for pointer to pointer          (e.g.  ppd_Name **pointer to double)

*************************************************************************************/


using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;


namespace GUI.MainForm.VideoScroller
{
	/// <summary>
	/// A horizontal graphical slider which can be used play a video in different speeds
	/// </summary>
	public class cShuttleBarH : Control
	{
		// Fired when the shutteling starts or ends
		public  delegate void StartStopDelegate(bool b_Active);
		public  event         StartStopDelegate StartStop;

		private int                       ms32_EndValue    = 64;
		private int                       ms32_PtrPos      = 0;
		private int                       ms32_Value       = 0;
		private int                       ms32_LastPtr     = 0;
		private bool                        mb_Dragging    = false;
		private bool                        mb_FireEvent   = false;
		private double                    mdbl_FactorGui   = 0;
		private double                    mdbl_FactorTimer = 0;
		private System.Windows.Forms.Timer  mi_RepeatTimer = null;
		private Bitmap                      mi_BackgrBmp   = null;
		private int                       ms32_ScaleDX     = 0;
		private int                       ms32_Interval    = 60;
		private int                       ms32_TimerMax    = 60;

		#region Construction

		// Constructor
		public cShuttleBarH()
		{
			this.Name = "cShuttleBarH";
			this.Text = "cShuttleBarH"; // invisible
			// The background color of the shuttlebar is a little brighter than the dialog background
			this.BackColor = ControlPaint.Light(SystemColors.Control);

			mi_RepeatTimer       = new System.Windows.Forms.Timer();
			mi_RepeatTimer.Tick += new EventHandler(OnRepeatTimer);
		}

		protected override void OnCreateControl()
		{
			// The height must always be odd, because we need a vertical middle
			Height = Height | 1;

			base.OnCreateControl();
			// The following makes the control draw its background
			// in WM_PAINT instead of WM_ERASEBACKGROUND
			SetStyle(ControlStyles.UserPaint,            true);
			SetStyle(ControlStyles.AllPaintingInWmPaint, true);
			// This avaoids flickering of the slider when it moves
			SetStyle(ControlStyles.DoubleBuffer,         true);
		}

		#endregion

		/// <summary>
		/// Sets the ShuttleBar in position zero and stops firing events
		/// </summary>
		public void Reset(bool b_Invalidate)
		{
			// Do NOT reset the shuttlebar while the mouse is down!
			// This would steel the shuttle slider while the user is using it
			if (mb_Dragging)
				return;

			bool b_FireOld = mb_FireEvent;

			ms32_PtrPos  = Width / 2; // set ALWAYS (required when resizing)!!
			ms32_Value   = 0;
			mb_Dragging  = false;
			mb_FireEvent = false;
			mi_RepeatTimer.Stop();

			if (StartStop != null && b_FireOld) 
				StartStop(false);

			if (b_Invalidate || b_FireOld) 
				Invalidate();
		}

		/// <summary>
		/// Is fired while the user drags the shuttle slider
		/// </summary>
		private void OnRepeatTimer(object sender, EventArgs e)
		{
			mi_RepeatTimer.Stop();  // IMPORTANT !!
			FireShuttleEvent();     // This may take a while...
			mi_RepeatTimer.Interval = ms32_Interval;
			mi_RepeatTimer.Start(); // RESTART here!!
		}

		#region Properties & Events

		public event ScrollEventHandler Shuttle = null;

		private void FireShuttleEvent()
		{
			if (Shuttle != null && Value != 0) 
				Shuttle(this, new ScrollEventArgs(ScrollEventType.ThumbTrack, Value));
		}

		/// <summary>
		/// This sets the maximum and minimum value which is reached when the user
		/// drags the slider to the leftmost / rightmost end of the control
		/// Example if you want the video to be played with maximum 32 fold speed 
		/// backwards and forward, set 32 here
		/// </summary>
		public int ShuttleEnd
		{
			get { return ms32_EndValue;  }
			set 
			{ 
				ms32_EndValue = Math.Max(2, value);
				mi_BackgrBmp  = null;
			}
		}

		/// <summary>
		/// returns the current position of the Shuttle slider
		/// the value is between -EndValue and +EndValue
		/// The Shuttle control uses a parabolic curve to calculate screen position to value
		/// so the small values can be fine-tuned and the 
		/// The value is 0 if the user released the slider
		/// </summary>
		public int Value
		{
			get { return ms32_Value; }
		}

		/// <summary>
		/// Set: sets the maximum timer interval which is valid since the fist line 
		/// of the scale (Value >= 2)
		/// Get: returns the CURRENT timer interval
		/// </summary>
		public int TimerInterval
		{
			set { ms32_TimerMax = value; }
			get { return ms32_Interval; }
		}
		
		#endregion

		#region Painting

		protected override void OnSizeChanged(EventArgs ev)
		{
			base.OnSizeChanged (ev);
			mi_BackgrBmp = null;
			Reset(true);
		}

		protected override void OnEnabledChanged(EventArgs ev)
		{
			base.OnEnabledChanged (ev);
			mi_BackgrBmp = null;
			Reset(true);
		}

		/// <summary>
		/// Only once create a bitmap which contains the background of the shuttlebar
		/// to increase drawing speed
		/// </summary>
		protected void CreateBackgroundBitmap(Graphics i_GraphDC)
		{
			if (mi_BackgrBmp != null)
				return;

			// ----- calculate constants -----

			int s32_PtrWidth = Math.Max(1, Width / 2 - 18);
			mdbl_FactorGui   = Math.Log(ShuttleEnd, 2) / s32_PtrWidth;
			// The distance of the lines on the scale in pixels
			ms32_ScaleDX     = (int)(1.0 / mdbl_FactorGui); // Log(2,2) == 1.0
			// Used for calculating the timer interval
			mdbl_FactorTimer = (double) ms32_ScaleDX / 30;

			// ----- create image -----

			mi_BackgrBmp = new Bitmap(Width -16, this.Height, i_GraphDC);

			Graphics i_GraphBmp = Graphics.FromImage(mi_BackgrBmp);

			// fill with BackgroundColor
			Brush i_Brush = new SolidBrush(this.BackColor);
			i_GraphBmp.FillRectangle(i_Brush, 0, 0, Width-16, this.Height);
			i_Brush.Dispose();

			Pen i_Pen = new Pen(Enabled ? Color.DarkSlateGray : SystemColors.GrayText);
			// a horizontal line
			i_GraphBmp.DrawLine(i_Pen, 0, Height/2, Width-8, Height/2);

			int s32_Center = Width / 2 - 8;
			// draw a line for every power of two value (at 0, 2, 4, 8, 16, 32, etc..)
			for (int X=0; X<s32_Center; X+=ms32_ScaleDX)
			{
				i_GraphBmp.DrawLine(i_Pen, s32_Center+X, Height/3, s32_Center+X, Height*2/3);
				i_GraphBmp.DrawLine(i_Pen, s32_Center-X, Height/3, s32_Center-X, Height*2/3);
			}

			// draw the center line a little longer and thicker than the others
			i_GraphBmp.DrawLine(i_Pen, s32_Center-1, 2, s32_Center-1, Height-3);
			i_GraphBmp.DrawLine(i_Pen, s32_Center,   2, s32_Center,   Height-3);
			i_GraphBmp.DrawLine(i_Pen, s32_Center+1, 2, s32_Center+1, Height-3);
			i_Pen.Dispose();
		}

		protected override void OnPaintBackground(PaintEventArgs ev)
		{
			CreateBackgroundBitmap(ev.Graphics);
			ev.Graphics.DrawImage (mi_BackgrBmp, 8, 0, Width -16, this.Height);
		}

		protected override void OnPaint(PaintEventArgs ev)
		{
			// The two small buttons at the left and right end of the control
			ControlPaint.DrawButton(ev.Graphics, 0,       0, 8, Height, ButtonState.Normal);
			ControlPaint.DrawButton(ev.Graphics, Width-8, 0, 8, Height, ButtonState.Normal);

			// The shuttle button
			ButtonState i_State = Enabled ? ButtonState.Normal : ButtonState.Flat;
			ControlPaint.DrawButton(ev.Graphics, ms32_PtrPos-10, 0, 20, Height, i_State);

			Color c_Pen = mb_FireEvent ? Color.Red : Color.Black;
			if (!Enabled) c_Pen = SystemColors.GrayText;
			Pen i_Pen = new Pen(c_Pen);

			int Y = Height / 2;
			ev.Graphics.DrawLine(i_Pen, ms32_PtrPos-7, Y, ms32_PtrPos+5, Y);

			// draw the two arrows on the slider button
			for (int i=1; i<4; i++)
			{
				ev.Graphics.DrawLine(i_Pen, ms32_PtrPos-7+i, Y+i, ms32_PtrPos-7+i, Y-i);
				ev.Graphics.DrawLine(i_Pen, ms32_PtrPos+5-i, Y+i, ms32_PtrPos+5-i, Y-i);
			}
			i_Pen.Dispose();
		}

		#endregion

		#region Mouse Events

		protected override void OnMouseDown(MouseEventArgs ev)
		{
			// Click on the slider button
			if (ev.X > ms32_PtrPos -10 && ev.X < ms32_PtrPos +10)
			{
				mb_Dragging  = true;
				mb_FireEvent = true;
				ms32_LastPtr = ms32_PtrPos;
				if (StartStop != null) StartStop(true);
				mi_RepeatTimer.Interval = ms32_Interval;
				mi_RepeatTimer.Start();
			}
			else Reset(true); // Click on the shaft
		}

		protected override void OnMouseUp(MouseEventArgs ev)
		{
			mb_Dragging = false;

			// The user clicked the slider without moving it -> stop firing
			if (ms32_LastPtr == ms32_PtrPos) Reset(true);
		}

		protected override void OnMouseMove(MouseEventArgs ev)
		{
			if (!mb_Dragging)
				return;
			
			ms32_PtrPos = Math.Max(18, Math.Min(Width -18, ev.X));

			// Calculate the Value from the slider position
			// This is a power of 2 curve
			double dbl_Pos = ms32_PtrPos - Width / 2;
			double dbl_Val = Math.Pow(2, Math.Abs(dbl_Pos) * mdbl_FactorGui);
			ms32_Value = (int)(dbl_Val + 0.5) * Math.Sign(dbl_Pos);

			// No we calculate the timer interval. The timer is slowed down
			// in the center of the control. From the first scale line on (2 fold speed)
			// at position ms32_ScaleDX the timer interval is always 60 ms.
			// But for speeds slower than 2 we calculate a parabolic curve which
			// goes through the coordinates (0,0) and (30,900) where the zero point of the
			// parabola is at ms32_ScaleDX and the highest point is in the shuttle's center
			// So reaching the center the timer slowes down from 0+60 ms to 900+60 ms
			int s32_Center = Width / 2;
			int s32_Dist   = Math.Abs(s32_Center - ms32_PtrPos);

			if (s32_Dist < ms32_ScaleDX)
			{
				double dbl_ParabX = (double)(ms32_ScaleDX - s32_Dist) / mdbl_FactorTimer;

				ms32_Interval = (int) dbl_ParabX;
				double dbl_ParabY = dbl_ParabX * dbl_ParabX;
				ms32_Interval = ms32_TimerMax + (int) dbl_ParabY;
			}
			else ms32_Interval = ms32_TimerMax;

			// This will improve the reaction delay on slider movement
			// if the timer is currently set to 900 ms -> change it immediately
			if (ms32_Interval + 100 < mi_RepeatTimer.Interval)
				OnRepeatTimer(null, null);

			Invalidate();
		}

		#endregion
	}
}
