/*
Video Scroller for Cuttermaran
Written by Elm (http://netcult.ch/elmue)
Date: 15-10-2006

----------------------------------------------------------------------------------
Using these conventions results in better readable code and less coding errors !
----------------------------------------------------------------------------------

     cName  for generic class definitions
     CName  for MFC     class definitions
     tName  for type    definitions
     eName  for enum    definitions
     kName  for struct  definitions

    e_Name  for enum variables
    E_Name  for enum constant values

    i_Name  for instances of classes
    h_Name  for handles

    T_Name  for Templates
    t_Name  for TCHAR or LPTSTR

    s_Name  for strings
   bs_Name  for BSTR
    f_Name  for function pointers
    k_Name  for contructs (struct)

    b_Name  bool,BOOL 1 Bit

   s8_Name    signed  8 Bit (char)
  s16_Name    signed 16 Bit (SHORT, WCHAR)
  s32_Name    signed 32 Bit (LONG, int)
  s64_Name    signed 64 Bit (LONGLONG)

   u8_Name  unsigned  8 Bit (BYTE)
  u16_Name  unsigned 16 bit (WORD)
  u32_Name  unsigned 32 Bit (DWORD, UINT)
  u64_Name  unsigned 64 Bit (ULONGLONG)

    d_Name  for double

  ----------------

    m_Name  for member variables of a class (e.g. ms32_Name for int member variable)
    g_Name  for global variables            (e.g. gu16_Name for global WORD)
    p_Name  for pointer                     (e.g.   ps_Name  *pointer to string)
   pp_Name  for pointer to pointer          (e.g.  ppd_Name **pointer to double)

*************************************************************************************/


using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace GUI.MainForm.VideoScroller
{
	/// <summary>
	/// Summary description for cVideoScroller.
	/// </summary>
	public class cVideoScroller : Control // do not derive from UserControl as GUI designer is useless for this control
	{
		protected cBackScrollBarH mi_ScrollBar;
		protected cShuttleBarH    mi_ShuttleBar;

		#region Construction

		public cVideoScroller()
		{
			mi_ScrollBar  = new cBackScrollBarH();
			mi_ShuttleBar = new cShuttleBarH();

			SuspendLayout();
			Controls.Add(mi_ScrollBar);
			Controls.Add(mi_ShuttleBar);
			Name = "cVideoScroller";
			Text = "cVideoScroller";
			Size = new System.Drawing.Size(200, 36);
			ResumeLayout(false);

			mi_ScrollBar.Scroll       += new ScrollEventHandler(OnScroll);
			mi_ScrollBar.ValueChanged += new EventHandler(OnValueChanged);
			mi_ShuttleBar.Shuttle     += new ScrollEventHandler(OnShuttle);
			mi_ShuttleBar.StartStop   += new cShuttleBarH.StartStopDelegate(OnShuttleStartStop);
		}

		/// <summary>
		/// positioning of the both subcontrols
		/// </summary>
		protected override void OnCreateControl()
		{
			// Windows sets the height of a HScrollBar to 17 pixels on 96 DPI, and 21 pixels on 120 DPI etc..
			// It is not required to adapt the height of the scrollbar HERE, because
			// this is done by the Magic Library, which calculates the whole GUI

			// The height for ShuttleBar must always be odd, because we need a vertical middle 
			const int s32_Height = 17; // Windows scrollbar height on 96 DPI

			Size k_Size = new System.Drawing.Size(this.Width, s32_Height);
			mi_ScrollBar. Size = k_Size;
			mi_ShuttleBar.Size = k_Size; 

			mi_ScrollBar. Location = new System.Drawing.Point(0, 0);
			mi_ShuttleBar.Location = new System.Drawing.Point(0, s32_Height + 2);

			this.Height = 2 * s32_Height + 2;

			base.OnCreateControl();
		}

		#endregion

		#region Properties & Events

		/// <summary>
		/// This event will ONLY be fired when the USER moved the scrollbar
		/// or when the ShuttleBar moved the scrollbar 
		/// It will NOT be fired when program code modified Value!
		/// </summary>
		public event ScrollEventHandler Scroll = null;

		/// <summary>
		/// Called on every move of the scrollbar no matter if the user
		/// initiated the movement, the Value property was set or the ShuttleBar
		/// has moved the scrollbar
		/// </summary>
		public event EventHandler ValueChanged = null;

		/// <summary>
		/// This event is fired when a shuttle action is started or stopped
		/// </summary>
		public event cShuttleBarH.StartStopDelegate ShuttleStartStop = null;

		/// <summary>
		/// sets or gets the minimum value of the scrollbar
		/// </summary>
		public int Minimum
		{
			set { mi_ScrollBar.Minimum = value; }
			get { return mi_ScrollBar.Minimum;  }
		}
		/// <summary>
		/// sets or gets the maximum value of the scrollbar
		/// </summary>
		public int Maximum
		{
			set { mi_ScrollBar.Maximum = value; }
			get { return mi_ScrollBar.Maximum;  }
		}

		/// <summary>
		/// Sets the maximum and minimum value of the ShuttleBar
		/// </summary>
		public int ShuttleEnd
		{
			set { mi_ShuttleBar.ShuttleEnd = value; }
		}

		/// <summary>
		/// returns the current Shuttle speed which is between -ShuttleEnd and +ShuttlEnd
		/// returns 0 if the ShuttleBar is not in use
		/// </summary>
		public int ShuttleSpeed
		{
			get { return mi_ShuttleBar.Value;  }
		}

		/// <summary>
		/// sets or gets the value by which the scrollbar is moved if
		/// the user clicks the scrollbar shaft
		/// Unlike HScrollBar this control will ALWAYS return the Maximum value
		/// when the scrollbar is at its end (and not Maximum - LargeChange)
		/// </summary>
		public int LargeChange
		{
			set { mi_ScrollBar.LargeChange = value; }
			get { return mi_ScrollBar.LargeChange;  }
		}
		/// <summary>
		/// sets or gets the value by which the scrollbar is moved if
		/// the user clicks the buttons at the scrollbar's end
		/// </summary>
		public int SmallChange
		{
			set { mi_ScrollBar.SmallChange = value; }
			get { return mi_ScrollBar.SmallChange;  }
		}

		/// <summary>
		/// sets or gets the value of the scrollbar (Always between Minimum and Maximum)
		/// </summary>
		public int Value
		{
			set { mi_ScrollBar.Value = value; }
			get { return mi_ScrollBar.Value;  }
		}

		/// <summary>
		/// sets the autorepeat delay of scrollbar and ShuttleBar
		/// </summary>
		public int RepeatDelay
		{
			set 
			{ 
				mi_ScrollBar. RepeatDelay   = value; 
				mi_ShuttleBar.TimerInterval = value; 
			}
		}

		#endregion

		/// <summary>
		/// Resets the ShuttleBar to its center position
		/// </summary>
		public void ResetShuttle(bool b_Invalidate)
		{
			mi_ShuttleBar.Reset(b_Invalidate);
		}

		/// <summary>
		/// Store a block of background color and its position and size in the Scrollbar
		/// Has to be called AFTER setting Minimum and Maximum !
		/// After adding the last color you must call Invalidate()
		/// </summary>
		public void AddBackgroundColor(int s32_StartPos, int s32_EndPos, Color i_Color, bool b_Append)
		{
			mi_ScrollBar.AddBackgroundColor(s32_StartPos, s32_EndPos, i_Color, b_Append);
		}

		/// <summary>
		/// Removes all blocks of defined background colors for the scrollbar
		/// </summary>
		public void ResetBackgroundColors()
		{
			mi_ScrollBar.ResetBackgroundColors();
		}

		#region Event handler

		/// <summary>
		/// Called when the user moved the scrollbar or Value was modified by C# code
		/// </summary>
		protected void OnValueChanged(object sender, EventArgs ev)
		{
			// pass the event through
			if (ValueChanged != null) ValueChanged(this, ev);
		}

		/// <summary>
		/// Only called when the USER moved the scrollbar
		/// or the Minimum or Maximum was reached
		/// </summary>
		protected void OnScroll(object sender, ScrollEventArgs ev)
		{
			if (Scroll != null ) 
				Scroll(this, ev);

			// Stop the ShuttleBar sending events
			// false avoids useless Invalidates
			mi_ShuttleBar.Reset(false);
		}

		/// <summary>
		/// Receives periodic events while the Shuttle slider in not in the center position
		/// </summary>
		protected void OnShuttle(object sender, ScrollEventArgs ev)
		{
			// Add the shuttle value to the current scrollbar's  position
			mi_ScrollBar.Value += ev.NewValue;

			if (Scroll != null ) 
				Scroll(this, new ScrollEventArgs(ScrollEventType.ThumbTrack, mi_ScrollBar.Value));
		}

		/// <summary>
		/// Called when a shuttle action starts or stops
		/// </summary>
		private void OnShuttleStartStop(bool b_Active)
		{
			if (ShuttleStartStop != null) 
				ShuttleStartStop(b_Active);
		}

		protected override void OnInvalidated(InvalidateEventArgs e)
		{
			base.OnInvalidated(e);
			mi_ScrollBar. Invalidate();
			mi_ShuttleBar.Invalidate();
		}

		protected override void OnSizeChanged(EventArgs e)
		{
			base.OnSizeChanged(e);
			mi_ScrollBar. Width = this.Width;
			mi_ShuttleBar.Width = this.Width;
		}


		#endregion

	}
}
