using System;
using System.IO;
using System.Configuration;
using System.Windows.Forms;

using Coremoran.AVStreams;
using Configuration;
using StringResources;
using GUI.data;
using GUI.MainForm;

namespace GUI
{
	/// <summary>
	/// Summary description for Mpeg12Encoder.
	/// </summary>
	public class Mpeg12Encoder : IVideoEncoder
	{
		private AviWriter aw=null;
		private Sequence startSequence;
		private Picture startPicture;
		private string destFile;

		public Mpeg12Encoder()
		{
		}

		private void OpenAVI(Mpeg12Video videoStream,int startFrame)
		{
			destFile=Path.Combine(Settings.CurrentSettings.TempDir,string.Format("X{0}",Guid.NewGuid()));
			ConversionType ct=Settings.CurrentSettings.TempAVIColorEncoding;
			startPicture=((Mpeg12Video)videoStream).GetPictureSecure(startFrame);
			startSequence=(Sequence)videoStream.GetPrevMpeg12Object(typeof(Sequence),startPicture);
			// Quantisierung Matrizen holen, falls vorhanden
			videoStream.EnsureExtendedData(startSequence);

			// Das Video ffnen
			using (VideoDecoder vd=VideoDecoder.Create(videoStream,ct))
			{
				// und positionieren
				vd.GoToFrame(startFrame);

				FrameInfoStruct fis=vd.CurrentFrameInfo;

				// AVI erzeugen
				aw=AviWriter.Create(Path.ChangeExtension(destFile,".avi"),videoStream.FramesPerSecond,fis.width,fis.height,ct==ConversionType.RGB24);
			}
		}

		#region IVideoEncoder Members

		public void AddFrames(BasicVideoStream bas,int startFrame,int endFrame,IProcessInformation pi)
		{
			if (aw==null)
				OpenAVI((Mpeg12Video)bas,startFrame);
			// Das Video ffnen
			ConversionType ct=Settings.CurrentSettings.TempAVIColorEncoding;
			using (VideoDecoder vd=VideoDecoder.Create(bas,ct))
			{
				// und positionieren
				vd.GoToFrame(startFrame);
				FrameInfoStruct fis=vd.CurrentFrameInfo;
				// Puffer fr das YUV Bild anlegen
				byte [] buffer=new byte [fis.size+2*fis.chroma_size];
				
				int percent = 0;
				pi.PercentageComplete = 0m;
				for (int i=startFrame;i<=endFrame;++i)
				{
					if (!BasicPacketStream.ManageLoop())
						return;

					// Don't display progress if this function is called from Mpeg12Video.Cut()
					// Display progress only if called from Mpeg12Video.TransferMpegObjects()
					if (endFrame - startFrame > 20)
					{
						int progress = 100 * (i - startFrame) / (endFrame - startFrame);
						if (progress != percent)
						{
							percent = progress;
							pi.PercentageComplete = (decimal)percent;
						}
					}

					// Bild holen
					vd.GetCurrentFrameData(buffer);
					// ..und wegschreiben
					aw.AddFrame(buffer);

					// nchstes Bild
					vd.MoveNextFrame();
				}
			}
		}

		public bool HasFrames
		{
			get
			{
				return aw!=null;
			}
		}

		public string Encode()
		{
			if (HasFrames)
			{
				try
				{
					// Fertig
					aw.Close();

					// Parameter aus Sequence bestcken
					EncodingData enc=new EncodingData();
					EncodingData.SourceRow source=enc.Source.NewSourceRow();
					source.AVIFile=Path.ChangeExtension(destFile,".avi");
					source.ResultMPGFile=Path.ChangeExtension(destFile,".mpv");
					source.IsMpeg2=startSequence.sequence_extension_exist;
					source.Width=startSequence.horizontal_size_value;
					source.Height=startSequence.vertical_size_value;
					source.FrameRate=startSequence.frame_rate_code;
					source.AspectRatio=startSequence.aspect_ratio_information;

					// Bitrate
					if (Settings.CurrentSettings.Encode_Bitrate>0)
						source.BitRate=1000*Settings.CurrentSettings.Encode_Bitrate; // in bps
					else
						source.BitRate=400*startSequence.bit_rate_value;  // in bps
					if (Settings.CurrentSettings.CreateDVDCompliantStream) // ggf. Bitrate limitieren
					{
						long complianceMaxBitrate=long.Parse(ConfigurationSettings.AppSettings["ComplianceMaxBitrate"]);
						if (source.BitRate>complianceMaxBitrate*1000)
							source.BitRate=complianceMaxBitrate*1000;
					}
					source.VBVBuffer=startSequence.vbv_buffer_size_value*2048; // in Bytes
					source.VideoFormat=startSequence.VideoFormat;
					source.IsProgressive=startSequence.progressive_sequence;

					// aus Picture
					source.intra_dc_precision=startPicture.intra_dc_precision;
					source.IsTopFieldFirst=startPicture.top_field_first;
					source.encode_fields=startPicture.second_field!=null;

					// Quantisierung
					if (startSequence.intra_quantizer_matrix!=null)
						source.intra_quantizer_matrix=startSequence.intra_quantizer_matrix;
					if (startSequence.non_intra_quantizer_matrix!=null)
						source.non_intra_quantizer_matrix=startSequence.non_intra_quantizer_matrix;
					// Darf Encoding Provider Meldungen ausgeben?
					source.Unattended=false;
					// Die aktuelle Sprache mitteilen
					source.UICulture=System.Threading.Thread.CurrentThread.CurrentUICulture.Name;

					enc.Source.AddSourceRow(source);

					int encoderResult=GUIHelper.CallExternalTool(Settings.CurrentSettings.UsedEncoderInfo,enc,Path.ChangeExtension(destFile,".xml"));

					string [] del=Directory.GetFiles(Path.GetDirectoryName(destFile),Path.GetFileNameWithoutExtension(destFile)+".*");
					foreach (string filename in del)
						if (Path.GetExtension(filename).ToLower()!=".mpv")
							File.Delete(filename);

					if (encoderResult==0)
					{
						if (File.Exists(source.ResultMPGFile))
						{
							// Sometimes the decoder (Quenc) writes a file of zero bytes length -> exception
							FileInfo i_Info = new FileInfo(source.ResultMPGFile);
							if (i_Info.Length > 0)
								return source.ResultMPGFile;
						}
						throw new ApplicationException(Resource.CurrentResource.FormatMessage(Resource.NO_ENCODER_RESULT,source.ResultMPGFile));
					}
					else
						throw new ApplicationException(Resource.CurrentResource[Resource.INTERNAL_ENCODER_ERROR]);
				}
				finally
				{
					aw=null; // Release
				}
			}
			else
				return string.Empty;
		}

		#endregion
	}
}
