/*
A label that can trim the text
Written by Elm (http://netcult.ch/elmue)
Date: 1-12-2006

----------------------------------------------------------------------------------
Using these conventions results in better readable code and less coding errors !
----------------------------------------------------------------------------------

     cName  for generic class definitions
     CName  for MFC     class definitions
     tName  for type    definitions
     eName  for enum    definitions
     kName  for struct  definitions

    e_Name  for enum variables
    E_Name  for enum constant values

    i_Name  for instances of classes
    h_Name  for handles

    T_Name  for Templates
    t_Name  for TCHAR or LPTSTR

    s_Name  for strings
   bs_Name  for BSTR
    f_Name  for function pointers
    k_Name  for contructs (struct)

    b_Name  bool,BOOL 1 Bit

   s8_Name    signed  8 Bit (char)
  s16_Name    signed 16 Bit (SHORT, WCHAR)
  s32_Name    signed 32 Bit (LONG, int)
  s64_Name    signed 64 Bit (LONGLONG)

   u8_Name  unsigned  8 Bit (BYTE)
  u16_Name  unsigned 16 bit (WORD)
  u32_Name  unsigned 32 Bit (DWORD, UINT)
  u64_Name  unsigned 64 Bit (ULONGLONG)

    d_Name  for double

  ----------------

    m_Name  for member variables of a class (e.g. ms32_Name for int member variable)
    g_Name  for global variables            (e.g. gu16_Name for global WORD)
    p_Name  for pointer                     (e.g.   ps_Name  *pointer to string)
   pp_Name  for pointer to pointer          (e.g.  ppd_Name **pointer to double)

*************************************************************************************/

using System;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Text;

namespace GUI
{
	/// <summary>
	/// A label that allows to draw text using a trimming mode.
	/// (Images are not supported)
	/// Why does !STUPID! Microsoft not allow to set this in a Forms.Label ???????
	/// And why are all the required functions in Forms.Label private 
	/// so we have to re-invent the wheel here ?????
	/// </summary>
	public class TrimmingLabel : Label
	{
		protected StringTrimming mi_Trimming = StringTrimming.None;

		/// <summary>
		/// Allows to set a user defined trimming
		/// Trimmings like EllipsisPath will only work correctly if the label
		/// displays ONE and only ONE line of text.
		/// The height must not be greater than 14 pixels (for the default font)
		/// </summary>
		public StringTrimming Trimming
		{
			set { mi_Trimming = value; }
		}

		/// <summary>
		/// Builds a StringAlignment which would be used by a Windows.Forms.Label
		/// </summary>
		protected StringFormat GetStringFormat()
		{
			StringFormat i_Format = new StringFormat();
			switch (TextAlign)
			{
				case ContentAlignment.BottomLeft: 
					i_Format.Alignment     = StringAlignment.Near;
					i_Format.LineAlignment = StringAlignment.Far;
					break;
				case ContentAlignment.BottomCenter: 
					i_Format.Alignment     = StringAlignment.Center;
					i_Format.LineAlignment = StringAlignment.Far;
					break;
				case ContentAlignment.BottomRight:
					i_Format.Alignment     = StringAlignment.Far;
					i_Format.LineAlignment = StringAlignment.Far;
					break;
				case ContentAlignment.MiddleLeft: 
					i_Format.Alignment     = StringAlignment.Near;
					i_Format.LineAlignment = StringAlignment.Center;
					break;
				case ContentAlignment.MiddleCenter: 
					i_Format.Alignment     = StringAlignment.Center;
					i_Format.LineAlignment = StringAlignment.Center;
					break;
				case ContentAlignment.MiddleRight:
					i_Format.Alignment     = StringAlignment.Far;
					i_Format.LineAlignment = StringAlignment.Center;
					break;
				case ContentAlignment.TopLeft: 
					i_Format.Alignment     = StringAlignment.Near;
					i_Format.LineAlignment = StringAlignment.Near;
					break;
				case ContentAlignment.TopCenter: 
					i_Format.Alignment     = StringAlignment.Center;
					i_Format.LineAlignment = StringAlignment.Near;
					break;
				case ContentAlignment.TopRight:
					i_Format.Alignment     = StringAlignment.Far;
					i_Format.LineAlignment = StringAlignment.Near;
					break;
			}

			if (RightToLeft == RightToLeft.Yes)
				i_Format.FormatFlags |= StringFormatFlags.DirectionRightToLeft;

			if (!UseMnemonic)
				i_Format.HotkeyPrefix = HotkeyPrefix.None;
			else if (ShowKeyboardCues)
				i_Format.HotkeyPrefix = HotkeyPrefix.Show;
			else
				i_Format.HotkeyPrefix = HotkeyPrefix.Hide;

			if (AutoSize)
				i_Format.FormatFlags |= StringFormatFlags.MeasureTrailingSpaces;

			i_Format.Trimming = mi_Trimming;
			return i_Format;
		}

		protected override void OnPaint(PaintEventArgs ev)
		{
			StringFormat i_Format = GetStringFormat();
			if (Enabled)
			{
				Brush i_Brush = new SolidBrush(ForeColor);
				ev.Graphics.DrawString(Text, Font, i_Brush, (RectangleF)ClientRectangle, i_Format);
				i_Brush.Dispose();
			}
			else
			{
				ControlPaint.DrawStringDisabled(ev.Graphics, Text, Font, SystemColors.Control, (RectangleF)ClientRectangle, i_Format);
			}
		}
	}
}
