/*
 * Cuttermaran - The Mpeg Cutter
 * 
 * Copyright (C) 2003-2005  Tobias Arnold (cuttermaran@gmx.net)
 * 
 * This file is part of cuttermaran, a free MPEG cut program.
 * See http://www.cuttermaran.de for updates.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

using System;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;
using Configuration;
using Coremoran.AVStreams;
using StringResources;
using Coremoran.Common;

namespace GUI
{
	/// <summary>
	/// Summary description for 
	/// </summary>
	public class libmpeg2 : VideoDecoder, IDisposable
	{
		#region LIBMPEG2 Functions
		[DllImport("libmpeg2.dll",EntryPoint="libMpeg2_Open")]
		private static extern int Open(string keyName,string filename,ConversionType ct);

		[DllImport("libmpeg2.dll",EntryPoint="libMpeg2_Close")]
		private static extern int Close(string keyName);

		[DllImport("libmpeg2.dll",EntryPoint="libMpeg2_GetCurrentFrame")]
		[return: MarshalAs(UnmanagedType.LPStruct)]
		private static extern FrameInfoStruct GetCurrentFrame(string keyName);

		[DllImport("libmpeg2.dll",EntryPoint="libMpeg2_GetCurrentFrameData")]
		private static extern int GetCurrentFrameBytes(string keyName,[Out,MarshalAs(UnmanagedType.LPArray)] byte [] data);

		[DllImport("libmpeg2.dll",EntryPoint="libMpeg2_GetCurrentFrameData")]
		private static extern int GetCurrentFrameData(string keyName,IntPtr data);

		[DllImport("libmpeg2.dll",EntryPoint="libMpeg2_NextFrame")]
		private static extern int NextFrame(string keyName);

		[DllImport("libmpeg2.dll",EntryPoint="libMpeg2_SkipFrames")]
		private static extern int SkipFrames(string keyName,int count);

		[DllImport("libmpeg2.dll",EntryPoint="libMpeg2_Seek")]
		private static extern int Seek(string keyName,long sequenz_pos,int full_reset);
		#endregion

		private string internalInstanceName=Guid.NewGuid().ToString();

		public libmpeg2(Mpeg12Video videoInfo,ConversionType ct) : base(videoInfo,ct)
		{
		}

		public override void GoToFrame(int Position)
		{
			lock (this)
			{
				int IFrame;
				// Vor dem Zielpunkt liegendes IFrame suchen
				if ((FrameType)videoStream.GetPicture(Position).picture_coding_type==FrameType.IFrame)
					IFrame=Position;
				else
					IFrame=videoStream.CalculatePrevFramePos(Position,FrameType.IFrame);

				// Sequence Header vor dem I-Frame
				Mpeg12Object seq=videoStream.GetPrevMpeg12Object(typeof(Sequence),videoStream.GetPicture(IFrame));

				if (seq!=null)
				{
					// Full reset damit der SequenceHeader gelesen wird!
					Seek(internalInstanceName,seq.adress,1); // return !=1 konnte nicht positioniert werden

					// Soft reset falls, die Sequence nicht direkt zum GOP gehrt.
					Mpeg12Object gop1=videoStream.GetNextMpeg12Object(typeof(GOP),seq);
					Mpeg12Object gop2=videoStream.GetPrevMpeg12Object(typeof(GOP),videoStream.GetPicture(IFrame));
					if (gop1!=gop2)
						Seek(internalInstanceName,gop2.adress,0); // return !=1 konnte nicht positioniert werden

					// aufs I-Frame positionieren:
					for (FrameInfoStruct fis=GetCurrentFrame(internalInstanceName);fis!=null && fis.type!=1;fis=GetCurrentFrame(internalInstanceName))
						NextFrame(internalInstanceName);
				}
				else
				{
					// Workaround fr den Fall das der Stream nicht korrekt ist
					// und daher nicht mit einer Sequence, bzw. GOP anfngt.
					Seek(internalInstanceName,0,1);
				}
				// auf die gewnschte Position vorrcken
				SkipFrames(internalInstanceName,Position-IFrame);
				currentPos=Position;
			}
		}

		public override bool GetPicture(int Position,ref Bitmap bitm,bool silenceMode,out bool newBMPcreated)
		{
			newBMPcreated=false;
			lock (this)
			{
				try
				{
					if (Position!=currentPos)
					{
						if (Position>currentPos && Position-currentPos<10)
							SkipFrames(internalInstanceName,Position-currentPos);
						else
							GoToFrame(Position);
					}
					FrameInfoStruct fis;
					fis=GetCurrentFrame(internalInstanceName);
					if (fis==null)
					{
						if (!silenceMode)
							MessageBox.Show(Resource.CurrentResource.FormatMessage(Resource.LIBMPEG2_DISPLAY,Position),"libmpeg2");
						getPictureResult=false;
						return false;
					}
					// Neues Bitmap erzeugen wenn noch gar keins da, oder sich die Gre gendert hat.
					if (bitm==null || bitm.Width!=fis.width || bitm.Height!=fis.height)
					{
						// Unmanaged ressources freigeben
						if (bitm!=null)
							bitm.Dispose();

						bitm=new Bitmap(fis.width,fis.height,PixelFormat.Format24bppRgb);
						newBMPcreated=true;
					}
					BitmapData bmd=bitm.LockBits(new Rectangle(0,0,fis.width,fis.height),ImageLockMode.WriteOnly,PixelFormat.Format24bppRgb);
					try
					{
						GetCurrentFrameData(internalInstanceName,bmd.Scan0);
					}
					finally
					{
						bitm.UnlockBits(bmd);
					}
				}
				catch (Exception e)
				{
					if (!silenceMode)
						MessageBox.Show(e.Message);
					getPictureResult=false;
					return false;
				}
				currentPos=Position;
				getPictureResult=true;
				return true;
			}
		}

		protected override bool OpenVideo(ConversionType ct)
		{
			lock (this)
			{
				currentPos=0;
				return Open(internalInstanceName,videoStream.StreamName,ct)!=0;
			}
		}

		public override bool CloseVideo()
		{
			lock (this)
			{
				currentPos=int.MaxValue;
				return Close(internalInstanceName)!=0;
			}
		}

		public override bool MoveNextFrame()
		{
			lock (this)
			{
				++currentPos;
				return NextFrame(internalInstanceName)!=0;
			}
		}

		public override FrameInfoStruct CurrentFrameInfo
		{
			get
			{
				lock (this)
				{
					return GetCurrentFrame(internalInstanceName);
				}
			}
		}

		public override void GetCurrentFrameData(byte [] data)
		{
			lock (this)
			{
				GetCurrentFrameBytes(internalInstanceName,data);
			}
		}
		#region IDisposable Members

		public override void Dispose()
		{
			CloseVideo();
		}

		#endregion
	}
}
