using System;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;
using libmpeg2Net;
using Coremoran.AVStreams;
using Configuration;
using Coremoran.Common;
using StringResources;

namespace GUI
{
	/// <summary>
	/// Summary description for libmpeg2Decoder.
	/// </summary>
	public class libmpeg2Decoder : VideoDecoder
	{
		private libmpeg2Net.libmpeg2 mpegDecoder=new libmpeg2Net.libmpeg2();

		public libmpeg2Decoder(Mpeg12Video videoInfo,Configuration.ConversionType ct) : base(videoInfo,ct)
		{
		}

		protected override bool OpenVideo(Configuration.ConversionType ct)
		{
			currentPos=0;
			return mpegDecoder.Open(videoStream.StreamName,libmpeg2Net.ConversionType.RGB24)!=0;
		}

		public override bool CloseVideo()
		{
			currentPos=int.MaxValue;
			return mpegDecoder.Close()!=0;
		}

		public override bool MoveNextFrame()
		{
			++currentPos;
			return mpegDecoder.libMpeg2_NextFrame()!=0;
		}

		private FrameInfoStruct Tfi2Fis(TFrameInfo tfi)
		{
			FrameInfoStruct fs=new FrameInfoStruct();
			fs.height=tfi.Height;
			fs.size=tfi.Size;
			fs.width=tfi.Width;
			fs.chroma_height=tfi.ChromaHeight;
			fs.chroma_size=tfi.ChromaSize;
			fs.chroma_width=tfi.ChromaWidth;
			fs.type=tfi.Type;
//			fs.Y=tfi.HoleY();
//			fs.U=tfi.HoleU();
//			fs.V=tfi.HoleV();
			fs.Y=IntPtr.Zero;
			fs.U=IntPtr.Zero;
			fs.V=IntPtr.Zero;
			return fs;
		}

		public override FrameInfoStruct CurrentFrameInfo
		{
			get
			{
				return Tfi2Fis(mpegDecoder.GetCurrentFrame);
			}
		}

		public override void GetCurrentFrameData(byte[] data)
		{
			mpegDecoder.GetCurrentFrameData(data);
		}

		public override void Dispose()
		{
			CloseVideo();
		}

		public override void GoToFrame(int Position)
		{
			int IFrame;
			// Vor dem Zielpunkt liegendes IFrame suchen
			if ((FrameType)videoStream.GetPicture(Position).picture_coding_type==FrameType.IFrame)
				IFrame=Position;
			else
				IFrame=videoStream.CalculatePrevFramePos(Position,FrameType.IFrame);

			// Sequence Header vor dem I-Frame
			Mpeg12Object seq=videoStream.GetPrevMpeg12Object(typeof(Sequence),videoStream.GetPicture(IFrame));

			if (seq!=null)
			{
				// Full reset damit der SequenceHeader gelesen wird!
				mpegDecoder.Seek(seq.adress,true); // return !=1 konnte nicht positioniert werden

				// Soft reset falls, die Sequence nicht direkt zum GOP gehrt.
				Mpeg12Object gop1=videoStream.GetNextMpeg12Object(typeof(GOP),seq);
				Mpeg12Object gop2=videoStream.GetPrevMpeg12Object(typeof(GOP),videoStream.GetPicture(IFrame));
				if (gop1!=gop2)
					mpegDecoder.Seek(gop2.adress,false); // return !=1 konnte nicht positioniert werden

				// aufs I-Frame positionieren:
				for (FrameInfoStruct fis=Tfi2Fis(mpegDecoder.GetCurrentFrame); fis!=null && fis.type!=1;fis=Tfi2Fis(mpegDecoder.GetCurrentFrame))
					mpegDecoder.libMpeg2_NextFrame();
			}
			else
			{
				// Workaround fr den Fall das der Stream nicht korrekt ist
				// und daher nicht mit einer Sequence, bzw. GOP anfngt.
				mpegDecoder.Seek(0,true);
			}
			// auf die gewnschte Position vorrcken
			mpegDecoder.SkipFrames(Position-IFrame);
			currentPos=Position;
		}

		public override bool GetPicture(int Position, ref System.Drawing.Bitmap bitm, bool silenceMode, out bool newBMPcreated)
		{
			newBMPcreated = false;
			try
			{
				if (Position!=currentPos)
				{
					if (Position>currentPos && Position-currentPos<10)
						mpegDecoder.SkipFrames(Position-currentPos);
					else
						GoToFrame(Position);
				}
				FrameInfoStruct fis;
				fis=Tfi2Fis(mpegDecoder.GetCurrentFrame);
				if (fis==null)
				{
					if (!silenceMode)
						MessageBox.Show(Resource.CurrentResource.FormatMessage(Resource.LIBMPEG2_DISPLAY,Position),"libmpeg2");
					getPictureResult=false;
					return false;
				}
				// Neues Bitmap erzeugen wenn noch gar keins da, oder sich die Gre gendert hat.
				if (bitm==null || bitm.Width!=fis.width || bitm.Height!=fis.height)
				{
					// Unmanaged ressources freigeben
					if (bitm!=null)
						bitm.Dispose();

					bitm=new Bitmap(fis.width,fis.height,PixelFormat.Format24bppRgb);
					newBMPcreated=true;
				}
				BitmapData bmd=bitm.LockBits(new Rectangle(0,0,fis.width,fis.height),ImageLockMode.WriteOnly,PixelFormat.Format24bppRgb);
				try
				{
					mpegDecoder.GetCurrentFrameData(bmd.Scan0);
				}
				finally
				{
					bitm.UnlockBits(bmd);
				}
			}
			catch (Exception e)
			{
				if (!silenceMode)
					MessageBox.Show(e.Message);
				getPictureResult=false;
				return false;
			}
			currentPos=Position;
			getPictureResult=true;
			return true;
		}


	}
}
