
/*************************************************************************
* @mainpage
*  mplex - General-purpose MPEG-1/2 multiplexer.
* (C) 2000, 2001 Andrew Stevens <andrew.stevens@philips.com>
* 
* Doxygen documentation and MPEG Z/Alpha multiplexing part by
* Gernot Ziegler <gz@lysator.liu.se>
*  Constructed using mplex - MPEG-1/SYSTEMS multiplexer as starting point
*  Copyright (C) 1994 1995 Christoph Moar
*  Mplex 2.2.4 ported from Linux to .NET by Elm <elmue@gmx.de> (Dec. 2006)
*  Siemens ZFE ST SN 11 / T SN 6
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.					 *
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.	
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software	
*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*************************************************************************/

#include "main.hpp"

using std::auto_ptr;

#if	!defined(HAVE_LROUND)
extern "C" 
{
	long lround(double x)
	{
		long l = (long)ceil(x);
		return(l);
	}
};
#endif



FileOutputStream::FileOutputStream( const char *name_pat ) 
{
	strncpy( filename_pat, name_pat, MAXPATHLEN );
	snprintf( cur_filename, MAXPATHLEN, filename_pat, segment_num );
}
      
int FileOutputStream::Open()
{
	strm = fopen( cur_filename, "wb" );
	if( strm == NULL )
	{
		mjpeg_error_exit1( "Could not open for writing: %s", cur_filename );
	}

	return 0;
}

void FileOutputStream::Close()
{ 
    fclose(strm);
}


off_t FileOutputStream::SegmentSize()
{
	struct stat stb;
    fstat(fileno(strm), &stb);
	off_t written = stb.st_size;
    return written;
}

void FileOutputStream::NextSegment( )
{
    auto_ptr<char> prev_filename_buf( new char[strlen(cur_filename)+1] );
    char *prev_filename = prev_filename_buf.get();
	fclose(strm);
	++segment_num;
    strcpy( prev_filename, cur_filename );
	snprintf( cur_filename, MAXPATHLEN, filename_pat, segment_num );
	if( strcmp( prev_filename, cur_filename ) == 0 )
	{
		mjpeg_error_exit1( 
			"Need to split output but there appears to be no %%d in the filename pattern %s", filename_pat );
	}
	strm = fopen( cur_filename, "wb" );
	if( strm == NULL )
	{
		mjpeg_error_exit1( "Could not open for writing: %s", cur_filename );
	}
}

void FileOutputStream::Write( uint8_t *buf, unsigned int len )
{
    if( fwrite( buf, 1, len, strm ) != len )
    {
        mjpeg_error_exit1( "Failed write: %s", cur_filename );
    }
}



/********************************
 *
 * IFileBitStream - Input bit stream class for bit streams sourced
 * from standard file I/O (this of course *includes* network sockets,
 * fifo's, et al).
 *
 * OLAF: To hook into your PES reader/reconstructor you need to define
 * a class like this one, where 'ReadStreamBytes' calls you code to
 * generate the required number of bytes of ES data and transfer it 
 * to the specified buffer.  The logical way to do this would be to
 * inherit IBitStream as a base class of the top-level classes for the ES
 * reconstructors.
 *
 ********************************/

// returns 0 on error
uint64_t IFileBitStream::GetFileSize(const char* filename)
{
	HANDLE hFile = CreateFile(filename, GENERIC_READ, FILE_SHARE_READ, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
	if (hFile == INVALID_HANDLE_VALUE)
		return 0;

	DWORD high;
	DWORD low = ::GetFileSize(hFile, &high);

	CloseHandle(hFile);
	return ((uint64_t)high << 32) + low;
}

IFileBitStream::IFileBitStream( const char *bs_filename,
                                unsigned int buf_size) :
    IBitStream()
{
    filesize = GetFileSize(bs_filename);

	if ((fileh = fopen(bs_filename, "rb")) == NULL)
	{
		mjpeg_error_exit1( "Unable to open file %s for reading.", bs_filename);
	}
	filename = strcpy( new char[strlen(bs_filename)+1], bs_filename );
    streamname = filename;

    SetBufSize(buf_size);
	eobs = false;
    byteidx = 0;
	if (!ReadIntoBuffer())
	{
		if (buffered==0)
		{
			mjpeg_error_exit1( "Unable to read from %s.", bs_filename);
		}
	}
}


/**
   Destructor: close the device containing the bit stream after a read
   process
*/
IFileBitStream::~IFileBitStream()
{
	if (fileh)
	{
		fclose(fileh);
		delete filename;
	}
	fileh = 0;
    Release();
}

/*******************************
 *
 * Command line job class - sets up a Multiplex Job based on command
 * line and File I/O...
 *
 ******************************/

// Example:
// argcount = 4
//     cmd[0] = 'i'
// options[0] = "C:\Videos\Terminator.m2v"
//     cmd[1] = 'i'
// options[1] = "C:\Videos\Terminator.mp2"
//     cmd[2] = 'o'
// options[2] = "C:\Videos\Terminator.mpg"
//     cmd[3] = 'f'
// options[3] = "8"
CmdLineMultiplexJob::CmdLineMultiplexJob(unsigned int argcount, char cmd[], char* options[]) 
	: MultiplexJob()
{
	mjpeg_header("************************************************");
	mjpeg_header("Mplex version 2.2.4 (2005/08/28)");
	mjpeg_header("Mplex is part of MJpegTools package version 1.8");
	mjpeg_header("ported from Linux to .NET by Elmue (2006/12/18)");
	mjpeg_header("************************************************");

	vector<IBitStream*> input_files;
	output_file = 0;

	for (unsigned int i=0; i<argcount; i++)
	{
		char* optarg = options[i];
		char  opt    = cmd[i];

		mjpeg_info("Command: '%c' Option: \"%s\"", opt, optarg);

		switch(opt)
		{
		case 'i':
			input_files.push_back(new IFileBitStream(optarg));
			break;

		case 'o':
			output_file = optarg;
			break;

		case 'v':
			verbose = atoi(optarg);
			if( verbose < 0 || verbose > 2 )
				Usage();
			break;

		case 'V':
			VBR = true;
			break;
	  
		case 'h':
			always_system_headers = true;
			break;

		case 'b':
            if (!ParseVideoOpt(optarg))
            {
                mjpeg_error( "Illegal video decoder buffer size(s): %s", 
                             optarg );
                Usage();
            }
            break;

        case 'L':
            if (!ParseLpcmOpt(optarg))
            {
                mjpeg_error( "Illegal LPCM option(s): %s", optarg );
                Usage();
            }
            break;

		case 'r':
			data_rate = atoi(optarg);
			if (data_rate < 0)
				Usage();
			/* Convert from kbit/sec (user spec) to 50B/sec units... */
			data_rate = (( data_rate * 1000 / 8 + 49) / 50 ) * 50;
			break;

		case 'O':
			if(!ParseTimeOffset(optarg))
			{
				mjpeg_error( "Time offset units if specified must: ms|s|mpt" );
				Usage();
			}
			break;
          
		case 'l': 
 			max_PTS = atoi(optarg);
			if( max_PTS < 1)
				Usage();
			break;
		
		case 'p': 
			packets_per_pack = atoi(optarg);
			if (packets_per_pack < 1 || packets_per_pack > 100)
				Usage();
			break;
		
	  
		case 'f':
			mux_format = atoi(optarg);
			if (mux_format < MPEG_FORMAT_MPEG1 || mux_format > MPEG_FORMAT_LAST)
				Usage();
			break;

		case 's':
			sector_size = atoi(optarg);
			if (sector_size < 256 || sector_size > 16384)
				Usage();
			break;

		case 'S':
			max_segment_size = atoi(optarg);
			if (max_segment_size < 0)
				Usage();
			break;

		case 'M':
			multifile_segment = true;
			break;

		default:
			Usage(); // throws exception
			break;
		}
	}

	(void)mjpeg_default_handler_verbosity(verbose);

	if (!output_file)
		mjpeg_error_exit1("No output file specified!");

	SetupInputStreams(input_files); // MUST be the LAST command here !!!!!
}

/*************************************************************************
 Usage banner for the command line wrapper.
*************************************************************************/

    
void CmdLineMultiplexJob::Usage()
{
    mjpeg_error_exit1("Usage: \n"
	"'i' {input stream}\n"
    "  Path to one of the input files (elementary video / audio file)\n"
	"'o' {output stream}\n"
    "  Path to output file (compound video file)\n"
	"'v' {verbose}\n"
    "  Level of verbosity. 0 = quiet, 1 = normal 2 = verbose/debug\n"
	"'f' {format}\n"
    "  Set defaults for particular MPEG profiles\n"
	"  [0 = Generic MPEG1, 1 = VCD, 2 = user-rate VCD, 3 = Generic MPEG2,\n"
    "   4 = SVCD, 5 = user-rate SVCD\n"
	"   6 = VCD Stills, 7 = SVCD Stills, 8 = DVD with NAV sectors, 9 = DVD]\n"
    "'r' {mux-bitrate}\n"
    "  Specify data rate of output stream in kbit/sec\n"
	"  (default 0=Compute from source streams)\n"
	"'b' {video-buffer} [, {video-buffer}...]\n"
    "  Specifies decoder buffers size in kB.  [20...2000]\n"
    "'L' {samppersec:chan:bits} [, {samppersec:chan:bits}]\n"
	"'l' {mux-limit}\n"
    "  Multiplex only num seconds of material (default 0=multiplex all)\n"
	"'O' {sync-offset} ms|s|mpt\n"
    "  Specify offset of timestamps (video-audio) in mSec\n"
	"'s' {sector-size}\n"
    "  Specify sector size in bytes for generic formats [256..16384]\n"
    "'V' {vbr}:\n"
    "  Multiplex variable bit-rate video\n"
	"'p' {packets-per-pack}\n"
    "  Number of packets per pack generic formats [1..100]\n"
	"'h' {system-headers}\n"
    "  Create System header in every pack in generic formats\n"
	"'S' {max-segment-size}\n"
    "  Maximum size of output file(s) in Mbyte (default: 0) (no limit)\n"
	"'M' {ignore-seqend-markers}\n"
    "  Don't switch to a new output file if a sequence end marker\n"
	"  is encountered in the input video.");	
}


bool CmdLineMultiplexJob::ParseLpcmOpt( const char *optarg )
{
    char *endptr, *startptr;
    unsigned int samples_sec;
    unsigned int channels;
    unsigned int bits_sample;
    endptr = const_cast<char *>(optarg);
    do 
    {
        startptr = endptr;
        samples_sec = static_cast<unsigned int>(strtol(startptr, &endptr, 10));
        if( startptr == endptr || *endptr != ':' )
            return false;


        startptr = endptr+1;
        channels = static_cast<unsigned int>(strtol(startptr, &endptr, 10));
        if(startptr == endptr || *endptr != ':' )
            return false;

        startptr = endptr+1;
        bits_sample = static_cast<unsigned int>(strtol(startptr, &endptr, 10));
        if( startptr == endptr )
            return false;

        LpcmParams *params = LpcmParams::Checked( samples_sec,
                                                  channels,
                                                  bits_sample );
        if( params == 0 )
            return false;
        lpcm_param.push_back(params);
        if( *endptr == ',' )
            ++endptr;
    } while( *endptr != '\0' );
    return true;
}


bool CmdLineMultiplexJob::ParseVideoOpt( const char *optarg )
{
    char *endptr, *startptr;
    unsigned int buffer_size;
    endptr = const_cast<char *>(optarg);
    do 
    {
        startptr = endptr;
        buffer_size = static_cast<unsigned int>(strtol(startptr, &endptr, 10));
        if( startptr == endptr )
            return false;

        VideoParams *params = VideoParams::Checked( buffer_size );
        if( params == 0 )
            return false;
        video_param.push_back(params);
        if( *endptr == ',' )
            ++endptr;
    } 
    while( *endptr != '\0' );
    return true;
}

bool CmdLineMultiplexJob::ParseTimeOffset(const char *optarg)
{
    double f;
    double persecond=1000.0;
    char *e;

    f=strtod(optarg,&e);
    if( *e ) {
        while(isspace(*e)) e++;
        if(!strcmp(e,"ms")) persecond=1000.0;
        else if(!strcmp(e,"s")) persecond=1.0;
        else if(!strcmp(e,"mpt")) persecond=90000.0;
		else
			return false;
    }
    video_offset = static_cast<int>(f*CLOCKS/(persecond));
	if( video_offset < 0 )
	{
		audio_offset = - video_offset;
		video_offset = 0;
	}
	return true;
}

int main (int argc, char* argv[])
{
	char  cmd[50];
	char* opt[50];

    cmd[0] = 'i';
	opt[0] = "E:\\Temp\\Videos\\Test.m2v";
    cmd[1] = 'i';
	opt[1] = "E:\\Temp\\Videos\\Test.mp2";
    cmd[2] = 'o';
	opt[2] = "E:\\Temp\\Videos\\Test.mpg";
    cmd[3] = 'f';
	opt[3] = "1";

	CmdLineMultiplexJob job(4, cmd, opt);

	FileOutputStream output(job.output_file);
	Multiplexor mux(job, output);
	mux.SetCpuLoad(90);
	mux.Multiplex();
    return 0;
}

			
