using System;
using System.IO;
using System.Text;
using System.Data;
using System.Drawing;
using System.Diagnostics;
using System.Collections;
using System.Windows.Forms;
using System.Threading;
using System.Runtime.InteropServices;
using Muxer;

namespace mplexProvider
{
	/// <summary>
	/// Summary description for OutputWindow.
	/// </summary>
	public class OutputWindow : System.Windows.Forms.Form
	{
		[DllImport("user32.dll", EntryPoint="SendMessageA")]
		private static extern int SendMessageA(IntPtr h_Wnd, int message, int wParam, int lParam);

		const int SB_BOTTOM           = 7;
		const int WM_VSCROLL          = 0x0115;
		const int WM_WINDOWPOSCHANGED = 0x0047;

		private delegate void delPrintMsg(Mplex.ePrintType e_Type, string s_Msg);
		private delegate void delThreadFinished();

		private Form            mi_Owner;
		private string          ms_WindowTitle;
		private string          ms_OutFile;
		private DataSet         mi_DataSet;
		private DataRow         mi_DataRow;
		private Mplex           mi_Mplex;
		private bool            mb_ErrorOccurred;
		private bool            mb_Aborted;
		private bool            mb_ThreadRunning;
		private bool            mb_Closed;
		private FormWindowState me_LastState;
		private int             ms32_Progress;
		private DialogResult    me_Result;
		
		public OutputWindow(DataSet i_DataSet, DataRow i_DataRow, Form i_Owner)
		{
			InitializeComponent();

			mi_Owner         = i_Owner;
			ms_WindowTitle   = this.Text; // from resources
			ms_OutFile       = "";
			mi_Mplex         = new Mplex();
			mi_DataSet       = i_DataSet;
			mi_DataRow       = i_DataRow;
			mb_ErrorOccurred = false;
			mb_ThreadRunning = false;
			mb_Aborted       = false;
			mb_Closed        = false;
			me_LastState     = this.WindowState;
			ms32_Progress    = 0;
		}

		public int Progress
		{
			get { return ms32_Progress; }
		}

		public bool HasClosed
		{
			get { return mb_Closed; }
		}

		private void OnLoad(object sender, System.EventArgs ev)
		{
			// An Owner MUST NOT be set !!
			// If CMMain is the owner, this dialog is destroyed when CMMain is minimized!
			this.Owner = null;

			SetCpuLoad();
			Thread i_Worker = new Thread(new ThreadStart(WorkerThread));
			i_Worker.Start();
		}

		private void WorkerThread()
		{
			mb_ThreadRunning = true;
			StringBuilder s_Commands  = new StringBuilder(1000);
			ArrayList     i_Arguments = new ArrayList();

			ms_OutFile = (string)mi_DataRow["OutputMPG"];

			if (Settings.FileSizeSplit)
			{
				s_Commands. Append('S'); // Split filesize in MByte
				i_Arguments.Add(Settings.FileSize.ToString());

				string s_Part1 = Path.GetDirectoryName(ms_OutFile);
				string s_Part2 = Path.GetFileNameWithoutExtension(ms_OutFile);
				string s_Part3 = Path.GetExtension(ms_OutFile);

				// Output will be "E:\Videos\File_01.mpg", "E:\Videos\File_02.mpg" etc..
				ms_OutFile = string.Format("{0}\\{1}_%02d{2}", s_Part1, s_Part2, s_Part3);
			}

			s_Commands. Append('o'); // Output file
			i_Arguments.Add(ms_OutFile);

			s_Commands. Append('i'); // Video file (in)
			i_Arguments.Add(mi_DataRow["VideoFile"]);

			foreach (DataRow i_AudioRow in mi_DataSet.Tables["Audio"].Rows)
			{
				s_Commands. Append('i'); // Audio file (in)
				i_Arguments.Add(i_AudioRow["AudioFile"]);
			}

			s_Commands.Append('f'); // Output format
			switch((int)mi_DataRow["MuxType"])
			{
				case 0: i_Arguments.Add("1"); break; // VCD
				case 1: i_Arguments.Add("4"); break; // SVCD
				case 2: i_Arguments.Add("8"); break; // DVD
				default:
					throw new Exception("Invalid video output format specified.");
			}

			if (Settings.BitrateFix)
			{
				s_Commands. Append('r'); // Bitrate in kBit/s
				i_Arguments.Add(Settings.Bitrate.ToString());
			}

			if (Settings.VariableBitrate)
			{
				s_Commands. Append('V'); // Variable Bitrate
				i_Arguments.Add("");     // no argument required
			}

			String[] s_Args = (String[]) i_Arguments.ToArray(typeof(String));

			Mplex.CallbackPrintMessage = new Mplex.delPrintMessage(OnPrintMplexMessage);

			try { mi_Mplex.Execute(s_Commands.ToString(), s_Args); }
			catch
			{
				// Exception comes after calling mi_Mplex.AbortMuxing() or fatal error
				// Nothing to do here as the error message has already been printed
			}

			me_Result = DialogResult.OK;
			if (mb_ErrorOccurred) me_Result = DialogResult.Cancel;
			if (mb_Aborted)       me_Result = DialogResult.Abort;

			// asynchronous call into GUI thread
			this.Invoke(new delThreadFinished(OnThreadFinished));
		}

		/// <summary>
		/// This is called from inside the GUI thread
		/// </summary>
		private void OnThreadFinished()
		{
			mb_ThreadRunning = false;
			if (mb_Aborted)
			{
				Thread.Sleep(1000); // 1 second to read "User aborted" error
				Close();
				return;
			}

			if (this.WindowState == FormWindowState.Minimized)
			{
				// If there was no error while window minimized -> close immediately
				// otherwise show "Error" on taskbar button
				if (mb_ErrorOccurred) this.Text = " Error";
				else Close();
			}

			if (Settings.FileSizeSplit)
				ms_OutFile = ms_OutFile.Replace("_%02d", "_01"); // "Video_01.mpg"

			// enable the button even if errors have occurred (Buffer underruns etc..)
			if (File.Exists(ms_OutFile))
				butPlay.Enabled = true;

			butAbort.Enabled = false;
		}

		/// <summary>
		/// Runs in the worker thread
		/// </summary>
		private void OnPrintMplexMessage(Mplex.ePrintType e_Type, string s_Message)
		{
			// asynchronous call into GUI thread
			Invoke(new delPrintMsg(this.OnPrintMplexMessageAsync), new object[]{e_Type, s_Message});

			// Let the worker thread sleep so the GUI thread has time 
			// to display the output and update the progressbar
			Thread.Sleep(10);
		}

		/// <summary>
		/// Runs in the GUI thread
		/// </summary>
		private void OnPrintMplexMessageAsync(Mplex.ePrintType e_Type, string s_Message)
		{
			if (e_Type == Mplex.ePrintType.Progress)
			{
				ms32_Progress     = Int32.Parse(s_Message);
				progressBar.Value = ms32_Progress;

				// If minimized -> show progress on Taskbar button of this window
				if (this.WindowState == FormWindowState.Minimized)
					this.Text = " " + s_Message + "%";
				return;
			}

			switch (e_Type)
			{
				case Mplex.ePrintType.Error: 
					richTextBox.SelectionColor = Color.HotPink;
					s_Message = "ERROR: " + s_Message;
					mb_ErrorOccurred = true;
					break;
				case Mplex.ePrintType.Warn:  
					richTextBox.SelectionColor = Color.Yellow;    
					s_Message = "WARNING: " + s_Message;
					break;
				case Mplex.ePrintType.Debug: 
					richTextBox.SelectionColor = Color.LightGray; 
					break;
				case Mplex.ePrintType.Header: 
					richTextBox.SelectionColor = Color.Cyan; 
					break;
				default: 
					richTextBox.SelectionColor = Color.White;    
					break;
			}

			richTextBox.AppendText(s_Message + "\r\n");

			// Scroll to the bottom of the rtfBox
			SendMessageA(richTextBox.Handle, WM_VSCROLL, SB_BOTTOM, 0);
		}

		/// <summary>
		/// IMPORTANT: Workaround for .NET bug in RichTextBox:
		/// If this window was opened in minimized state and the user normalizes
		/// the window, the RTF box will not display anything altough it has content.
		/// A scroll down command will let the content appear.
		/// </summary>
		private void OnTimer(object sender, System.Timers.ElapsedEventArgs e)
		{
			SendMessageA(richTextBox.Handle, WM_VSCROLL, SB_BOTTOM, 0);
		}

		/// <summary>
		/// When the user changes the main window's state the Cpu load is adapted
		/// (Normal / Minimized / Maximized)
		/// This is called from inside the GUI thread
		/// </summary>
		protected override void WndProc(ref Message Msg)
		{
			base.WndProc(ref Msg);

			// check this AFTER base.WndProc()
			if (Msg.Msg == WM_WINDOWPOSCHANGED && me_LastState != this.WindowState)
			{
				SetCpuLoad();

				me_LastState = this.WindowState;
				if (this.WindowState != FormWindowState.Minimized)
				{
					this.CenterToScreen();
					System.Timers.Timer i_Timer = new System.Timers.Timer(500);
					i_Timer.AutoReset = false;
					i_Timer.Elapsed  += new System.Timers.ElapsedEventHandler(OnTimer);
					i_Timer.Start();
				}
			}
		}

		/// <summary>
		/// Reduce the CPU load to 40% if this window is minimized
		/// </summary>
		private void SetCpuLoad()
		{
			if (this.WindowState == FormWindowState.Minimized)
				mi_Mplex.SetCpuLoad(40);
			else
			{
				mi_Mplex.SetCpuLoad(95);
				this.Text = ms_WindowTitle;
			}
		}

		// User clicked the close button -> abort thread
		private void OnClosing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			mb_Aborted = true;
			mi_Mplex.AbortMuxing(); // threadsafe!

			// Block closing the window before worker thread has finished and closed all files
			if (mb_ThreadRunning) 
				e.Cancel = true;
			else 
				DialogResult = me_Result;
		}

		protected override void OnClosed(EventArgs e)
		{
			base.OnClosed (e);
			mb_Closed = true;
		}

		public void OnButtonAbort(object sender, System.EventArgs e)
		{
			mb_Aborted = true;
			mi_Mplex.AbortMuxing(); // threadsafe!

			// When external call after mplex finished (Cancel Button in ThreadControl)
			if (!mb_ThreadRunning) Close();
		}

		/// <summary>
		/// Play the muxed video in the default video player
		/// </summary>
		private void OnButtonPlay(object sender, System.EventArgs e)
		{
			try { Process.Start(ms_OutFile); }
			catch (Exception Ex)
			{
				MessageBox.Show(mi_Owner, Ex.Message + "\n" + ms_OutFile);
			}
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}
		
		private System.Windows.Forms.Button butPlay;
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.RichTextBox richTextBox;
		private System.Windows.Forms.ProgressBar progressBar;
		private System.Windows.Forms.Button butAbort;

		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(OutputWindow));
			this.richTextBox = new System.Windows.Forms.RichTextBox();
			this.progressBar = new System.Windows.Forms.ProgressBar();
			this.butAbort = new System.Windows.Forms.Button();
			this.butPlay = new System.Windows.Forms.Button();
			this.SuspendLayout();
			// 
			// richTextBox
			// 
			this.richTextBox.AccessibleDescription = resources.GetString("richTextBox.AccessibleDescription");
			this.richTextBox.AccessibleName = resources.GetString("richTextBox.AccessibleName");
			this.richTextBox.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("richTextBox.Anchor")));
			this.richTextBox.AutoSize = ((bool)(resources.GetObject("richTextBox.AutoSize")));
			this.richTextBox.BackColor = System.Drawing.Color.Black;
			this.richTextBox.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("richTextBox.BackgroundImage")));
			this.richTextBox.BulletIndent = ((int)(resources.GetObject("richTextBox.BulletIndent")));
			this.richTextBox.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("richTextBox.Dock")));
			this.richTextBox.Enabled = ((bool)(resources.GetObject("richTextBox.Enabled")));
			this.richTextBox.Font = ((System.Drawing.Font)(resources.GetObject("richTextBox.Font")));
			this.richTextBox.ForeColor = System.Drawing.SystemColors.HighlightText;
			this.richTextBox.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("richTextBox.ImeMode")));
			this.richTextBox.Location = ((System.Drawing.Point)(resources.GetObject("richTextBox.Location")));
			this.richTextBox.MaxLength = ((int)(resources.GetObject("richTextBox.MaxLength")));
			this.richTextBox.Multiline = ((bool)(resources.GetObject("richTextBox.Multiline")));
			this.richTextBox.Name = "richTextBox";
			this.richTextBox.ReadOnly = true;
			this.richTextBox.RightMargin = ((int)(resources.GetObject("richTextBox.RightMargin")));
			this.richTextBox.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("richTextBox.RightToLeft")));
			this.richTextBox.ScrollBars = ((System.Windows.Forms.RichTextBoxScrollBars)(resources.GetObject("richTextBox.ScrollBars")));
			this.richTextBox.Size = ((System.Drawing.Size)(resources.GetObject("richTextBox.Size")));
			this.richTextBox.TabIndex = ((int)(resources.GetObject("richTextBox.TabIndex")));
			this.richTextBox.Text = resources.GetString("richTextBox.Text");
			this.richTextBox.Visible = ((bool)(resources.GetObject("richTextBox.Visible")));
			this.richTextBox.WordWrap = ((bool)(resources.GetObject("richTextBox.WordWrap")));
			this.richTextBox.ZoomFactor = ((System.Single)(resources.GetObject("richTextBox.ZoomFactor")));
			// 
			// progressBar
			// 
			this.progressBar.AccessibleDescription = resources.GetString("progressBar.AccessibleDescription");
			this.progressBar.AccessibleName = resources.GetString("progressBar.AccessibleName");
			this.progressBar.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("progressBar.Anchor")));
			this.progressBar.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("progressBar.BackgroundImage")));
			this.progressBar.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("progressBar.Dock")));
			this.progressBar.Enabled = ((bool)(resources.GetObject("progressBar.Enabled")));
			this.progressBar.Font = ((System.Drawing.Font)(resources.GetObject("progressBar.Font")));
			this.progressBar.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("progressBar.ImeMode")));
			this.progressBar.Location = ((System.Drawing.Point)(resources.GetObject("progressBar.Location")));
			this.progressBar.Name = "progressBar";
			this.progressBar.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("progressBar.RightToLeft")));
			this.progressBar.Size = ((System.Drawing.Size)(resources.GetObject("progressBar.Size")));
			this.progressBar.TabIndex = ((int)(resources.GetObject("progressBar.TabIndex")));
			this.progressBar.Text = resources.GetString("progressBar.Text");
			this.progressBar.Visible = ((bool)(resources.GetObject("progressBar.Visible")));
			// 
			// butAbort
			// 
			this.butAbort.AccessibleDescription = resources.GetString("butAbort.AccessibleDescription");
			this.butAbort.AccessibleName = resources.GetString("butAbort.AccessibleName");
			this.butAbort.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("butAbort.Anchor")));
			this.butAbort.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("butAbort.BackgroundImage")));
			this.butAbort.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("butAbort.Dock")));
			this.butAbort.Enabled = ((bool)(resources.GetObject("butAbort.Enabled")));
			this.butAbort.FlatStyle = ((System.Windows.Forms.FlatStyle)(resources.GetObject("butAbort.FlatStyle")));
			this.butAbort.Font = ((System.Drawing.Font)(resources.GetObject("butAbort.Font")));
			this.butAbort.Image = ((System.Drawing.Image)(resources.GetObject("butAbort.Image")));
			this.butAbort.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("butAbort.ImageAlign")));
			this.butAbort.ImageIndex = ((int)(resources.GetObject("butAbort.ImageIndex")));
			this.butAbort.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("butAbort.ImeMode")));
			this.butAbort.Location = ((System.Drawing.Point)(resources.GetObject("butAbort.Location")));
			this.butAbort.Name = "butAbort";
			this.butAbort.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("butAbort.RightToLeft")));
			this.butAbort.Size = ((System.Drawing.Size)(resources.GetObject("butAbort.Size")));
			this.butAbort.TabIndex = ((int)(resources.GetObject("butAbort.TabIndex")));
			this.butAbort.Text = resources.GetString("butAbort.Text");
			this.butAbort.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("butAbort.TextAlign")));
			this.butAbort.Visible = ((bool)(resources.GetObject("butAbort.Visible")));
			this.butAbort.Click += new System.EventHandler(this.OnButtonAbort);
			// 
			// butPlay
			// 
			this.butPlay.AccessibleDescription = resources.GetString("butPlay.AccessibleDescription");
			this.butPlay.AccessibleName = resources.GetString("butPlay.AccessibleName");
			this.butPlay.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("butPlay.Anchor")));
			this.butPlay.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("butPlay.BackgroundImage")));
			this.butPlay.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("butPlay.Dock")));
			this.butPlay.Enabled = ((bool)(resources.GetObject("butPlay.Enabled")));
			this.butPlay.FlatStyle = ((System.Windows.Forms.FlatStyle)(resources.GetObject("butPlay.FlatStyle")));
			this.butPlay.Font = ((System.Drawing.Font)(resources.GetObject("butPlay.Font")));
			this.butPlay.Image = ((System.Drawing.Image)(resources.GetObject("butPlay.Image")));
			this.butPlay.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("butPlay.ImageAlign")));
			this.butPlay.ImageIndex = ((int)(resources.GetObject("butPlay.ImageIndex")));
			this.butPlay.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("butPlay.ImeMode")));
			this.butPlay.Location = ((System.Drawing.Point)(resources.GetObject("butPlay.Location")));
			this.butPlay.Name = "butPlay";
			this.butPlay.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("butPlay.RightToLeft")));
			this.butPlay.Size = ((System.Drawing.Size)(resources.GetObject("butPlay.Size")));
			this.butPlay.TabIndex = ((int)(resources.GetObject("butPlay.TabIndex")));
			this.butPlay.Text = resources.GetString("butPlay.Text");
			this.butPlay.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("butPlay.TextAlign")));
			this.butPlay.Visible = ((bool)(resources.GetObject("butPlay.Visible")));
			this.butPlay.Click += new System.EventHandler(this.OnButtonPlay);
			// 
			// OutputWindow
			// 
			this.AccessibleDescription = resources.GetString("$this.AccessibleDescription");
			this.AccessibleName = resources.GetString("$this.AccessibleName");
			this.AutoScaleBaseSize = ((System.Drawing.Size)(resources.GetObject("$this.AutoScaleBaseSize")));
			this.AutoScroll = ((bool)(resources.GetObject("$this.AutoScroll")));
			this.AutoScrollMargin = ((System.Drawing.Size)(resources.GetObject("$this.AutoScrollMargin")));
			this.AutoScrollMinSize = ((System.Drawing.Size)(resources.GetObject("$this.AutoScrollMinSize")));
			this.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("$this.BackgroundImage")));
			this.ClientSize = ((System.Drawing.Size)(resources.GetObject("$this.ClientSize")));
			this.Controls.Add(this.butPlay);
			this.Controls.Add(this.progressBar);
			this.Controls.Add(this.richTextBox);
			this.Controls.Add(this.butAbort);
			this.Enabled = ((bool)(resources.GetObject("$this.Enabled")));
			this.Font = ((System.Drawing.Font)(resources.GetObject("$this.Font")));
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("$this.ImeMode")));
			this.Location = ((System.Drawing.Point)(resources.GetObject("$this.Location")));
			this.MaximizeBox = false;
			this.MaximumSize = ((System.Drawing.Size)(resources.GetObject("$this.MaximumSize")));
			this.MinimumSize = ((System.Drawing.Size)(resources.GetObject("$this.MinimumSize")));
			this.Name = "OutputWindow";
			this.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("$this.RightToLeft")));
			this.SizeGripStyle = System.Windows.Forms.SizeGripStyle.Show;
			this.StartPosition = ((System.Windows.Forms.FormStartPosition)(resources.GetObject("$this.StartPosition")));
			this.Text = resources.GetString("$this.Text");
			this.Closing += new System.ComponentModel.CancelEventHandler(this.OnClosing);
			this.Load += new System.EventHandler(this.OnLoad);
			this.ResumeLayout(false);

		}
		#endregion

	}
}
