///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Author: Elm (http://netcult.ch/elmue)
// Date: 23-11-2006
//
// Filename: Map.hpp
//
// Class CMap
//
// Purpose: This tiny class is a simple (and ultra fast) replacement for std::map
// It is strongly recommended to use this class instead, because using #include <map>
// will create a dependency to MSVCP70.DLL or MSVCP71.DLL or MSVCP80.DLL
// If you use std::map you have to deliver one of these DLLs with you application 
// because these DLLs are NOT available on every computer.
// Using CMap instead eliminates this problem!
// This class is threadsafe
//

#pragma once


// Dynamic memory management is not necessary in this context.
// We will never need to hook on more than 1000 windows!
// If the maximum is exceeded an exception will be thrown, 
// but this will never happen because normally there are 
// not more than TWO window handles in this map!
#define __MaxEntries 1000

template <class tKey, class tValue> class CMap
{
private:
	struct kData
	{
		tKey   Key;
		tValue Value;
	};

	int    ms32_Count;
	kData  mk_Data[__MaxEntries];
	HANDLE mh_Mutex;

public:
	CMap()
	{
		mh_Mutex = CreateMutex(0, FALSE, 0); 
		Clear();
	}

	~CMap()
	{
		CloseHandle(mh_Mutex);
	}

	void Clear()
	{
		WaitForSingleObject(mh_Mutex, INFINITE);
		ms32_Count = 0;
		ReleaseMutex(mh_Mutex);
	}

	int Count()
	{
		WaitForSingleObject(mh_Mutex, INFINITE);
		return ms32_Count;
		ReleaseMutex(mh_Mutex);
	}

	// Returns the zero based index of the Key or -1 if not found
	int Find(tKey Key)
	{
		WaitForSingleObject(mh_Mutex, INFINITE);
		
		int s32_Pos = -1;
		for (int i=0; i<ms32_Count; i++)
		{
			if (mk_Data[i].Key == Key)
			{
				s32_Pos = i;
				break;
			}
		}
		
		ReleaseMutex(mh_Mutex);
		return s32_Pos;
	}

	// Delete the given Key if it exists
	void Erase(tKey Key)
	{
		WaitForSingleObject(mh_Mutex, INFINITE);
		
		int s32_Pos = Find(Key);
		if (s32_Pos >= 0)
		{
			ms32_Count --; // Order FIRST!
			memmove(&mk_Data[s32_Pos], &mk_Data[s32_Pos+1], sizeof(kData)*(ms32_Count-s32_Pos));
		}
		
		ReleaseMutex(mh_Mutex);
	}

	// Sets a value for the given key
	// If the key already exists, its value is overwritten
	// If the key does not yet exist, it is created
	void Set(tKey Key, tValue Value)
	{
		WaitForSingleObject(mh_Mutex, INFINITE);
		
		int s32_Pos = Find(Key);
		if (s32_Pos < 0)
		{
			if (ms32_Count >= __MaxEntries)
				throw "Overflow in CMap::Set()";

			s32_Pos = ms32_Count; // create new entry
			ms32_Count ++;
		}

		mk_Data[s32_Pos].Key   = Key;
		mk_Data[s32_Pos].Value = Value;
		
		ReleaseMutex(mh_Mutex);
	}

	// Returns the value of the key at the given position.
	// Before you must retrieve the position with Find(Key)
	// Do not call GetAt() if Find() returned -1 !!
	tValue GetAt(int s32_Pos)
	{
		WaitForSingleObject(mh_Mutex, INFINITE);
		
		if (s32_Pos < 0 || s32_Pos >= ms32_Count)
			throw "Invalid position in CMap::Get()";
		
		tValue Result = mk_Data[s32_Pos].Value;
		
		ReleaseMutex(mh_Mutex);
		return Result;
	}

	// Combines Find() and GetAt()
	// returns NULL if entry does not exist
	tValue Get(tKey Key)
	{
		WaitForSingleObject(mh_Mutex, INFINITE);
		
		tValue t_Result = NULL;
		int s32_Pos = Find(Key);
		if (s32_Pos >= 0)
		{
			t_Result = GetAt(s32_Pos);
		}

		ReleaseMutex(mh_Mutex);
		return t_Result;
	}
};

