/*
 * Cuttermaran - The Mpeg Cutter
 * 
 * Copyright (C) 2004 Tobias Arnold (cuttermaran@gmx.net)
 * 
 * This file is part of cuttermaran, a free MPEG cut program.
 * See http://cuttermaran.kickme.to for updates.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

using System;
using System.IO;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

namespace QuEncProvider
{
	/// <summary>
	/// Summary description for Connector.
	/// </summary>
	public class Connector
	{
		private Form    mi_OwnerForm = null;
		private Process mi_Process   = null;

		private static decimal [] Mpeg1RatioData=new decimal []
			{
				1.0m, 1.0m, 0.6735m, 0.7031m,
				0.7615m,0.8055m,0.8437m,0.8935m,
				0.9157m,0.9815m,1.0255m,1.0695m,
				1.095m,1.1575m,1.2051m,1m
			};

		private static byte [] stdIntraMatrix = new byte []
			{
				8,16,19,22,26,27,29,34,
				16,16,22,24,27,29,34,37,
				19,22,26,27,29,34,34,38,
				22,22,26,27,29,34,37,40,
				22,26,27,29,32,35,40,48,
				26,27,29,32,35,40,48,58,
				26,27,29,34,38,46,56,69,
				27,29,35,38,46,56,69,83
			};

		private static byte [] stdNonIntraMatrix = new byte []
			{
				16,16,16,16,16,16,16,16,
				16,16,16,16,16,16,16,16,
				16,16,16,16,16,16,16,16,
				16,16,16,16,16,16,16,16,
				16,16,16,16,16,16,16,16,
				16,16,16,16,16,16,16,16,
				16,16,16,16,16,16,16,16,
				16,16,16,16,16,16,16,16
			};

		private static byte [,] zickZackMatrix = new byte [8,8]
			{
				{ 0, 1, 5, 6,14,15,27,28},
				{ 2, 4, 7,13,16,26,29,42},
				{ 3, 8,12,17,25,30,41,43},
				{ 9,11,18,24,31,40,44,53},
				{10,19,23,32,39,45,52,54},
				{20,22,33,38,46,51,55,60},
				{21,34,37,47,50,56,59,61},
				{35,36,48,49,57,58,62,63}
			};

		public Connector()
		{
		}

		/// <summary>
		/// Abort the encoding process
		/// This function is called by reflection
		/// </summary>
		public void AbortAction()
		{
			if (mi_Process != null)
				mi_Process.Kill();
		}

		/// <summary>
		/// returns the current progress (0-100)
		/// This function is called by reflection
		/// </summary>
		public int GetProgress()
		{
			// not supported by Quenc
			return 0;
		}

		/// <summary>
		/// This will set CMMain or Confirguration.Settings as parent for all dialogs / messageboxes
		/// This function is called by reflection
		/// </summary>
		public void SetOwnerWindow(Form i_Owner)
		{
			mi_OwnerForm = i_Owner;
		}

		/// <summary>
		/// This function is called by reflection
		/// </summary>
		public int Encode(DataSet data)
		{
			DataRow dr=null;
			try
			{
				// Hier gibts die Daten
				dr=data.Tables[0].Rows[0];

				// Dies wird die Commandozeile
				StringBuilder cmdLine=new StringBuilder();

				cmdLine.AppendFormat("-i \"{0}\" ",new AviSynthWriter().WriteScript(data));
				//cmdLine.AppendFormat("-i \"{0}\" ",dr["AVIFile"]);
				cmdLine.AppendFormat("-o \"{0}\" ",dr["ResultMPGFile"]);

				cmdLine.AppendFormat("-b {0} ",((Int64)dr["BitRate"])/1000);

				cmdLine.AppendFormat("-maxbitrate {0} ",((Int64)dr["BitRate"])/1000);

				cmdLine.AppendFormat("-dc {0} ",dr["intra_dc_precision"]);

				if (Settings.TwoPassEncoding)
					cmdLine.Append("-2 "); // 2-pass
				else
					cmdLine.Append("-1 "); // 1-pass

				if ((bool)dr["IsMpeg2"]) // MPEG2
					cmdLine.Append("-mpeg2mux (seperate) ");
				else
					cmdLine.Append("-mpeg1mux (seperate) ");

				if (Settings.HighQuality)
					cmdLine.Append("-hq ");
				else
					cmdLine.Append("-nohq ");

				if (Settings.UseVBR)
					cmdLine.Append("-vbr ");
				else
					cmdLine.Append("-novbr ");

				if (Settings.SeceneDetection)
					cmdLine.Append("-scene ");
				else
					cmdLine.Append("-noscene ");

				if (Settings.UseTrellisQuant)
					cmdLine.Append("-trell ");
				else
					cmdLine.Append("-notrell ");

				cmdLine.Append("-nocgop ");
				if ((bool)dr["IsProgressive"])
					cmdLine.Append("-nointerlaced ");
				else
					cmdLine.Append("-interlaced ");

				if (Settings.ExtremeSettings)
					cmdLine.Append("-extreme ");
				else
					cmdLine.Append("-noextreme ");

				cmdLine.AppendFormat("-gopsize {0} ",Settings.GOPSize);
				cmdLine.AppendFormat("-maxbframes {0} ",Settings.MaxBFrames);

				// Quantisierungs Matrizen
				if (!dr.IsNull("intra_quantizer_matrix") || !dr.IsNull("non_intra_quantizer_matrix"))
				{
					using (FileStream fs=File.OpenWrite(dr["AVIFile"].ToString()+".xcm"))
					{
						if (!dr.IsNull("intra_quantizer_matrix"))
							for (int y=0;y<8;++y)
								for (int x=0;x<8;++x)
									fs.WriteByte(((byte [])dr["intra_quantizer_matrix"])[zickZackMatrix[x,y]]);
						else
							fs.Write(stdIntraMatrix,0,64);
						if (!dr.IsNull("non_intra_quantizer_matrix"))
							for (int y=0;y<8;++y)
								for (int x=0;x<8;++x)
									fs.WriteByte(((byte [])dr["non_intra_quantizer_matrix"])[zickZackMatrix[x,y]]);
						else
							fs.Write(stdNonIntraMatrix,0,64);
					}
					cmdLine.AppendFormat("-cmatrix \"{0}\" ",dr["AVIFile"].ToString()+".xcm");
				}
				else
					cmdLine.Append("-nocmatrix ");

				cmdLine.Append("-aspectratio ");
				short aspectRatio=(short)dr["AspectRatio"];
				if (!(bool)dr["IsMpeg2"]) // MPEG1;
				{
					aspectRatio=1;
					if ((short)dr["AspectRatio"]>1) // ungleich 1:1
					{
						decimal xyratio=(short)dr["Width"]*Mpeg1RatioData[(short)dr["AspectRatio"]]/(short)dr["Height"];
						if (0.745m<xyratio && xyratio<0.755m) // 4:3
							aspectRatio=2;
						if (0.5575m<xyratio && xyratio<0.5675m) // 16:9
							aspectRatio=3;
						if (0.4475m<xyratio && xyratio<0.4575m) // 2.21:1
							aspectRatio=4;
					}
				}
				switch (aspectRatio)
				{
					case 2:
						cmdLine.Append("4:3 ");
						break;
					case 3:
						cmdLine.Append("16:9 ");
						break;
					case 4:
						cmdLine.Append("2:21 ");
						break;
					default:
						cmdLine.Append("1:1 ");
						break;
				}

				// Open Quenc hidden if user has defined this in configuration settings
				// OR if the main window is minimized
				if (Settings.SilentMode || (bool)dr["Unattended"] ||
					(mi_OwnerForm != null && mi_OwnerForm.WindowState == FormWindowState.Minimized))
				{
					cmdLine.Append("-silent ");
				}
				cmdLine.Append("-auto ");
				cmdLine.Append("-close");

				mi_Process = Process.Start(Settings.QuEncExe,cmdLine.ToString());
				mi_Process.WaitForExit();
				mi_Process = null; // for AbortAction()

				return 0;
			}
			catch (Exception ex)
			{
				if (dr!=null && !(bool)dr["Unattended"])
					MessageBox.Show(mi_OwnerForm, ex.Message,"QuEncProvider");
				return -1;
			}
		}

		public void Config()
		{
			Settings settings=new Settings();
			settings.ShowDialog(mi_OwnerForm);
		}
	}
}
