/*
 * Cuttermaran - The Mpeg Cutter
 * 
 * Copyright (C) 2003-2005 Tobias Arnold (cuttermaran@gmx.net)
 * 
 * This file is part of cuttermaran, a free MPEG cut program.
 * See http://www.cuttermaran.de for updates.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

using System;
using System.IO;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using System.Text.RegularExpressions;
using StringResources;

namespace Configuration
{
	public abstract class MRUBaseManager
	{
		protected ArrayList ItemList=new ArrayList();

		public abstract void AddItem(string itemName);
		public virtual ArrayList Entries
		{
			get
			{
				return (ArrayList)ItemList.Clone();
			}
			set
			{
				ItemList=(ArrayList)value.Clone();
			}
		}

		protected virtual bool RemoveItem(object item)
		{
			int pos=ItemList.IndexOf(item);
			if (pos!=-1)
			{
				ItemList.RemoveAt(pos);
				return true;
			}
			return false;
		}
	}

	/// <summary>
	/// Summary description for MRUManager.
	/// </summary>
	public class MRUFileManager : MRUBaseManager
	{
		public delegate bool MRUItemClick(string filename, bool b_AskSaveChanges);

		private MRUItemClick ItemClick;
		private MenuItem parent;
		private MenuItem emptyItem=new MenuItem(Resource.CurrentResource[Resource.EMPTY]);
		private const int MRULength=4;

		public MRUFileManager(MenuItem Parent,MRUItemClick MRUClickCallback)
		{
			ItemClick=MRUClickCallback;
			parent=Parent;
			emptyItem.Enabled=false;
			DrawMRUObjects();
		}

		public override void AddItem(string filename)
		{
			int pos;
			for (pos=0;pos<ItemList.Count;++pos)
			{
				MenuItem currentItem=(MenuItem)ItemList[pos];
				if (currentItem.Text.ToLower()==filename.ToLower())
				{
					ItemList.RemoveAt(pos);
					ItemList.Insert(0,currentItem);
					if (!File.Exists(filename))
						RemoveItem(currentItem);
					break;
				}
			}
			if (pos==ItemList.Count) // nicht gefunden
			{
				if (!File.Exists(filename))
					return;
				MenuItem newItem=new MenuItem(filename,new EventHandler(OnClick));
				ItemList.Insert(0,newItem);
				if (ItemList.Count>MRULength)
					ItemList.RemoveAt(MRULength);
			}
			DrawMRUObjects();
		}

		private void DrawMRUObjects()
		{
			parent.MenuItems.Clear();
			if (ItemList.Count>0)
				parent.MenuItems.AddRange((MenuItem [])(ItemList.ToArray(typeof(MenuItem))));
			else
				parent.MenuItems.Add(emptyItem);
		}

		protected override bool RemoveItem(object item)
		{
			if (base.RemoveItem(item))
			{
				DrawMRUObjects();
				return true;
			}
			return false;
		}

		private void OnClick(object sender,EventArgs e)
		{
			MenuItem currentItem=sender as MenuItem;
			if (!File.Exists(currentItem.Text))
				RemoveItem(currentItem);
			else
			{
				AddItem(currentItem.Text); // Als Topobjekt
				ItemClick(currentItem.Text, true);
			}
		}

		public override ArrayList Entries
		{
			get
			{
				ArrayList result=new ArrayList();
				foreach (MenuItem mi in ItemList)
					result.Add(mi.Text);
				return result;
			}
			set
			{
				ItemList.Clear();
				if (value.Count>0)
				{
					for (int pos=value.Count-1;pos>=0;--pos)
						AddItem((string)value[pos]);
				}
			}
		}
	}

	public class MRUPathManager : MRUBaseManager
	{
		private const int MRULength=8;

		public override void AddItem(string itemName)
		{
			int pos;
			for (pos=0;pos<ItemList.Count;++pos)
			{
				string currentItem=(string)ItemList[pos];
				if (currentItem.ToLower()==itemName.ToLower())
				{
					ItemList.RemoveAt(pos);
					itemName=FindExistingSubPath(itemName);
					if (itemName!=string.Empty)
						ItemList.Insert(0,itemName);
					break;
				}
			}
			if (pos==ItemList.Count) // nicht gefunden
			{
				itemName=FindExistingSubPath(itemName);
				if (itemName==string.Empty)
					return;
				ItemList.Insert(0,itemName);
				if (ItemList.Count>MRULength)
					ItemList.RemoveAt(MRULength);
			}
		}

		/// <summary>
		/// Sucht rckwrts aus einem Pfadnamen einen exisitierenden Pfad
		/// </summary>
		/// <param name="path"></param>
		/// <returns></returns>
		public static string FindExistingSubPath(string path)
		{
			path=path.Trim();
			while (path!=string.Empty && !Directory.Exists(path))
			{
				int index=path.LastIndexOf('\\');
				if (index==-1)
					return string.Empty;
				path=path.Substring(0,index);
			}
			if (path.Length>0 && !path.EndsWith(@"\"))
				path+=@"\";
			return path;
			//return Regex.Replace(path,@"^(?<param>.+[^\\])$",@"${param}\");
		}

		public void FillRecentList(ComboBox comboList)
		{
			comboList.Items.Clear();
			comboList.Text=string.Empty;
			foreach (string path in Entries)
				comboList.Items.Add(path);
			if (Entries.Count>0)
				comboList.Text=(string)Entries[0];
		}

	}
}
