/*
 * Cuttermaran - The Mpeg Cutter
 * 
 * Copyright (C) 2003-2006 Tobias Arnold (cuttermaran@gmx.net)
 * 
 * This file is part of cuttermaran, a free MPEG cut program.
 * See http://www.cuttermaran.de for updates.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

using System;
using System.IO;
using System.Data;
using System.Drawing;
using System.Reflection;
using System.Collections;
using System.Diagnostics;
using System.Windows.Forms;
using System.Runtime.Serialization.Formatters.Binary;
using Configuration.data;
using StringResources;

using Coremoran.Algorithms;

namespace Configuration
{
	/// <summary>
	/// Summary description for Settings.
	/// </summary>
	public class Settings
	{
		private string settingsXml;
		private SettingsData data;
		private ExternalToolProviderData extTools;
		private const string guiVersion="1.66.2";
		private FilterGraphData customFilter;

		private int newAspectRatio; // Dieser Wert wird nicht mit abgespeichert!!!

		private static Settings currentSettings=new Settings();
		public  static Settings CurrentSettings
		{
			get
			{
				return currentSettings;
			}
		}

		public Settings()
		{
			// Einstellungen
			data=new SettingsData();
			settingsXml=Path.Combine(SettingsPath,"Main.Settings.xml");

#if BETA

			// If the beta version is started for the first time --> copy 
			// the settingsfile from final version to Main.Settings.Beta.xml
			string settingsBeta = Path.Combine(SettingsPath, "Main.Settings.Beta.xml");
			if (File.Exists(settingsXml) && !File.Exists(settingsBeta))
				File.Copy(settingsXml, settingsBeta);
			settingsXml = settingsBeta;

#endif
			if (File.Exists(settingsXml))
			{
				try
				{
					data.ReadXml(settingsXml);
					data.AcceptChanges();
				}
				catch
				{
					data=new SettingsData(); // Im Fehlerfall, alles neu!
				}
			}

			// Filtergraphen
			customFilter=new FilterGraphData();
			string filterXml=Path.Combine(SettingsPath,"CustomFilter.config");
			bool rewriteFilter=true;
			if (File.Exists(filterXml))
			{
				try
				{
					customFilter.ReadXml(filterXml);
					rewriteFilter=false;
				}
				catch
				{
					customFilter=new FilterGraphData(); // Im Fehlerfall, alles neu!
				}
			}
			if (rewriteFilter)
			{
				string xmlResource = "Configuration.data.FilterGraphDefault.xml";
				using (Stream s = System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(xmlResource))
				{
					customFilter.ReadXml(s);
				}
				customFilter.WriteXml(filterXml);
			}

			// EncoderProvider laden
			extTools=new ExternalToolProviderData();

			string s_Encoder = Path.Combine(LocalPath,@"Provider\encode");
			if (Directory.Exists(s_Encoder)) // this avoids a crash
			{
				string [] matchlist=Directory.GetFiles(s_Encoder,"*.configuration.xml");
				foreach (string config in matchlist)
					extTools.ReadXml(config);
			}

			string s_Muxer = Path.Combine(LocalPath,@"Provider\mux");
			if (Directory.Exists(s_Muxer)) // this avoids a crash
			{
				string [] matchlist=Directory.GetFiles(s_Muxer,"*.configuration.xml");
				foreach (string config in matchlist)
					extTools.ReadXml(config);
			}

			foreach (ExternalToolProviderData.ConfigurationRow confRow in extTools.Configuration)
				if (confRow.IsActive==false || (confRow.ToolType!="encoder" && confRow.ToolType!="muxer"))
					confRow.Delete();
		}

		public bool RestoreDefaults()
		{
			try
			{
				Directory.Delete(SettingsPath,true);
				return true;
			}
			catch
			{
				return false;
			}
		}

		#region Properties
		private string LocalPath
		{
			get
			{
				return Path.GetDirectoryName(Assembly.GetEntryAssembly().Location);
			}
		}

		public string SettingsPath
		{
			get
			{
				string sPath=Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData),"Cuttermaran");
				if (!Directory.Exists(sPath))
					Directory.CreateDirectory(sPath);
				return sPath;
			}
		}

		public ExternalToolProviderData ExternalTools
		{
			get
			{
				return extTools;
			}
		}

		public decimal SecCutOutSearch
		{
			get
			{
				return decimal.Parse(GetValueRow("SecCutOutSearch","45").value);
			}
			set
			{
				SetValueRow("SecCutOutSearch",value.ToString());
			}
		}

		public decimal SecCutPreview
		{
			get
			{
				return decimal.Parse(GetValueRow("SecCutPreview","5").value);
			}
			set
			{
				SetValueRow("SecCutPreview",value.ToString());
			}
		}

		public int SecQuickJumpStep
		{
			get
			{
				return int.Parse(GetValueRow("SecQuickJumpStep","5").value);
			}
			set
			{
				SetValueRow("SecQuickJumpStep",value.ToString());
			}
		}

		/// <summary>
		/// Show also the Cut-In picture after the selection in the cutlist has changed
		/// </summary>
		public bool ShowCutInPicture
		{
			get
			{
				return bool.Parse(GetValueRow("ShowCutInPicture",false.ToString()).value);
			}
			set
			{
				SetValueRow("ShowCutInPicture",value.ToString());
			}
		}

		public int SkipFramesPlay
		{
			get
			{
				return int.Parse(GetValueRow("SkipFramesPlay","0").value);
			}
			set
			{
				SetValueRow("SkipFramesPlay",value.ToString());
			}
		}

		public int SliderPlacement
		{
			get
			{
				return int.Parse(GetValueRow("VideoScroller.LargeChange","25").value);
			}
			set
			{
				SetValueRow("VideoScroller.LargeChange",value.ToString());
			}
		}

		public int CustomPlacementStep
		{
			get
			{
				return int.Parse(GetValueRow("CustomPlacementStep","100").value);
			}
			set
			{
				SetValueRow("CustomPlacementStep",value.ToString());
			}
		}

		public bool EncodeMode
		{
			get
			{
				return bool.Parse(GetValueRow("EncodeMode",false.ToString()).value);
			}
			set
			{
				SetValueRow("EncodeMode",value.ToString());
			}
		}

		public int Encode_Bitrate
		{
			get
			{
				return int.Parse(GetValueRow("Encode_Bitrate","0").value);
			}
			set
			{
				SetValueRow("Encode_Bitrate",value.ToString());
			}
		}

		public bool UseNTSCDropFrame
		{
			get
			{
				return bool.Parse(GetValueRow("UseNTSCDropFrame",true.ToString()).value);
			}
			set
			{
				SetValueRow("UseNTSCDropFrame",value.ToString());
			}
		}

		public string OpenDialogs_InitialDirectory
		{
			get
			{
				return GetValueRow("openDialogs.InitialDirectory","").value;
			}
			set
			{
				SetValueRow("openDialogs.InitialDirectory",value);
			}
		}

		public string ProjectDialog_InitialDirectory
		{
			get
			{
				return GetValueRow("ProjectDialog.InitialDirectory","").value;
			}
			set
			{
				SetValueRow("ProjectDialog.InitialDirectory",value);
			}
		}

		public string SaveFrameDialog_InitialDirectory
		{
			get
			{
				return GetValueRow("saveFrameDialog.InitialDirectory","").value;
			}
			set
			{
				SetValueRow("saveFrameDialog.InitialDirectory",value);
			}
		}

		public string Language
		{
			get
			{
				return GetValueRow("Language","en").value;
			}
			set
			{
				SetValueRow("Language",value);
			}
		}

		public bool MuxingMode
		{
			get
			{
				return bool.Parse(GetValueRow("MuxingMode",false.ToString()).value);
			}
			set
			{
				SetValueRow("MuxingMode",value.ToString());
			}
		}

		public string UsedEncoder
		{
			get
			{
				return GetValueRow("UsedEncoder","").value;
			}
			set
			{
				SetValueRow("UsedEncoder",value);
			}
		}

		public string UsedMuxer
		{
			get
			{
				return GetValueRow("UsedMuxer","mplex").value;
			}
			set
			{
				SetValueRow("UsedMuxer",value);
			}
		}

		public int MPEG2MuxTarget
		{
			get
			{
				return int.Parse(GetValueRow("MPEG2MuxTarget",1.ToString()).value);
			}
			set
			{
				SetValueRow("MPEG2MuxTarget",value.ToString());
			}
		}

		public ExternalToolProviderData.ConfigurationRow UsedEncoderInfo
		{
			get
			{
				return extTools.Configuration.FindByDisplayName(UsedEncoder);
			}
		}

		public ExternalToolProviderData.ConfigurationRow UsedMuxerInfo
		{
			get
			{
				return extTools.Configuration.FindByDisplayName(UsedMuxer);
			}
		}

		public string MuxOutputFolder
		{
			get
			{
				ArrayList al=MRUMuxPathEntries;
				if (al.Count==0)
					return string.Empty;
				else
					return (string)al[0];
			}
		}

		public bool MuxPause
		{
			get
			{
				return bool.Parse(GetValueRow("MuxPause",true.ToString()).value);
			}
			set
			{
				SetValueRow("MuxPause",value.ToString());
			}
		}

		public bool UseUpdateDetection
		{
			get
			{
				return bool.Parse(GetValueRow("UseUpdateDetection",false.ToString()).value);
			}
			set
			{
				SetValueRow("UseUpdateDetection",value.ToString());
			}
		}

		public DateTime LastUpdateLookup
		{
			get
			{
				return DateTime.Parse(GetValueRow("LastUpdateLookup",new DateTime().ToString()).value);
			}
			set
			{
				SetValueRow("LastUpdateLookup",value.ToString());
			}
		}

		public bool CreateInfoFile
		{
			get
			{
				return bool.Parse(GetValueRow("CreateInfoFile",true.ToString()).value);
			}
			set
			{
				SetValueRow("CreateInfoFile",value.ToString());
			}
		}

		public int FirstSequence
		{
			get
			{
				return int.Parse(GetValueRow("FirstSequence",1.ToString()).value);
			}
			set
			{
				SetValueRow("FirstSequence",value.ToString());
			}
		}

		public bool CreateDVDCompliantStream
		{
			get
			{
				return bool.Parse(GetValueRow("CreateDVDCompliantStream",false.ToString()).value);
			}
			set
			{
				SetValueRow("CreateDVDCompliantStream",value.ToString());
			}
		}

		public bool OnMuxDeleteES
		{
			get
			{
				return bool.Parse(GetValueRow("OnMuxDeleteES",false.ToString()).value);
			}
			set
			{
				SetValueRow("OnMuxDeleteES",value.ToString());
			}
		}

		public int SequenceEndCode
		{
			get
			{
				return int.Parse(GetValueRow("SequenceEndCode",1.ToString()).value);
			}
			set
			{
				SetValueRow("SequenceEndCode",value.ToString());
			}
		}

		public ArrayList MRUFileEntries
		{
			get
			{
				ArrayList result=new ArrayList();
				foreach (SettingsData.RecentFilesRow rfr in data.RecentFiles.Rows)
					result.Add(rfr.value);
				return result;
			}
			set
			{
				SaveList(value,data.RecentFiles);
			}
		}

		public ArrayList MRUMuxPathEntries
		{
			get
			{
				ArrayList result=new ArrayList();
				foreach (SettingsData.RecentMuxPathsRow rfr in data.RecentMuxPaths.Rows)
				{
					string path=MRUPathManager.FindExistingSubPath(rfr.value);
					if (path!=string.Empty && !result.Contains(path))
						result.Add(path);
				}
				return result;
			}
			set
			{
				SaveList(value,data.RecentMuxPaths);
			}
		}

		public ArrayList MRUOutputPathsEntries
		{
			get
			{
				ArrayList result=new ArrayList();
				foreach (SettingsData.RecentOutputPathsRow rfr in data.RecentOutputPaths.Rows)
				{
					string path=MRUPathManager.FindExistingSubPath(rfr.value);
					if (path!=string.Empty && !result.Contains(path))
						result.Add(path);
				}
				return result;
			}
			set
			{
				SaveList(value,data.RecentOutputPaths);
			}
		}


		public bool DisplayFullPathInformations
		{
			get
			{
				return bool.Parse(GetValueRow("DisplayFullPathInformations",false.ToString()).value);
			}
			set
			{
				SetValueRow("DisplayFullPathInformations",value.ToString());
			}
		}

		public bool CreateD2VFile
		{
			get
			{
				return bool.Parse(GetValueRow("CreateD2VFile",false.ToString()).value);
			}
			set
			{
				SetValueRow("CreateD2VFile",value.ToString());
			}
		}

		public string D2VFileType
		{
			get
			{
				return GetValueRow("D2VFileType",d2vBaseWriter.Types.dgIndexV13).value;
			}
			set
			{
				SetValueRow("D2VFileType",value);
			}
		}

		public string TempDir
		{
			get
			{
				string path=GetValueRow("TempDir",Path.GetTempPath()).value;
				if (!Directory.Exists(path))
				{
					path=Path.GetTempPath();
					TempDir=path;
				}
				return path;
			}
			set
			{
				SetValueRow("TempDir",value);
			}
		}

		public bool IfoEditChapters
		{
			get
			{
				return bool.Parse(GetValueRow("IfoEditChapters",false.ToString()).value);
			}
			set
			{
				SetValueRow("IfoEditChapters",value.ToString());
			}
		}

		public bool MaestroChapters
		{
			get
			{
				return bool.Parse(GetValueRow("MaestroChapters",false.ToString()).value);
			}
			set
			{
				SetValueRow("MaestroChapters",value.ToString());
			}
		}

		public bool ScenaristChapters
		{
			get
			{
				return bool.Parse(GetValueRow("ScenaristChapters",false.ToString()).value);
			}
			set
			{
				SetValueRow("ScenaristChapters",value.ToString());
			}
		}

		public bool CCEChapters
		{
			get
			{
				return bool.Parse(GetValueRow("CCEChapters",false.ToString()).value);
			}
			set
			{
				SetValueRow("CCEChapters",value.ToString());
			}
		}

		public bool OGGChapters
		{
			get
			{
				return bool.Parse(GetValueRow("OGGChapters",false.ToString()).value);
			}
			set
			{
				SetValueRow("OGGChapters",value.ToString());
			}
		}

		public bool DVDAuthorChapters
		{
			get
			{
				return bool.Parse(GetValueRow("DVDAuthorChapters",false.ToString()).value);
			}
			set
			{
				SetValueRow("DVDAuthorChapters",value.ToString());
			}
		}

		public bool AutoCreateChapters
		{
			get
			{
				return bool.Parse(GetValueRow("AutoCreateChapters",false.ToString()).value);
			}
			set
			{
				SetValueRow("AutoCreateChapters",value.ToString());
			}
		}

		public int AutoChaptersValue
		{
			get
			{
				return int.Parse(GetValueRow("AutoChaptersValue",5.ToString()).value);
			}
			set
			{
				SetValueRow("AutoChaptersValue",value.ToString());
			}
		}

		public bool RecalcBitrate
		{
			get
			{
				return bool.Parse(GetValueRow("RecalcBitrate",false.ToString()).value);
			}
			set
			{
				SetValueRow("RecalcBitrate",value.ToString());
			}
		}

		public bool RecalcSourceBitrate
		{
			get
			{
				return bool.Parse(GetValueRow("RecalcSourceBitrate",false.ToString()).value);
			}
			set
			{
				SetValueRow("RecalcSourceBitrate",value.ToString());
			}
		}

		public int SDEaction
		{
			get
			{
				return int.Parse(GetValueRow("SDEaction",0.ToString()).value);
			}
			set
			{
				SetValueRow("SDEaction",value.ToString());
			}		
		}

		public ConversionType TempAVIColorEncoding
		{
			get
			{
				string val=GetValueRow("TempAVIColorEncoding",ConversionType.RGB24.ToString()).value;
				switch (val)
				{
					case "YV12":
						return ConversionType.YV12;
					case "RGB24":
					default:
						return ConversionType.RGB24;
				}
			}
			set
			{
				SetValueRow("TempAVIColorEncoding",value.ToString());
			}		
		}

		public bool ShowCurrentFrameCaption
		{
			get
			{
				return bool.Parse(GetValueRow("CurrentFrameCaption",true.ToString()).value);
			}
			set
			{
				SetValueRow("CurrentFrameCaption",value.ToString());
			}
		}

		public int QuickJumpX
		{
			get
			{
				return int.Parse(GetValueRow("QuickJumpX",(-1).ToString()).value);
			}
			set
			{
				SetValueRow("QuickJumpX",value.ToString());
			}
		}

		public int QuickJumpY
		{
			get
			{
				return int.Parse(GetValueRow("QuickJumpY",(-1).ToString()).value);
			}
			set
			{
				SetValueRow("QuickJumpY",value.ToString());
			}
		}

		public bool ApplyAspectRatio
		{
			get
			{
				return bool.Parse(GetValueRow("ApplyAspectRatio",false.ToString()).value);
			}
			set
			{
				SetValueRow("ApplyAspectRatio",value.ToString());
			}
		}

		public bool AdditionalScrollBar
		{
			get
			{
				return bool.Parse(GetValueRow("AdditionalScrollBar",false.ToString()).value);
			}
			set
			{
				SetValueRow("AdditionalScrollBar",value.ToString());
			}
		}

		public bool AllowGUIResize
		{
			get
			{
				return bool.Parse(GetValueRow("AllowGUIResize",true.ToString()).value);
			}
			set
			{
				SetValueRow("AllowGUIResize",value.ToString());
			}
		}

		public bool AllowGUIFloating
		{
			get
			{
				return bool.Parse(GetValueRow("AllowGUIFloating",false.ToString()).value);
			}
			set
			{
				SetValueRow("AllowGUIFloating",value.ToString());
			}
		}

		public bool DPIRescaling
		{
			get
			{
				return bool.Parse(GetValueRow("DPIRescaling",true.ToString()).value);
			}
			set
			{
				SetValueRow("DPIRescaling",value.ToString());
			}
		}
		
		public bool ForceCutAndClose
		{
			get
			{
				return bool.Parse(GetValueRow("ForceCutAndClose",false.ToString()).value);
			}
			set
			{
				SetValueRow("ForceCutAndClose",value.ToString());
			}
		}

		public int NewAspectRation
		{
			get
			{
				return (newAspectRatio==0)?-1:newAspectRatio;
			}
			set
			{
				newAspectRatio=value;
			}
		}

		#endregion

		#region Presets

		public class GUIPresetData
		{
			public Point pt;
			public Size sz;
			public byte [] data;
			private Hashtable customSettings;

			public GUIPresetData(Point p,Size s,byte [] d,byte [] privateData)
			{
				pt=p;
				sz=s;
				data=d;
				if (privateData==null)
					customSettings=new Hashtable();
				else
					PrivateData=privateData;
			}

			public object this [string key]
			{
				get
				{
					return customSettings[key];
				}
				set
				{
					customSettings[key]=value;
				}
			}

			public byte [] PrivateData
			{
				get
				{
					using (MemoryStream ms=new MemoryStream())
					{
						BinaryFormatter bf=new BinaryFormatter();
						bf.Serialize(ms,customSettings);
						return ms.ToArray();
					}
				}
				set
				{
					using (MemoryStream ms=new MemoryStream(value))
					{
						BinaryFormatter bf=new BinaryFormatter();
						customSettings=(Hashtable)bf.Deserialize(ms);
					}
				}
			}
		}

		public string PresetCurrentName
		{
			get
			{
				return "current";
			}
		}

		public GUIPresetData LoadInternalGUIPreset(string name)
		{
			SettingsData sd=new SettingsData();
			string xmlResource = "Configuration.data.GUIPresets.xml";

			using (Stream s = System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(xmlResource))
			{
				sd.ReadXml(s);
			}
			SettingsData.GUISettingsRow row=sd.GUISettings.FindByName(name);
			return new GUIPresetData(new Point(-1,-1),new Size(row.SizeX,row.SizeY),row.MagicData,row.PrivateData);
		}

		public void SaveGUIPreset(string name,GUIPresetData gpd)
		{
			SettingsData.GUISettingsRow row=data.GUISettings.FindByName(name);
			if (row!=null)
			{
				row.LocationX=gpd.pt.X;
				row.LocationY=gpd.pt.Y;
				row.SizeX=gpd.sz.Width;
				row.SizeY=gpd.sz.Height;
				row.MagicData=gpd.data;
				row.PrivateData=gpd.PrivateData;
				row.Version=guiVersion;
			}
			else
				data.GUISettings.AddGUISettingsRow(name,gpd.pt.X,gpd.pt.Y,gpd.sz.Width,gpd.sz.Height,guiVersion,gpd.data,gpd.PrivateData);
		}

		public GUIPresetData LoadGUIPreset(string name)
		{
			try
			{
				SettingsData.GUISettingsRow row=data.GUISettings.FindByName(name);
				if (row!=null && !row.IsVersionNull() && row.Version==guiVersion)
					return new GUIPresetData(new Point(row.LocationX,row.LocationY),new Size(row.SizeX,row.SizeY),row.MagicData,row.PrivateData);
				else
					return new GUIPresetData(new Point(0,0),new Size(100,100),null,null);
			}
			catch
			{
				return new GUIPresetData(new Point(0,0),new Size(100,100),null,null);
			}
		}

		public void DeletePreset(string name)
		{
			SettingsData.GUISettingsRow row=data.GUISettings.FindByName(name);
			if (row!=null)
				row.Delete();
		}

		public string [] GetPresets()
		{
			string filter=string.Format("name<>'current' and Version='{0}'",guiVersion);
			DataRow [] rows=data.GUISettings.Select(filter,"name asc");
			string [] result=new string [rows.Length];
			for (int index=0;index<rows.Length;++index)
				result[index]=((SettingsData.GUISettingsRow)rows[index]).Name;
			return result;
		}

		#endregion

		#region Methods

		public FilterGraphData.MediaRow GetFilter(FilterMediaType fmt)
		{
			FilterGraphData.MediaRow result=customFilter.Media.FindBytype(fmt.ToString());

			// Enabled?
			if (result!=null && !result.IsenabledNull() && result.enabled==false)
				result=null;

			// Filter/Connections vorhanden?
			if (result!=null && (result.GetconnectRows().Length==0 || result.GetFILTERRows().Length==0))
				result=null;

			return result;
		}

		#endregion

		#region Helper

		public void CommitChanges()
		{
			if (data.HasChanges())
			{
#if DEBUG
				try
				{
#endif
					data.AcceptChanges();

					// Existiert Cuttermaran Folder?
					if (!Directory.Exists(Path.GetDirectoryName(settingsXml)))
						Directory.CreateDirectory(Path.GetDirectoryName(settingsXml));
					data.WriteXml(settingsXml,XmlWriteMode.IgnoreSchema);
#if DEBUG
				}
				catch
				{
					MessageBox.Show("Error writing Settings");
				}
#endif
			}
		}

		private SettingsData.ConfigurationValueRow GetValueRow(string name,string defaultValue)
		{
			SettingsData.ConfigurationValueRow row=data.ConfigurationValue.FindBykey(name);
			if (row==null)
			{
				row=data.ConfigurationValue.NewConfigurationValueRow();
				row.key=name;
				row.value=defaultValue;
				data.ConfigurationValue.AddConfigurationValueRow(row);
			}
			return row;
		}

		private void SetValueRow(string name,string Value)
		{
			SettingsData.ConfigurationValueRow row=data.ConfigurationValue.FindBykey(name);
			if (row==null)
				data.ConfigurationValue.AddConfigurationValueRow(name,Value);
			else
				if (row.value!=Value)
				row.value=Value;
		}

		public void FillCombo(ComboBox combo,string filter,Button configBtn,CheckBox enableBox, string usedTool)
		{
			DataView filterView  = new DataView(extTools.Configuration);
			filterView.RowFilter = "ToolType=" + filter;

			// ATTENTION:
			// Do NOT use combo.DataSource=filterView here!
			// Bug in .NET framework: Although combo.SelectedIndex=-1 -> Index 0 will be displayed!!!
			// It is very important to display an empty ComboBox if the user has set
			// ImagoMuxer in XML, but the file ImagoMPGMuxerProvider.dll does not exist for any reason!
			// Otherwise the Combobox will switch to "mplex" (the first in the list)
			// BUT when cutting Cuttermaran will try to start the not existing Imago Muxer
			// ALTHOUGH the combobox displayed "mplex" --> very confusing !!
			// Additionally the user will receive EXTREMELY confusing error messages!
			// The checkbox for muxing / encoding is turned off on an invalid Encoder/Muxer
			foreach (DataRowView row in filterView)
				combo.Items.Add(row["DisplayName"]);

			combo.SelectedIndex = -1; // nothing selected in ComboBox!!
			configBtn.Enabled   = false;

			ExternalToolProviderData.ConfigurationRow confRow=extTools.Configuration.FindByDisplayName(usedTool);
			if (confRow!=null)
			{
				combo.SelectedItem =  confRow.DisplayName;
				configBtn.Enabled  = !confRow.IsConfigMethodNull();
			}
			else enableBox.Checked = false; // disable muxing / encoding
		}

		public void CallExternalToolConfig(string toolName, Form i_Owner)
		{
			ExternalToolProviderData.ConfigurationRow confRow=extTools.Configuration.FindByDisplayName(toolName);
			if (confRow!=null)
			{
				try
				{
					string relpath;
					if (confRow.ToolType=="encoder")
						relpath=@"Provider\encode";
					else
						relpath=@"Provider\mux";
					if (confRow.IsAssembly)
					{
						// .NET Assembly in AppDomain laden
						AppDomain ad=AppDomain.CurrentDomain;
						ad.AppendPrivatePath(relpath);
						Assembly bridge=ad.Load(confRow.Executive);
						object connector=bridge.CreateInstance(confRow.ClassName);

						// Set the owner window for dialogs and messageboxes
						MethodInfo f_SetOwner = connector.GetType().GetMethod("SetOwnerWindow");
						if (f_SetOwner != null)
							f_SetOwner.Invoke(connector, new object[] {i_Owner});
						else // Display error message only once per provider
							DisplayOldProviderError(i_Owner, bridge, confRow.Executive, confRow.DisplayName);

						MethodInfo config=connector.GetType().GetMethod(confRow.ConfigMethod);
						config.Invoke(connector,new object [] {});
						ad.ClearPrivatePath();
					}
					else
					{
						string executableFile=Path.Combine(Path.Combine(LocalPath,relpath),confRow.Executive);
						string cmdArgs=string.Format("{0} {1}",confRow.ConfigMethod,System.Threading.Thread.CurrentThread.CurrentUICulture.Name);

						System.Diagnostics.Debug.WriteLine(string.Format("ExternalToolConfig: {0}, CmdLine: {1}",
						                                   executableFile, cmdArgs));

						// Native executable
						Process start=Process.Start(executableFile, cmdArgs);
						start.WaitForExit();
					}
				}
				catch (Exception exp)
				{
					MessageBox.Show(i_Owner, exp.Message,Resource.CurrentResource[Resource.ERROR]);
				}
			}
		}

		// stores the old providers which have already displayed their error message
		private static ArrayList mi_ProviderErrors = new ArrayList();

		/// <summary>
		/// Displays an error if an old version of the provider DLL is used
		/// The error will be displayed only once
		/// </summary>
		public static void DisplayOldProviderError(Form i_Owner, Assembly i_Ass, string s_Executive, string s_DisplayName)
		{
			if (mi_ProviderErrors.Contains(s_Executive.ToUpper()))
				return;
			
			mi_ProviderErrors.Add(s_Executive.ToUpper());
			string s_Format;
			switch (s_Executive.ToUpper())
			{
				case "MPLEXPROVIDER":
				case "TMPGENCPROVIDER":
				case "QUENCPROVIDER":
				case "IMAGOMPGMUXERPROVIDER":
					s_Format = Resource.OLD_PROVIDER1;
					break;
				default:
					s_Format = Resource.OLD_PROVIDER2;
					break;
			}
			string s_Msg = string.Format(Resource.CurrentResource[s_Format], s_DisplayName, Path.GetFileName(i_Ass.Location));
			MessageBox.Show(i_Owner, s_Msg, "Provider");
		}

		private void SaveList(ArrayList entries,DataTable dest)
		{
			if (data.Lists.Count==0)
				data.Lists.AddListsRow();
			dest.Clear();
			foreach (string val in entries)
				dest.Rows.Add(new object [] {val,0});
		}

		#endregion

	}
}
