/*
 * Cuttermaran - The Mpeg Cutter
 * 
 * Copyright (C) 2003,2004  Tobias Arnold (cuttermaran@gmx.net)
 * 
 * This file is part of cuttermaran, a free MPEG cut program.
 * See http://cuttermaran.kickme.to for updates.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

using System;
using System.IO;
using System.Data;
using System.Text;
using System.Diagnostics;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace TMPGEncProvider
{
	/// <summary>
	/// Summary description for Connector.
	/// </summary>
	public class Connector
	{
		private Form    mi_OwnerForm;
		private Process mi_Process;

		private static decimal [] Mpeg1RatioData=new decimal []
		{
			1.0m, 1.0m, 0.6735m, 0.7031m,
			0.7615m,0.8055m,0.8437m,0.8935m,
			0.9157m,0.9815m,1.0255m,1.0695m,
			1.095m,1.1575m,1.2051m,1m
		};

		private static byte [,] zickZackMatrix = new byte [8,8]
		{
			{ 0, 1, 5, 6,14,15,27,28},
			{ 2, 4, 7,13,16,26,29,42},
			{ 3, 8,12,17,25,30,41,43},
			{ 9,11,18,24,31,40,44,53},
			{10,19,23,32,39,45,52,54},
			{20,22,33,38,46,51,55,60},
			{21,34,37,47,50,56,59,61},
			{35,36,48,49,57,58,62,63}
		};

		public Connector()
		{
		}

		/// <summary>
		/// Abort the encoding process
		/// This function is called by reflection
		/// </summary>
		public void AbortAction()
		{
			if (mi_Process != null)
				mi_Process.Kill();
		}

		/// <summary>
		/// returns the current progress (0-100)
		/// This function is called by reflection
		/// </summary>
		public int GetProgress()
		{
			// not supported by TmpgEnc
			return 0;
		}

		/// <summary>
		/// This will set CMMain or Confirguration.Settings as parent for all dialogs / messageboxes
		/// This function is called by reflection
		/// </summary>
		public void SetOwnerWindow(Form i_Owner)
		{
			mi_OwnerForm = i_Owner;
		}

		/// <summary>
		/// This function is called by reflection
		/// </summary>
		public int Encode(DataSet data)
		{
			DataRow dr=null;
			try
			{
				dr=data.Tables[0].Rows[0];

				TMPGEncProjectFileCreator tprCreator;

				if ((bool)dr["IsMpeg2"]) // MPEG2
					tprCreator=new TMPGEncProjectFileCreator(Settings.TMPGEnc_Mpeg2Template);
				else
					tprCreator=new TMPGEncProjectFileCreator(Settings.TMPGEnc_Mpeg1Template);

				tprCreator.SetParameter("Job.MPEG.Video.Width",dr["Width"]);
				tprCreator.SetParameter("Job.MPEG.Video.Height",dr["Height"]);
	
				// 1=23.976 fps, 2=24 fps, 3=25 fps, 4=29.97 fps
				// 5=30 fps, 6=50 fps, 7=59.94 fps, 8=60 fps
				tprCreator.SetParameter("Job.MPEG.Video.FrameRate",dr["FrameRate"]);

				// MPEG1: 1=1:1, 12=4:3 NTSC, 8=4:3 PAL, 6=16:9 NTSC, 3=16:9 PAL
				// MPEG2: 1=1:1, 2=4:3, 3=16:9, 4=2.11:1
				if ((bool)dr["IsMpeg2"]) // MPEG2
					tprCreator.SetParameter("Job.MPEG.Video.AspectRatio",dr["AspectRatio"]);
				else
				{
					byte ratio=1;
					if ((short)dr["AspectRatio"]>1) // ungleich 1:1
					{
						decimal xyratio=(short)dr["Width"]*Mpeg1RatioData[(short)dr["AspectRatio"]]/(short)dr["Height"];
						if (0.745m<xyratio && xyratio<0.755m) // 4:3
							ratio=(byte)(((short)dr["VideoFormat"]==2)?0x0C:0x08); // NTSC:PAL
						if (0.5575m<xyratio && xyratio<0.5675m) // 16:9
							ratio=(byte)(((short)dr["VideoFormat"]==2)?0x06:0x03); // NTSC:PAL
						if (0.4475m<xyratio && xyratio<0.4575m) // 2.21:1
							ratio=1; // Unsupported from TMPGEnc
					}
					tprCreator.SetParameter("Job.MPEG.Video.AspectRatio",ratio);
				}
				tprCreator.SetParameter("Job.MPEG.Video.CBR_BitRate",dr["BitRate"]);

				tprCreator.SetParameter("Job.MPEG.Video.VBVBufferSize",dr["VBVBuffer"]);
		
				// 1=PAL, 2=NTSC
				tprCreator.SetParameter("Job.MPEG.Video.VideoFormat",dr["VideoFormat"]);
		
				if ((bool)dr["IsMpeg2"]) // MPEG2
				{
					// Interlace oder Progressive
					string EncodeMode;
					if ((bool)dr["IsProgressive"])
						EncodeMode="MPEGVideoEncoder_VideoEncodeMode_Progressive";
					else
						EncodeMode="MPEGVideoEncoder_VideoEncodeMode_Interlace";
					tprCreator.SetParameter("Job.MPEG.Video.VideoEncodeMode",EncodeMode);
				}
				// MPEG1: 01=1:1, 0c=4:3 NTSC, 2c=4:3 NTSC 704, 08=4:3 PAL, 28=4:3 PAL 704, 06=16:9 NTSC, 03=16:9 PAL
				// MPEG2: 01=1:1, 0d=4:3, 0e=16:9, 0f=2.11:1
				if ((bool)dr["IsMpeg2"]) // MPEG2
				{
					byte ratio=(byte)((short)dr["AspectRatio"]+0x0b);
					if (ratio==0x0c)
						ratio=0x01; // 1:1
					tprCreator.SetParameter("Video.AspectRatio",ratio);
				}
				else
				{
					byte ratio=1;
					if ((short)dr["AspectRatio"]>1) // ungleich 1:1
					{
						decimal xyratio=(short)dr["Width"]*Mpeg1RatioData[(short)dr["AspectRatio"]]/(short)dr["Height"];
						if (0.745m<xyratio && xyratio<0.755m) // 4:3
							ratio=(byte)(((short)dr["VideoFormat"]==2)?0x0C:0x08); // NTSC:PAL
						if (0.5575m<xyratio && xyratio<0.5675m) // 16:9
							ratio=(byte)(((short)dr["VideoFormat"]==2)?0x06:0x03); // NTSC:PAL
						if (0.4475m<xyratio && xyratio<0.4575m) // 2.21:1
							ratio=1; // Unsupported from TMPGEnc
					}
					tprCreator.SetParameter("Video.AspectRatio",ratio);
				}
				tprCreator.SetParameter("Video.InterlaceImage",!(bool)dr["IsProgressive"]);
				tprCreator.SetParameter("Job.MPEG.Video.DC_prec",dr["intra_dc_precision"]);
				tprCreator.SetParameter("Video.TopFieldFirst",dr["IsTopFieldFirst"]);
				tprCreator.SetParameter("Video.SourceFileName",dr["AVIFile"]);

				// Quantisierungs Matrizen
				if (!dr.IsNull("intra_quantizer_matrix"))
				{
					byte [] matrix=new byte[64];
					for (int y=0;y<8;++y)
						for (int x=0;x<8;++x)
							matrix[y*8+x]=((byte [])dr["intra_quantizer_matrix"])[zickZackMatrix[x,y]];
					tprCreator.SetParameter("Job.MPEG.Video.QuantizeMatrix_Intra",matrix);
				}
				if (!dr.IsNull("non_intra_quantizer_matrix"))
				{
					byte [] matrix=new byte[64];
					for (int y=0;y<8;++y)
						for (int x=0;x<8;++x)
							matrix[y*8+x]=((byte [])dr["non_intra_quantizer_matrix"])[zickZackMatrix[x,y]];
					tprCreator.SetParameter("Job.MPEG.Video.QuantizeMatrix_NonIntra",matrix);
				}

				tprCreator.SetParameter("Job.OutputFileName",dr["ResultMPGFile"]);
				tprCreator.Write(Path.ChangeExtension((string)dr["AVIFile"],".tpr"));

				string s_Cmd = string.Format("\"{0}\" /encode /close",Path.ChangeExtension((string)dr["AVIFile"],".tpr"));

				// Open TMPGenc hidden if user has defined this in configuration settings
				// OR if the main window is minimized
				bool b_Hidden = (Settings.SilentMode || (bool)dr["Unattended"] ||
				     (mi_OwnerForm != null && mi_OwnerForm.WindowState == FormWindowState.Minimized));
                
				// TmpgEnc completely ignores ProcessStartInfo.WindowStyle, so we have to
				// inject ProcessStarter.DLL into TmpgEnc to force it to hide its window!
				mi_Process = StartProcessEx(Settings.TMPGEncExe, s_Cmd, b_Hidden);
				if (mi_Process != null)
				{
					mi_Process.WaitForExit();
					mi_Process = null; // for AbortAction()
				}
				return 0;
			}
			catch (Exception ex)
			{
				if (dr!=null && !(bool)dr["Unattended"])
					MessageBox.Show(mi_OwnerForm, ex.Message,"TMPGEncProvider");
				return -1;
			}
		}

		public void Config()
		{
			Settings settings=new Settings();
			settings.ShowDialog(mi_OwnerForm);
		}

		// -------------- Start process hidden --------------------

		// used for Windows 98, ME
		[DllImport("ProcessStarter.dll", EntryPoint="StartProcessHiddenA", CharSet=CharSet.Ansi)]
		private static extern int StartProcessHiddenA(string s_Path, string s_CmdLine, bool b_WaitForExit, out int s32_ProcessID, out int s32_ExitCode);
		[DllImport("ProcessStarter.dll", EntryPoint="ExplainAPIErrorA", CharSet=CharSet.Ansi)]
		private static extern void ExplainAPIErrorA(int s32_ErrorCode, StringBuilder s_Buffer, int s32_BufSize);

		// used for Windows NT, 2000, XP...
		[DllImport("ProcessStarter.dll", EntryPoint="StartProcessHiddenW", CharSet=CharSet.Unicode)]
		private static extern int StartProcessHiddenW(string s_Path, string s_CmdLine, bool b_WaitForExit, out int s32_ProcessID, out int s32_ExitCode);
		[DllImport("ProcessStarter.dll", EntryPoint="ExplainAPIErrorW", CharSet=CharSet.Unicode)]
		private static extern void ExplainAPIErrorW(int s32_ErrorCode, StringBuilder s_Buffer, int s32_BufSize);

		/// <summary>
		/// This function can be used by any provider (TMPGEnc, ImagoMuxer...)
		/// Starts a process normally or hidden.
		/// b_Hidden=true: All toplevel windows which have a minimize button will be hidden.
		/// returns a Process instance or null if the process already exited
		/// </summary>
		static public Process StartProcessEx(string s_Path, string s_CmdLine, bool b_Hidden)
		{
			Debug.WriteLine(string.Format("Starting: {0}, CmdLine: {1}, Hidden {2}", s_Path, s_CmdLine, b_Hidden));

			if (!b_Hidden) // Start normally using .NET framework
				return Process.Start(s_Path, s_CmdLine);

			// Ansi for Windows 98 and ME, otherwise Unicode (NT, 2000, XP...)
			bool  b_Unicode = (Environment.OSVersion.Platform == PlatformID.Win32NT);
			int s32_ProcessID;
			int s32_Err;
			int s32_ExitCode; // not used here because b_WaitForExit = false

			if (b_Unicode) s32_Err = StartProcessHiddenW(s_Path, s_CmdLine, false, out s32_ProcessID, out s32_ExitCode);
			else           s32_Err = StartProcessHiddenA(s_Path, s_CmdLine, false, out s32_ProcessID, out s32_ExitCode);

			if (s32_Err != 0)
			{
				StringBuilder s_Error = new StringBuilder(2000);
				if (b_Unicode) ExplainAPIErrorW(s32_Err, s_Error, s_Error.Capacity);
				else           ExplainAPIErrorA(s32_Err, s_Error, s_Error.Capacity);

				throw new Exception(s_Error +"\n"+ s_Path);
			}
		
			try   { return Process.GetProcessById(s32_ProcessID); }
			catch { return null; } // may fail if process has already exited
		}
	}
}
