/*
 * Cuttermaran - The Mpeg Cutter
 * 
 * Copyright (C) 2003-2005  Tobias Arnold (cuttermaran@gmx.net)
 * 
 * This file is part of cuttermaran, a free MPEG cut program.
 * See http://www.cuttermaran.de for updates.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#include <windows.h>
#include <io.h>
#include <fcntl.h>
extern "C"
{
#include "inttypes.h"
#include "mpeg2.h"
#include "mpeg2convert.h"
}
#include "libmpeg2dll.h"
#include "global.h"

#include <string>
#include <map>

using namespace std;

typedef map<string,DecoderContainer *> DecoderMapType;
static DecoderMapType decoderMap;

// Interne Hilfsfunktionen
static int NextFrameLoop(DecoderContainer *pdc);
static void InternalClose(DecoderContainer *pdc);
static DecoderContainer * FindDecoder(string handle)
{
	DecoderMapType::iterator item;
	item=decoderMap.find(handle);
	if (item!=decoderMap.end())
		return item->second;
	else
		return 0;
}

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
	switch(ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
			break;
	    case DLL_THREAD_ATTACH:
			break;
		case DLL_THREAD_DETACH:
			break;
		case DLL_PROCESS_DETACH:
			for (DecoderMapType::iterator it=decoderMap.begin();it!=decoderMap.end();++it)
				InternalClose(it->second);
			break;
	}
    return TRUE;
}

static char * openMessage="libmpeg2 - Open %s Handle %x";
static char * closeMessage="libmpeg2 - Closing %x";
static char charBuffer[1024];

__declspec(dllexport) int __stdcall libMpeg2_Open(const char *keyName,const char *filename,enum ConversionType ct)
{
	DecoderContainer *pdc=FindDecoder(keyName);
	if (pdc!=0)
		libMpeg2_Close(keyName);

	pdc=new DecoderContainer();
	decoderMap.insert(DecoderMapType::value_type(keyName,pdc));

	pdc->stream_end_read=0;
	pdc->filehandle=CreateFile(filename,GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,0,0);
	if (pdc->filehandle==INVALID_HANDLE_VALUE)
	{
		pdc->filehandle=0;
		return 0;
	}

	#ifdef _DEBUG
		sprintf(charBuffer,openMessage,filename,pdc->filehandle);
		OutputDebugString(charBuffer);
	#endif

	pdc->conv_type=ct;

	pdc->mpeg2dec = mpeg2_init ();
    if (pdc->mpeg2dec == NULL)
	{
		libMpeg2_Close(keyName);
		return 0;
	}
    pdc->info = mpeg2_info (pdc->mpeg2dec);
	if (NextFrameLoop(pdc)==0)
	{
		libMpeg2_Close(keyName);
		return 0;
	}
	return 1;

}

static void InternalClose(DecoderContainer *pdc)
{
	if (pdc->filehandle!=0)
	{
		#ifdef _DEBUG
			sprintf(charBuffer,closeMessage,pdc->filehandle);
			OutputDebugString(charBuffer);
		#endif

		mpeg2_close(pdc->mpeg2dec);
		CloseHandle(pdc->filehandle);
	}
	pdc->filehandle=0;

	delete pdc;
}

__declspec(dllexport) int __stdcall libMpeg2_Close(const char *keyName)
{
	DecoderContainer *pdc=FindDecoder(keyName);
	if (pdc==0)
		return 0;

	InternalClose(pdc);

	decoderMap.erase(keyName);

	return 1;
}

__declspec(dllexport) struct TFrameInfo * __stdcall libMpeg2_GetCurrentFrame(const char *keyName)
{
	DecoderContainer *pdc=FindDecoder(keyName);
	if (pdc==0)
		return 0;

	if (pdc->filehandle==0 || pdc->filehandle==INVALID_HANDLE_VALUE)
		return 0;
	return pdc->p_frameInfo;
}

__declspec(dllexport) int __stdcall libMpeg2_NextFrame(const char *keyName)
{
	DecoderContainer *pdc=FindDecoder(keyName);
	if (pdc==0)
		return 0;

	if (pdc->filehandle==0 || pdc->filehandle==INVALID_HANDLE_VALUE)
		return 0;
	return NextFrameLoop(pdc);
}

__declspec(dllexport) int __stdcall libMpeg2_SkipFrames(const char *keyName,int count)
{
	DecoderContainer *pdc=FindDecoder(keyName);
	if (pdc==0)
		return 0;

	if (pdc->filehandle==0 || pdc->filehandle==INVALID_HANDLE_VALUE)
		return 0;
	if (count<0)
		return 0;
	while (count>0)
	{
		if (NextFrameLoop(pdc)==0)
			return 0;
		--count;
	}
	return 1;
}

__declspec(dllexport) int __stdcall libMpeg2_Seek(const char *keyName,__int64 sequenz_pos,int full_reset)
{
	DecoderContainer *pdc=FindDecoder(keyName);
	if (pdc==0)
		return 0;

	LARGE_INTEGER new_pos;
	new_pos.QuadPart=sequenz_pos;
	new_pos.LowPart=SetFilePointer(pdc->filehandle,new_pos.LowPart,&new_pos.HighPart,FILE_BEGIN);
	if (new_pos.LowPart==INVALID_SET_FILE_POINTER && GetLastError()!= NO_ERROR)
		return 0;

	/* Seit Version 0.4.0
	   full_reset=0: lib starts decoding at the next picture
	   full_reset=1: lib starts looking for the next sequence header.
	   1 ist fr Mpeg2 hier zwingend notwendig, da beim Positionieren ein anderer Sequence-Header
	   gelesen werden muss, sonst wrde er u.U. als repeated Sequence gewertet und somit verworfen werden.
	*/
	mpeg2_reset(pdc->mpeg2dec,full_reset);
	pdc->stream_end_read=0;

	if (NextFrameLoop(pdc)==0)
		return 2;
	return 1;
}

static int NextFrameLoop(DecoderContainer *pdc)
{
	int size;
	int state;

	size = BUFFER_SIZE;
    do
	{
		state = mpeg2_parse (pdc->mpeg2dec);
		switch (state)
		{
			case STATE_BUFFER:
				ReadFile(pdc->filehandle,pdc->buffer,BUFFER_SIZE,(LPDWORD)&size,NULL);
				if (size!=0)
					pdc->stream_end_read=0;
				else
				{
					// Enthlt der Stream keinen SequenceEndCode, so schieben wir diesen
					// manuell nach um die letzten Bilder zu erhalten
					if (pdc->stream_end_read==0)
					{
						size=4;
						pdc->buffer[0]=pdc->buffer[1]=0x00;
						pdc->buffer[2]=0x01;
						pdc->buffer[3]=0xb7;
					}
				}
				mpeg2_buffer (pdc->mpeg2dec, pdc->buffer, pdc->buffer + size);
				break;
			case STATE_SEQUENCE:
				switch (pdc->conv_type)
				{
					case RGB24:
						mpeg2_convert (pdc->mpeg2dec, mpeg2convert_bgr24, NULL);
						break;
					case RGB32:
						mpeg2_convert (pdc->mpeg2dec, mpeg2convert_bgr32, NULL);
						break;
				}
				break;
			case STATE_END:
				pdc->stream_end_read=1;
			case STATE_SLICE:
			case STATE_INVALID_END:
				if (pdc->info->display_fbuf)
				{
					pdc->p_frameInfo=&pdc->frameinfo;
					pdc->frameinfo.Y=pdc->info->display_fbuf->buf[0];
					pdc->frameinfo.U=pdc->info->display_fbuf->buf[1];
					pdc->frameinfo.V=pdc->info->display_fbuf->buf[2];
					pdc->frameinfo.width=pdc->info->sequence->width;
					pdc->frameinfo.height=pdc->info->sequence->height;
					pdc->frameinfo.type=pdc->info->display_picture->flags&0x03;
					pdc->frameinfo.chroma_width=pdc->info->sequence->chroma_width;
					pdc->frameinfo.chroma_height=pdc->info->sequence->chroma_height;
					switch (pdc->conv_type)
					{
						case RGB24:
							pdc->frameinfo.size=pdc->frameinfo.width*pdc->frameinfo.height*3;
							pdc->frameinfo.chroma_size=0;
							break;
						case RGB32:
							pdc->frameinfo.size=pdc->frameinfo.width*pdc->frameinfo.height*4;
							pdc->frameinfo.chroma_size=0;
							break;
						case YV12:
							pdc->frameinfo.size=pdc->frameinfo.width*pdc->frameinfo.height;
							pdc->frameinfo.chroma_size=pdc->frameinfo.chroma_width*pdc->frameinfo.chroma_height;
					}
					return 1;
				}
				break;
		}
    } while (size);
	pdc->p_frameInfo=0;
	return 0;
}

__declspec(dllexport) int __stdcall libMpeg2_GetCurrentFrameData(const char *keyName,void *data)
{
	DecoderContainer *pdc=FindDecoder(keyName);
	if (pdc==0)
		return 0;

	char *adr=(char *)data;
	if (pdc->filehandle==0 || pdc->filehandle==INVALID_HANDLE_VALUE)
		return 0;

	switch (pdc->conv_type)
	{
	case RGB24:
	case RGB32:
		memcpy(adr,pdc->frameinfo.Y,pdc->frameinfo.size);
		break;
	case YV12:
		memcpy(adr,pdc->frameinfo.Y,pdc->frameinfo.size);
		adr+=pdc->frameinfo.size;
		memcpy(adr,pdc->frameinfo.V,pdc->frameinfo.chroma_size);
		adr+=pdc->frameinfo.chroma_size;
		memcpy(adr,pdc->frameinfo.U,pdc->frameinfo.chroma_size);
		break;
	}
	return 1;
}
